/** 
 *******************************************************************************
 * @file    main.c
 * @brief   ADC Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-06-25 17:51:37 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */
#include "uart_io.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup ADC ADC Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Private_macro ADC Private Macro
 *  @{
 */
/** 
 *  @defgroup CountConfiguration Time Configuration
 *  @brief    Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_OUTPUT_COUNT_TIME             ((uint32_t)(500))
/** 
 *  @}
 */ /* End of group CountConfiguration */
/** 
 *  @defgroup DataLengtheConfiguration Data Length Configuration
 *  @brief    Data Length Configuration.
 *  @{
 */
#define CFG_DATA_LENGTH            ((uint32_t)20)
/** 
 *  @}
 */ /* End of group DataLengtheConfiguration */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

/** 
 *  @defgroup ADC_MOD0 DAC Control
 *  @brief    DAC Control.
 *  @{
 */
#define ADC_MOD0_DACON                  ((uint32_t)0x00000001)    /*!< DAC Control ON.  */
/**
 *  @}
 */ /* End of group ADC_MOD0 */

/** 
 *  @defgroup ADC_CLK Clock Setting
 *  @brief    Clock Setting.
 *  @{
 */
#define ADC_CLK_160MHz                  ((uint32_t)0x00000008)    /*!< convert time:  ADCLK/1  1.0[us] (SCLK=160MHz).  */
#define ADC_CLK_80MHz                   ((uint32_t)0x00000001)    /*!< convert time:  ADCLK/2  1.1[us] (SCLK=80MHz).   */
/**
 *  @}
 */ /* End of group ADC_CLK */
 
/** 
 *  @defgroup ADC_MOD1 MODE Setting
 *  @brief    MODE Setting.
 *  @{
 */
#define ADC_MOD1_160MHz                 ((uint32_t)0x00306122)    /*!< convert time:  1.0[us] (SCLK=160MHz).  */
#define ADC_MOD1_80MHz                  ((uint32_t)0x00104011)    /*!< convert time:  1.1[us] (SCLK=80MHz).   */
/**
 *  @}
 */ /* End of group ADC_MOD1 */
/** 
 *  @defgroup ADC_MOD2 MODE Setting
 *  @brief    MODE Setting.
 *  @{
 */
#define ADC_MOD2_TMPM4Ky                 ((uint32_t)0x00000000)    /*!< TMPM4Ky Series.  */
/**
 *  @}
 */ /* End of group ADC_MOD2 */

/** 
 *  @defgroup ADC_CR1 DMA Request Init
 *  @brief    DMA Request Init.
 *  @{
 */
#define ADC_CR1_INIT                     ((uint32_t)0x00000000)    /*!< Not DMA Request.  */
/**
 *  @}
 */ /* End of group ADC_CR1 */
 
/** 
 *  @defgroup ADC_CR0 ADC Convert Start
 *  @brief    ADC Convert Start.
 *  @{
 */
#define ADC_CR0_ADEN_ENABLE              ((uint32_t)0x00000080)    /*!< ADC Enable.                      */
#define ADC_CR0_SGL_START                ((uint32_t)0x00000082)    /*!< ADC Single Convert Start.        */
#define ADC_CR0_CNT_START                ((uint32_t)0x00000081)    /*!< ADC Continuation Convert Start.  */
/**
 *  @}
 */ /* End of group ADC_CR0 */
 
/** 
 *  @defgroup ADC_ADxREGx ADC Convert Result Store Register
 *  @brief    ADC Convert Result Store Register.
 *  @{
 */
#define ADC_ADRF0_EXIST                  ((uint32_t)0x00000001)    /*!< ADC Convert Result Exist.        */
#define ADC_ADR0_MASK                    ((uint32_t)0x0000FFF0)    /*!< ADC Convert Result Exist.        */
/**
 *  @}
 */ /* End of group ADC_ADxREGx */
 
/** 
 *  @}
 */ /* End of group ADC_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Private_variables ADC Private Variables
 *  @{
 */
static uint32_t output_flag    = 0;           /*!< Output flag.        */
static uint32_t data0 = 0;
static uint32_t data1 = 0;
static uint8_t adc_data0[4] = {'\0'};
static uint8_t adc_data1[4] = {'\0'};
static uint32_t counter  = CFG_OUTPUT_COUNT_TIME;    /* Blink Time Counter        */
/**
 *  @}
 */ /* End of group ADC_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Private_fuctions ADC Private Fuctions
 *  @{
 */
static void GPIO_ADC_Config(void);
static void ADC_Config(void);
static void ADC_Start(void);
static void wait(int32_t count_p);

/*--------------------------------------------------*/
/** 
  * @brief  GPIO_ADC_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_ADC_Config(void)
{
  /* (1) Enable the peripheral clock of PORT M */
  TSB_CG_FSYSMENA_IPMENA11 = FSYS_ENABLE;
  /* (2) Select not output mode on PORT M pin 6(AINA01)/7(AINA00) */
  TSB_PM_CR_PM6C = GPIO_DISABLE;
  TSB_PM_CR_PM7C = GPIO_DISABLE;
  /* (3) Select not input mode on  PORT M pin 6(AINA01)/7(AINA00) */
  TSB_PM_IE_PM6IE = GPIO_DISABLE;
  TSB_PM_IE_PM7IE = GPIO_DISABLE;
  /* (4) Select pullup disable on PORT M pin 6(AINA01)/7(AINA00) */
  TSB_PM_PUP_PM6UP = GPIO_DISABLE;
  TSB_PM_PUP_PM7UP = GPIO_DISABLE;
  /* (5) Select pulldown disable on PORT M pin 6(AINA01)/7(AINA00) */
  TSB_PM_PDN_PM6DN = GPIO_DISABLE;
  TSB_PM_PDN_PM7DN = GPIO_DISABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void ADC_Config(void)
{
  /* (1) Enable the peripheral clock of ADC UnitA */
  TSB_CG_FSYSMENB_IPMENB02 = FSYS_ENABLE;
  TSB_CG_SPCLKEN_ADCKEN0 = FSYS_ENABLE; 
  
  /* Configure ADC unit A  */
  /* (1) Mode Register 0 Setting,   RCUT=0,DACON=1 */
  /* (2) Clock Register Setting, VADCLK[2:0]=000, EXAZ[3:0]=0001
         ADCLK/1, 1.0[us] (SCLK=160MHz)                          */
  /* (3) Mode Register 1 Setting,   MOD1[31:0]=0x00306122
         1.0us (SCLK=160MHz)                                     */
  /* (4) Mode Register 2 Setting,   MOD2[31:0]=0x00000000        */
  TSB_ADA->MOD0 = ADC_MOD0_DACON;     /* (1) */
  wait(1000);
  TSB_ADA->CLK  = ADC_CLK_160MHz;     /* (2) */
  TSB_ADA->MOD1 = ADC_MOD1_160MHz;    /* (3) */
  TSB_ADA->MOD2 = ADC_MOD2_TMPM4Ky;   /* (4) */
  
  /* Convert Program Setting */
  /* (1) ADxTSET0 Setting,   ENINT0=0,TRGS0[1:0]=01,AINST0[4:0]=00000:AINx00  */
  /* (2) ADxTSET1 Setting,   ENINT0=1,TRGS0[1:0]=01,AINST0[4:0]=00001:AINx01  */
  TSB_ADA->TSET0 = 0x00000020;
  TSB_ADA->TSET1 = 0x000000A1;
  NVIC_SetPriority (INTADACNT_IRQn, 10);
  NVIC_EnableIRQ(INTADACNT_IRQn); /* ADC unit A Continuity program interrupt enable */
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void ADC_Start(void)
{
  /* [ADxCR1] =0x00000000 */
  TSB_ADA->CR1 = ADC_CR1_INIT;
  /* [ADxCR0] =0x00000081 */
  TSB_ADA->CR0 = ADC_CR0_CNT_START;
}
/*--------------------------------------------------*/
/** 
  * @brief  Wait
  * @param  count_p
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void wait(int32_t count_p)
{
    int32_t i;
    for(i=0; i<count_p; i++){
        __NOP();
    }
}
/**
 *  @}
 */ /* End of group ADC_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Exported_functions ADC Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  uint32_t period;    /* Period for Systick */
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_ADC_Config();
  /*------------------------*/
  /* UART Setting            */
  /*------------------------*/
  uart_io_initialize();
  /*------------------------*/
  /* ADC Setting            */
  /*------------------------*/
  ADC_Config();
  /* Period for 10msec SYSTICK */
  SystemCoreClockUpdate();
  period = SystemCoreClock / 100;
  if (SysTick_Config(period ))    /* systick interrupt cycle setting */
  {
  }
  /* Interrupt Enable */
  NVIC_SetPriority (SysTick_IRQn, 2);
  /* Start ADC Convert */
  ADC_Start();
  /*------------------------*/
  /*          Loop                         */
  /*------------------------*/ 
  while(1){  
    if(output_flag == 1)
    {
		printf("AIN0 > 0x%04x\r\n", data0);
		printf("AIN1 > 0x%04x\r\n", data1);
        /* output flag clear */
        output_flag = 0;
        NVIC_EnableIRQ(INTADACNT_IRQn); /* ADC unit A Continuity program interrupt enable */
    }
  } 
}
/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
    if (counter-- == 0) 
    {
      output_flag = 1;
      counter = CFG_OUTPUT_COUNT_TIME;
      NVIC_DisableIRQ(INTADACNT_IRQn); /* ADC unit A Continuity program interrupt disable */
    }else{
        /* no processing */
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  INTADASGL IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   45: ADC unit A Continuity program interrupt
  */
/*--------------------------------------------------*/
void INTADACNT_IRQHandler(void)
{
   /* ADxREG0 [AIN0] AD Result Value Get */
   if((TSB_ADA->REG0 & ADC_ADRF0_EXIST) == ADC_ADRF0_EXIST)
   {
      data0 =( (TSB_ADA->REG0 & ADC_ADR0_MASK) >> 4);
      sprintf((char *)adc_data0, "%03x", data0);
   }
   /* ADxREG1 [AIN1] AD Result Value Get */
   if((TSB_ADA->REG1 & ADC_ADRF0_EXIST) == ADC_ADRF0_EXIST)
   {
      data1 =( (TSB_ADA->REG1 & ADC_ADR0_MASK) >> 4);
      sprintf((char *)adc_data1, "%03x", data1);
   }
}

/**
 *  @}
 */ /* End of group ADC_Exported_functions */

/**
 *  @}
 */ /* End of group ADC */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
