/** 
 *******************************************************************************
 * @file    main.c
 * @brief   A_ENC32 Sample Application.
 * @version V1.0.0.1
 * $Date:: 2019-05-10 15:07:39 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */
#include <stdlib.h>

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup A_ENC32 A_ENC32 Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup A_ENC32_Private_macro A_ENC32 Private Macro
 *  @{
 */

/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */
/** 
 *  @name  ENxTNCR Macro Definition.
 *  @brief ENxTNCR Register Macro Definition.
 *  @{
 */
#define ENxTNCR_INIT       ((uint32_t)0x00000000)     /*!< CMPSEL=0,UDMD[1:0]=00,TOVMD=0,MCMPMD=0,DECMD[1:0]=00,SDTEN=0,MODE[2:0]=000,P3EN=0,TRGCAPMD=0,SFTCAP=0,ENCLR=0,
                                                           ZESEL[1:0]=00,ZEN=0,ENRUN=0,ZEACT=0;ENDEV[2:0]=0                                                                */
/**
 *  @}
 */ /* End of name ENxTNCR Macro Definition */

/** 
 *  @name  ENxINPCR Macro Definition.
 *  @brief ENxINPCR Register Macro Definition.
 *  @{
 */
#define ENxINPCR_INIT       ((uint32_t)0x00001F00)     /*!< NCT[6:0]=0011111,PDSTP=0,PDSTT=0,SYNCNCZEN=0,SYNCSPLMD=0,SYNCSPLEN=0 */
/**
 *  @}
 */ /* End of name ENxINPCR Macro Definition */

/** 
 *  @name  ENxINTCR Macro Definition.
 *  @brief ENxINTCR Register Macro Definition.
 *  @{
 */
#define ENxINTCR_INIT       ((uint32_t)0x00000000)     /*!< MCMPIE=0,RLDIE=0,CMPIE=0,ERRIE=0,CAPIE=0,TPLSIE=0  */
/**
 *  @}
 */ /* End of name ENxINTCR Macro Definition */
/** 
 *  @name  ENxCLKCR Macro Definition.
 *  @brief ENxCLKCR Register Macro Definition.
 *  @{
 */
#define ENxCLKCR_FSYS       ((uint32_t)0x00000000)     /*!< sampling clock:fsys    */
#define ENxCLKCR_FSYS_DIV2  ((uint32_t)0x00000001)     /*!< sampling clock:fsys/2  */
#define ENxCLKCR_FSYS_DIV4  ((uint32_t)0x00000002)     /*!< sampling clock:fsys/4  */
#define ENxCLKCR_FSYS_DIV8  ((uint32_t)0x00000003)     /*!< sampling clock:fsys/8  */
/**
 *  @}
 */ /* End of name ENxCLKCR Macro Definition */
 
/** 
 *  @name  ENRUN Macro Definition.
 *  @brief ENRUN Enable/Disable Macro Definition.
 *  @{
 */
#define ENRUN_DISABLE       ((uint32_t)0x00000000)     /*!< Enc input circuit disable */
#define ENRUN_ENABLE        ((uint32_t)0x00000040)     /*!< Enc input circuit enable  */
/**
 *  @}
 */ /* End of name ENRUN Macro Definition */
 
/** 
 *  @name  ENxSTS_UD Macro Definition.
 *  @brief ENxSTS UD Macro Definition.
 *  @{
 */
#define ENxSTS_UD_CCW       ((uint32_t)0x00000000)     /*!< CounterClockWise direction */
#define ENxSTS_UD_CC        ((uint32_t)0x00002000)     /*!< ClockWise direction        */
/**
 *  @}
 */ /* End of name ENxSTS_UD Macro Definition */
 
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

/** 
 *  @}
 */ /* End of group A_ENC32_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup A_ENC32_Private_variables A_ENC32 Private Variables
 *  @{
 */
static uint32_t enccount = 0;                   /* ENC Counter         */
static uint32_t enccount_old = 0;               /* ENC Counter old     */
static uint32_t speed = 0;                      /* speed               */
static uint32_t sec_count = 0;                  /* 1sec timer          */
static int32_t cw_flag = 0;
/**
 *  @}
 */ /* End of group A_ENC32_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup A_ENC32_Private_fuctions A_ENC32 Private Fuctions
 *  @{
 */
static void GPIO_Config(void);
static void GPIO_ENC_Configure(void);
static void ENC_Config(void);
static void ENC_Start(void);
static void Delay1(void);
static void Delay2(uint32_t time);
/*--------------------------------------------------*/
/** 
  * @brief  Delay1
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void Delay1(void)
{
    uint32_t TimeCounter;
    uint32_t DelayTime = 500;
    for (TimeCounter = 0U; TimeCounter < DelayTime; TimeCounter++) {
        /* Do nothing */
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Delay2
  * @param  time
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void Delay2(uint32_t time)
{
    uint32_t count;
    for (count = 0U; count < time; count++){
        Delay1();
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 1 (LED1) */
  TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 1 (LED1) */
  TSB_PJ_CR_PJ1C = GPIO_ENABLE;
  /* (6) Turn off PORT J pin 2 (LED2) */
  TSB_PJ_DATA_PJ2 = GPIO_DISABLE;
  /* (7) Select output mode on PORT J pin 2 (LED2) */
  TSB_PJ_CR_PJ2C = GPIO_ENABLE;
  /* (8) Turn off PORT J pin 3 (LED3) */
  TSB_PJ_DATA_PJ3 = GPIO_DISABLE;
  /* (9) Select output mode on PORT J pin 3 (LED3) */
  TSB_PJ_CR_PJ3C = GPIO_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  GPIO_ENC_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_ENC_Configure(void)
{
  /* (1) Enable the peripheral clock of GPIO R */
  TSB_CG_FSYSMENA_IPMENA14 = FSYS_ENABLE;
  /* (2) Select function mode (ENC1A) on PORT R pin 3    */
  TSB_PR_FR5_PR3F5 = GPIO_ENABLE;
  /* (3) Select input mode on PORT R pin 3    */
  TSB_PR_IE_PR3IE  = GPIO_ENABLE;
  /* (4) Select function mode (ENC1B) on PORT R pin 4    */
  TSB_PR_FR5_PR4F5 = GPIO_ENABLE;
  /* (5) Select input mode on PORT R pin 4    */
  TSB_PR_IE_PR4IE  = GPIO_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  ENC_Config
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void ENC_Config(void)
{
  /* (1) Enable the peripheral clock of A-ENC32 ch1 */
  /* (2) Setting ENC Control, decoder mode
         CMPSEL=0,UDMD[1:0]=00,TOVMD=0,MCMPMD=0,DECMD[1:0]=00,SDTEN=0,MODE[2:0]=000,P3EN=0,
         TRGCAPMD=0,SFTCAP=0,ENCLR=0,ZESEL[1:0]=00,ZEN=0,ENRUN=0,ZEACT=0;ENDEV[2:0]=0        */
  /* (3) Setting Noise Cancell, Filters out pulses narrower than 31/fsys as noise
         NCT[6:0]=0011111,PDSTP=0,PDSTT=0,SYNCNCZEN=0,SYNCSPLMD=0,SYNCSPLEN=0    */
  /* (4) Interrupt Enable Contorl
         MCMPIE=0,RLDIE=0,CMPIE=0,ERRIE=0,CAPIE=0,TPLSIE=0       */
  /* (5) Setting sample clock fsys                               */
  /* (6) Setting Reload Register                                 */
  TSB_CG_FSYSMENB_IPMENB07 = FSYS_ENABLE;    /* (1) */
  TSB_EN1->TNCR   = ENxTNCR_INIT;            /* (2) */
  TSB_EN1->INPCR  = ENxINPCR_INIT;           /* (3) */
  TSB_EN1->INTCR  = ENxINTCR_INIT;           /* (4) */
  TSB_EN1->CLKCR  = ENxCLKCR_FSYS;           /* (5) */
  TSB_EN1->RELOAD = 0xFFFFFFFF;              /* (6) */
  TSB_EN1->INT    = 0x00000000;              /* (7) */
}
/*--------------------------------------------------*/
/** 
  * @brief  ENC_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void ENC_Start(void)
{
    uint32_t tmp = 0;
    tmp = TSB_EN1->TNCR;
    TSB_EN1->TNCR = (tmp | ENRUN_ENABLE);
}
    
/**
 *  @}
 */ /* End of group A_ENC32_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup A_ENC32_Exported_functions A_ENC32 Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  uint32_t period;
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_Config();
  GPIO_ENC_Configure();
  /* Period for 10msec SYSTICK */
  SystemCoreClockUpdate();
  period = SystemCoreClock / 100;
  if (SysTick_Config(period ))    /* systick interrupt cycle setting */
  {
  }
  /*------------------------*/
  /* T32A CountUp Setting   */
  /*------------------------*/ 
  ENC_Config();
  /* Timer Start */
  ENC_Start();
  /*------------------------*/
  /* Loop */
  /*------------------------*/
    /* LED0 blink speed based on the rolling of mouse wheel */
    while(1)
    {
        if(speed > 0)
        {
            if(cw_flag == 1)
            {
                TSB_PJ_DATA_PJ0 = 1;
                Delay2(speed);
                TSB_PJ_DATA_PJ0 = 0;
                Delay2(50);
            }
            else if(cw_flag == 2)
            {
                TSB_PJ_DATA_PJ1 = 1;
                Delay2(speed);
                TSB_PJ_DATA_PJ1 = 0;
                Delay2(50);
            }
        }
        else
        {
            TSB_PJ_DATA_PJ0 = 0;
            TSB_PJ_DATA_PJ1 = 0;
        }
    } 
}
/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
    uint32_t status;
    
    status = TSB_EN1->STS;
    if((status & 0x2000) == 0x2000)
    {
        cw_flag = 1;
        TSB_PJ_DATA_PJ1 = 0;
    }
    else
    {
        cw_flag = 2;
        TSB_PJ_DATA_PJ0 = 0;
    }
    if(++sec_count >= 100){
        sec_count = 0;
        enccount = TSB_EN1->CNT;
        speed = abs((int)(enccount - enccount_old));
        enccount_old = enccount;
    }
}

/**
 *  @}
 */ /* End of group A_ENC32_Exported_functions */

/**
 *  @}
 */ /* End of group A_ENC32 */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
