/** 
 *******************************************************************************
 * @file    main.c
 * @brief   CAN Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-06-25 15:43:28 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */
#include "uart_io.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup CAN CAN Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CAN_Private_macro CAN Private Macro
 *  @{
 */
/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_OUTPUT_COUNT_TIME             ((uint32_t)(500))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */
/** 
 *  @defgroup DataLengtheConfiguration Data Length Configuration
 *  @brief    Data Length Configuration.
 *  @{
 */
#define CFG_DATA_LENGTH            ((uint32_t)20)
/** 
 *  @}
 */ /* End of group DataLengtheConfiguration */
/** 
 *  @defgroup MSG_ID_CFG Message ID  Configuration
 *  @brief    Message ID  Configuration.
 *  @{
 */
#define MSG_ID                      (0x00000014U)
#define BAUDRATE_PRESCALER          (0x000003FFU)
#define MSG_BOX_TX_BITNO            (0x00000001U)
#define MSG_BOX_RX_BITNO            (0x00000400U)
/** 
 *  @}
 */ /* End of group MSG_ID_CFG */

 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

/** 
 *  @defgroup ADC_MOD0 DAC Control
 *  @brief    DAC Control.
 *  @{
 */
#define ADC_MOD0_DACON                  ((uint32_t)0x00000001)    /*!< DAC Control ON.  */
/**
 *  @}
 */ /* End of group ADC_MOD0 */

/** 
 *  @defgroup ADC_CLK Clock Setting
 *  @brief    Clock Setting.
 *  @{
 */
#define ADC_CLK_160MHz                  ((uint32_t)0x00000008)    /*!< convert time:  ADCLK/1  1.0[us] (SCLK=160MHz).  */
#define ADC_CLK_80MHz                   ((uint32_t)0x00000001)    /*!< convert time:  ADCLK/2  1.1[us] (SCLK=80MHz).   */
/**
 *  @}
 */ /* End of group ADC_CLK */
 
/** 
 *  @defgroup ADC_MOD1 MODE Setting
 *  @brief    MODE Setting.
 *  @{
 */
#define ADC_MOD1_160MHz                 ((uint32_t)0x00306122)    /*!< convert time:  1.0[us] (SCLK=160MHz).  */
#define ADC_MOD1_80MHz                  ((uint32_t)0x00104011)    /*!< convert time:  1.1[us] (SCLK=80MHz).   */
/**
 *  @}
 */ /* End of group ADC_MOD1 */
/** 
 *  @defgroup ADC_MOD2 MODE Setting
 *  @brief    MODE Setting.
 *  @{
 */
#define ADC_MOD2_TMPM4Ky                 ((uint32_t)0x00000000)    /*!< TMPM4Ky Series.  */
/**
 *  @}
 */ /* End of group ADC_MOD2 */

/** 
 *  @defgroup ADC_CR1 DMA Request Init
 *  @brief    DMA Request Init.
 *  @{
 */
#define ADC_CR1_INIT                     ((uint32_t)0x00000000)    /*!< Not DMA Request.  */
/**
 *  @}
 */ /* End of group ADC_CR1 */
 
/** 
 *  @defgroup ADC_CR0 ADC Convert Start
 *  @brief    ADC Convert Start.
 *  @{
 */
#define ADC_CR0_ADEN_ENABLE              ((uint32_t)0x00000080)    /*!< ADC Enable.                      */
#define ADC_CR0_SGL_START                ((uint32_t)0x00000082)    /*!< ADC Single Convert Start.        */
#define ADC_CR0_CNT_START                ((uint32_t)0x00000081)    /*!< ADC Continuation Convert Start.  */
/**
 *  @}
 */ /* End of group ADC_CR0 */
 
/** 
 *  @defgroup ADC_ADxREGx ADC Convert Result Store Register
 *  @brief    ADC Convert Result Store Register.
 *  @{
 */
#define ADC_ADRF0_EXIST                  ((uint32_t)0x00000001)    /*!< ADC Convert Result Exist.        */
#define ADC_ADR0_MASK                    ((uint32_t)0x0000FFF0)    /*!< ADC Convert Result Exist.        */
/**
 *  @}
 */ /* End of group ADC_ADxREGx */
 
/** 
 *  @}
 */ /* End of group CAN_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CAN_Private_variables CAN Private Variables
 *  @{
 */
static uint32_t output_flag    = 0;           /*!< Output flag.        */
static uint32_t data0 = 0;
static uint32_t data1 = 0;
static uint32_t rev0 = 0;
static uint32_t rev1 = 0;
static uint8_t rev_data0[4] = {'\0'};
static uint8_t rev_data1[4] = {'\0'};
static uint32_t counter  = CFG_OUTPUT_COUNT_TIME;    /* Blink Time Counter        */
static uint8_t fIntADC = 0U;
/**
 *  @}
 */ /* End of group CAN_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CAN_Private_fuctions CAN Private Fuctions
 *  @{
 */
static void GPIO_ADC_Config(void);
static void ADC_Config(void);
static void ADC_Start(void);
static void GPIO_CAN_Config(void);
static void CAN_BitConfig(void);
static void CAN_RX_Config(void);
static void CAN_TX_Config(void);
static void wait(int32_t count_p);


/*--------------------------------------------------*/
/** 
  * @brief  GPIO_ADC_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_ADC_Config(void)
{
  /* (1) Enable the peripheral clock of PORT M */
  TSB_CG_FSYSMENA_IPMENA11 = FSYS_ENABLE;
  /* (2) Select not output mode on PORT M pin 6(AINA01)/7(AINA00) */
  TSB_PM_CR_PM6C = GPIO_DISABLE;
  TSB_PM_CR_PM7C = GPIO_DISABLE;
  /* (3) Select not input mode on  PORT M pin 6(AINA01)/7(AINA00) */
  TSB_PM_IE_PM6IE = GPIO_DISABLE;
  TSB_PM_IE_PM7IE = GPIO_DISABLE;
  /* (4) Select pullup disable on PORT M pin 6(AINA01)/7(AINA00) */
  TSB_PM_PUP_PM6UP = GPIO_DISABLE;
  TSB_PM_PUP_PM7UP = GPIO_DISABLE;
  /* (5) Select pulldown disable on PORT M pin 6(AINA01)/7(AINA00) */
  TSB_PM_PDN_PM6DN = GPIO_DISABLE;
  TSB_PM_PDN_PM7DN = GPIO_DISABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void ADC_Config(void)
{
  /* (1) Enable the peripheral clock of ADC UnitA */
  TSB_CG_FSYSMENB_IPMENB02 = FSYS_ENABLE;
  TSB_CG_SPCLKEN_ADCKEN0 = FSYS_ENABLE; 
  
  /* Configure ADC unit A  */
  /* (1) Mode Register 0 Setting,   RCUT=0,DACON=1 */
  /* (2) Clock Register Setting, VADCLK[2:0]=000, EXAZ[3:0]=0001
         ADCLK/1, 1.0[us] (SCLK=160MHz)                          */
  /* (3) Mode Register 1 Setting,   MOD1[31:0]=0x00306122
         1.0us (SCLK=160MHz)                                     */
  /* (4) Mode Register 2 Setting,   MOD2[31:0]=0x00000300        */
  TSB_ADA->MOD0 = ADC_MOD0_DACON;     /* (1) */
  wait(1000);
  TSB_ADA->CLK  = ADC_CLK_160MHz;     /* (2) */
  TSB_ADA->MOD1 = ADC_MOD1_160MHz;    /* (3) */
  TSB_ADA->MOD2 = ADC_MOD2_TMPM4Ky;   /* (4) */
  
  /* Convert Program Setting */
  /* (1) ADxTSET0 Setting,   ENINT0=0,TRGS0[1:0]=01,AINST0[4:0]=00000:AINx00  */
  /* (2) ADxTSET1 Setting,   ENINT0=1,TRGS0[1:0]=01,AINST0[4:0]=00001:AINx01  */
  TSB_ADA->TSET0 = 0x00000020;
  TSB_ADA->TSET1 = 0x000000A1;
  NVIC_SetPriority (INTADACNT_IRQn, 10);
  NVIC_EnableIRQ(INTADACNT_IRQn); /* ADC unit A Continuity program interrupt enable */
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void ADC_Start(void)
{
  /* [ADxCR1] =0x00000000 */
  TSB_ADA->CR1 = ADC_CR1_INIT;
  /* [ADxCR0] =0x00000081 */
  TSB_ADA->CR0 = ADC_CR0_CNT_START;
}

/*--------------------------------------------------*/
/** 
  * @brief  CAN_CAN_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_CAN_Config(void)
{
  /* (1) Enable the peripheral clock of PORT E */
  TSB_CG_FSYSMENA_IPMENA04 = FSYS_ENABLE;
  /* (2) Select function mode on PORT E pin 0    */
  TSB_PE_FR1_PE0F1 = GPIO_ENABLE;
  /* (3) Select output mode on PORT E pin 0    */
  TSB_PE_CR_PE0C = GPIO_ENABLE;
  TSB_PE_CR_PE1C = GPIO_DISABLE;
  /* (4) Select function mode on PORT E pin 1    */
  TSB_PE_FR1_PE1F1 = GPIO_ENABLE;
  /* (5) Select input mode on PORT E pin 1    */
  TSB_PE_IE_PE0IE = GPIO_DISABLE;
  TSB_PE_IE_PE1IE = GPIO_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  CAN_BitConfigure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void CAN_BitConfig(void)
{
  /* Enable the peripheral clock of CAN  */
  TSB_CG_FSYSMENA_IPMENA27 = FSYS_ENABLE;
  
  /* Can SW Reset [CANMCR]<SRES> to '1'       */
  TSB_CAN->MCR = 0x00000081;
  
  wait(500);
  /* Setting Prescaler [CANBCR1]<BRP[9:0]> to '1'  */
  TSB_CAN->BCR1 = 0x00000001;
  /* Setting Prescaler [CANBCR2]<SJW[1:0]> to '11:4*TQ',<SAM> to '0',<TSEG2[2:0]> to '111:8*TQ', <TSEG1[3:0]> to '1010:11*TQ'  */
  TSB_CAN->BCR2 = 0x0000037A;
  /* Test loop back mode, set :CANMCR<TSTLB> to '1' */  
  TSB_CAN->MCR = 0x00000280;
  /* Setting Normal operation mode, set :CANMCR<CCR> to '0' */    
  TSB_CAN->MCR = 0x00000200;
  /* Checking Normal operation mode, set :CANGSR<CCE> to '0' */
  while((TSB_CAN->GSR & 0x00000080) != 0)
  {
  }
}

/*--------------------------------------------------*/
/** 
  * @brief  CAN_RX_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void CAN_RX_Config(void)
{
  uint32_t tmp = 0;
  /* Configure mailbox No.10 for reception: set [CANMC]<MDn> to '1'*/
  TSB_CAN->MD = MSG_BOX_RX_BITNO;
  /* Setup mailbox No.10 identifier and [CANMBnID]<IDE> bit for Standard ID or extended ID
      set <GAME_LAME> to '1', <GAME_LAME> to '1' */
  TSB_CANMB10->ID = (MSG_ID | 0x60000000);
  /* Setup mailbox No.10 set [CANMBnTSVMCF]<RTR> to '0' and <DLC[3:0]> to '1000' */
  TSB_CANMB10->TSVMCF = 0x00000008;
  /* Enable mailbox No.10 : set [CANMC]<MCn> to '1' */
  tmp = 0;
  tmp = TSB_CAN->MC;
  TSB_CAN->MC = (tmp | MSG_BOX_RX_BITNO);
  TSB_CANMB10->DL = 0x0U; /* Reset data mailbox10 */
  TSB_CANMB10->DH = 0x0U; /* Reset data mailbox10 */
}


/*--------------------------------------------------*/
/** 
  * @brief  CAN_TX_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void CAN_TX_Config(void)
{
  uint32_t tmp = 0;

  /*Disable mailbox: set [CANMC]<MCn> to '0'*/
  TSB_CAN->MC = 0x0;
  /* Configure mailbox No.0 for reception: set [CANMC]<MDn> to '0'*/
  tmp = TSB_CAN->MD;
  TSB_CAN->MD = (tmp & ~MSG_BOX_TX_BITNO);
  /* Setup mailbox No.0 identifier and [CANMBnID]<IDE> bit for Standard ID or extended ID
      set <RFH> to '1'                                            */
  TSB_CANMB0->ID = (MSG_ID | 0x20000000);
  /* Setup mailbox No.0 set [CANMBnTSVMCF]<RTR> to '0' and <DLC[3:0]> to '1000' */
  TSB_CANMB0->TSVMCF = 0x00000008;
  /* Enable mailbox No.0 : set [CANMC]<MCn> to '1'                */
  tmp = 0;
  tmp = TSB_CAN->MC;
  TSB_CAN->MC = (tmp | MSG_BOX_TX_BITNO);
}

/*--------------------------------------------------*/
/** 
  * @brief  Wait
  * @param  count_p
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void wait(int32_t count_p)
{
    int32_t i;
    for(i=0; i<count_p; i++){
        __NOP();
    }
}
/**
 *  @}
 */ /* End of group CAN_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CAN_Exported_functions CAN Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  uint32_t period;    /* Period for Systick */

  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_ADC_Config();
  GPIO_CAN_Config();
  /*------------------------*/
  /* UART Setting            */
  /*------------------------*/
  uart_io_initialize();
  /*------------------------*/
  /* ADC Setting            */
  /*------------------------*/
  ADC_Config();
  /*------------------------*/
  /* CAN Setting            */
  /*------------------------*/
  /* baudrate set */
  CAN_BitConfig();
  CAN_TX_Config();
  CAN_RX_Config();
  /* Period for 10msec SYSTICK */
  SystemCoreClockUpdate();
  period = SystemCoreClock / 100;
  if (SysTick_Config(period ))    /* systick interrupt cycle setting */
  {
  }
  /* Interrupt Enable */
  NVIC_SetPriority (SysTick_IRQn, 2);
  
  /* Start ADC Convert */
  ADC_Start();
  /*------------------------*/
  /*          Loop                         */
  /*------------------------*/ 
  while(1){
    if(fIntADC == 1U)
    {
        fIntADC = 0U;
        /* Copy data to CANMBxDH/CANMBxDL */
        TSB_CANMB0->DL = data0;
        TSB_CANMB0->DH = data1; /* Put the ADC result into mailbox0 */
        if((TSB_CAN->TRS & MSG_BOX_TX_BITNO)==0)
        {
            TSB_CAN->TRS = MSG_BOX_TX_BITNO;   /* mailnox No.0 Start to send */
        }else{
        }
        /* Polling TX, Wait TX Ack */
        while((TSB_CAN->TA & 0x00000001) != 0x00000001)
        {
        }
        /* CANRPM Check */
        while((TSB_CAN->RMP & MSG_BOX_RX_BITNO) != MSG_BOX_RX_BITNO)
        {
        }
        /* Read Receive data */
        rev0 = TSB_CANMB10->DL;
        rev1 = TSB_CANMB10->DH;
        /* Convert HEX to string */
        sprintf((char *)rev_data0, "%x", rev0);
        sprintf((char *)rev_data1, "%x", rev1);
    }
    
    /* ADC Result Display */
    if(output_flag == 1)
    {
		printf("AIN0 > 0x%04x\r\n", rev0);
		printf("AIN1 > 0x%04x\r\n", rev1);
        /* output flag clear */
        output_flag = 0;
        NVIC_EnableIRQ(INTADACNT_IRQn); /* ADC unit A Continuity program interrupt enable */
    }
  } 
}
/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
    if (counter-- == 0) 
    {
      output_flag = 1;
      counter = CFG_OUTPUT_COUNT_TIME;
      NVIC_DisableIRQ(INTADACNT_IRQn); /* ADC unit A Continuity program interrupt disable */
    }else{
        /* no processing */
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  INTADASGL IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   45: ADC unit A Continuity program interrupt
  */
/*--------------------------------------------------*/
void INTADACNT_IRQHandler(void)
{
   /* ADxREG0 [AIN0] AD Result Value Get */
   if((TSB_ADA->REG0 & ADC_ADRF0_EXIST) == ADC_ADRF0_EXIST)
   {
      data0 =( (TSB_ADA->REG0 & ADC_ADR0_MASK) >> 4);
   }
   /* ADxREG1 [AIN1] AD Result Value Get */
   if((TSB_ADA->REG1 & ADC_ADRF0_EXIST) == ADC_ADRF0_EXIST)
   {
      data1 =( (TSB_ADA->REG1 & ADC_ADR0_MASK) >> 4);
   }
   fIntADC = 1U;
}
/**
 *  @}
 */ /* End of group CAN_Exported_functions */

/**
 *  @}
 */ /* End of group CAN */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
