/** 
 *******************************************************************************
 * @file    main.c
 * @brief   CG_IDLE Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-05-10 15:55:32 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup CG_IDLE CG_IDLE Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_IDLE_Private_macro CG_IDLE Private Macro
 *  @{
 */

/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */

/** 
 *  @defgroup ProcModeConfiguration Processing Mode Configuration
 *  @brief    Processing Mode Normal Start.
 *  @{
 */
#define CFG_PROC_DEFAULT               ((uint32_t)(0))
#define CFG_PROC_NORMAL_START          ((uint32_t)(1))
/** 
 *  @}
 */ /* End of group ProcModeConfiguration */

/** 
 *  @defgroup  SIWDxEN  SIWDxEN Register
 *  @brief     WD Enable Register.
 *  @{
 */
/* WDTE */
#define SIWDxEN_WDTE_RW_DISABLE         ((uint32_t)0x00000000)    /*!< WDTE  :[R/W] :Disable    */
#define SIWDxEN_WDTE_RW_ENABLE          ((uint32_t)0x00000001)    /*!< WDTE  :[R/W] :Enable     */
/* WDCR */
#define SIWDxCR_WDCR_W_DISABLE          ((uint32_t)0x000000B1)    /*!< WDCR  :[W] :Disable      */
#define SIWDxCR_WDCR_W_CLEAR            ((uint32_t)0x0000004E)    /*!< WDCR  :[W] :Clear        */
/** 
 *  @}
 */ /* End of group SIWDxEN */
 /** 
 *  @defgroup  CGSTBYCR  CGSTBYCR Register
 *  @brief     Standby Control Register.
 *  @{
 */
/* STBY */
#define CGSTBYCR_STBY_RW_IDLE           ((uint32_t)0x00000000)    /*!< STBY :[R/W] :IDLE   */
#define CGSTBYCR_STBY_RW_STOP1          ((uint32_t)0x00000001)    /*!< STBY :[R/W] :STOP1  */
#define CGSTBYCR_STBY_RW_STOP2          ((uint32_t)0x00000002)    /*!< STBY :[R/W] :STOP2  */
/** 
 *  @}
 */ /* End of group CGSTBYCR */
 /** 
 *  @defgroup  FCSR0  FCSR0 Register
 *  @brief     Flash status Register 0.
 *  @{
 */
/*RDYBSY */
#define FCSR0_RDYBSY_R_BUSY             ((uint32_t)0x00000000)    /*!< WRDYBSY :[R] :Busy      */
#define FCSR0_RDYBSY_R_READY            ((uint32_t)0x00000001)    /*!< WRDYBSY :[R] :Ready     */
/** 
 *  @}
 */ /* End of group FCSR0 */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

/** 
 *  @}
 */ /* End of group CG_IDLE_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_IDLE_Private_variables CG_IDLE Private Variables
 *  @{
 */
static uint32_t proc     = CFG_PROC_DEFAULT;      /*!< Processing mode.        */
static uint32_t counter  = CFG_LED_BLINK_TIME;    /* Blink Time Counter        */
/**
 *  @}
 */ /* End of group CG_IDLE_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_IDLE_Private_fuctions CG_IDLE Private Fuctions
 *  @{
 */
static void GPIO_Config(void);
static void wdt_disable(void);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 1 (LED1) */
  TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 1 (LED1) */
  TSB_PJ_CR_PJ1C = GPIO_DISABLE;
  /* (6) Enable the peripheral clock of PORT A */
  TSB_CG_FSYSMENA_IPMENA00 = FSYS_ENABLE;
  /* (7) Select output mode on PORT A pin 0 (SWS) */
  TSB_PA_CR_PA0C = GPIO_ENABLE;
  TSB_PA_DATA_PA0 = GPIO_ENABLE;   /*SW select 1:SW7-4 */
  /* (8) Select input mode on PORT A pin 2 (SW4) */
  TSB_PA_IE_PA2IE = GPIO_ENABLE;
  /* (9) Select input mode on PORT A pin 1 (SW5) */
  TSB_PA_IE_PA1IE = GPIO_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  WDT Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void wdt_disable(void)
{
    TSB_SIWD0->EN = SIWDxEN_WDTE_RW_DISABLE;
    TSB_SIWD0->CR = SIWDxCR_WDCR_W_DISABLE;
}

/**
 *  @}
 */ /* End of group CG_IDLE_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_IDLE_Exported_functions CG_IDLE Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  uint32_t period;    /* Period for Systick */
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_Config();
  /*--- IBIMC033: Enable ---*/
  TSB_IB->IMC033 = (uint8_t)(0x05);
  
  /*------------------------*/
  /* Normal Mode Processing */
  /*------------------------*/
  while(1){
    /* Period for 10msec SYSTICK */
    SystemCoreClockUpdate();
    period = SystemCoreClock / 100;
    if (SysTick_Config(period ))    /* systick interrupt cycle setting */
    {
    }
  
    while(TSB_PA_DATA_PA1 == 0)
    {
        /* no processing */
    }
    
    /*------------------------*/
    /* IDLE Mode Processing   */
    /*------------------------*/
    /*----------------------*/
    /* WDT Disable          */
    /*----------------------*/
    wdt_disable();
    /*----------------------*/
    /* Wait FC status Ready */
    /*----------------------*/
    while(!(TSB_FC->SR0 && FCSR0_RDYBSY_R_READY)==FCSR0_RDYBSY_R_READY)
    {
    }
    /*----------------------*/
    /* Set STBY             */
    /*----------------------*/
    /* [CGSTBYCR]<STBY>    :IDLE            */
    TSB_CG->STBYCR = CGSTBYCR_STBY_RW_IDLE;
    /*----------------------*/
    /* Check STBY           */
    /*----------------------*/
    /* [CGSTBYCR]<STBY>    :IDLE            */
    while(!(TSB_CG->STBYCR == CGSTBYCR_STBY_RW_IDLE))
    {
    }
    /*>>>Wait For Interrupt */
    __WFI();
    /* LED status IDLE Set */
    TSB_PJ_CR_PJ0C = GPIO_DISABLE;        /* switch off led0 on PJ0 */
    TSB_PJ_CR_PJ1C = GPIO_ENABLE;         /* set led1 on PJ1          */
    TSB_PJ_DATA_PJ1 = 1;
    /* Interrupt Enable */
    NVIC_EnableIRQ(INT00_IRQn);     /* Interrupt pin 00 enable */
  
    while(proc != CFG_PROC_NORMAL_START)
    {
        /* no processing */
    }
  }
}

/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
    if (counter-- == 0) 
    {
      /* The end of the counting down period, the data is
         inverted and stored in a register PJ0           */
      TSB_PJ_DATA_PJ0 ^= 1;     /* led0 blink for 1sec period */
      counter = CFG_LED_BLINK_TIME;
    }else{
        /* no processing */
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  INT00 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   0: Interrupt Pin0
  */
/*--------------------------------------------------*/
void INT00_IRQHandler(void)
{
    /*--- IBIMC033: INTxCLR ---*/
    TSB_IB->IMC033 = (uint8_t)(0xC5);
    /* Dummy read is need. */
    {
        __IO uint8_t imc = TSB_IB->IMC033;
    }
    /* Process IDLE -> NORMAL_START */
    proc = CFG_PROC_NORMAL_START;
    /* LED status NORMAL Set */
    TSB_PJ_CR_PJ1C = GPIO_DISABLE;        /* switch off led1 on PJ1 */
    TSB_PJ_CR_PJ0C = GPIO_ENABLE;         /* set led0 on PJ0        */
}
/**
 *  @}
 */ /* End of group CG_IDLE_Exported_functions */

/**
 *  @}
 */ /* End of group CG_IDLE */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
