/** 
 *******************************************************************************
 * @file    main.c
 * @brief   CG_STOP1 Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-05-10 16:04:32 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup CG_STOP1 CG_STOP1 Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_STOP1_Private_macro CG_STOP1 Private Macro
 *  @{
 */

/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */
/** 
 *  @defgroup ProcModeConfiguration Processing Mode Configuration
 *  @brief    Processing Mode Normal Start.
 *  @{
 */
#define CFG_PROC_DEFAULT               ((uint32_t)(0))
#define CFG_PROC_NORMAL_START          ((uint32_t)(1))
/** 
 *  @}
 */ /* End of group ProcModeConfiguration */
/** 
 *  @defgroup IHOSCConfiguration IHOSC Configuration
 *  @brief    IHOSC Configuration.
 *  @{
 */
#define CFG_IHOSC_WARM_UP_TIME  ((uint32_t)(5000))      /*!< Warm up time(us).  Minimam warm-up time is 100(us). */
#define CFG_IHOSC_CLOCK         ((uint32_t)(10000000))  /*!< Clock(hz).        */
/** 
 *  @}
 */ /* End of group IHOSCConfiguration */
 
/** 
 *  @defgroup  SIWDxEN  SIWDxEN Register
 *  @brief     WD Enable Register.
 *  @{
 */
/* WDTE */
#define SIWDxEN_WDTE_RW_DISABLE         ((uint32_t)0x00000000)    /*!< WDTE  :[R/W] :Disable    */
#define SIWDxEN_WDTE_RW_ENABLE          ((uint32_t)0x00000001)    /*!< WDTE  :[R/W] :Enable     */
/* WDCR */
#define SIWDxCR_WDCR_W_DISABLE          ((uint32_t)0x000000B1)    /*!< WDCR  :[W] :Disable      */
#define SIWDxCR_WDCR_W_CLEAR            ((uint32_t)0x0000004E)    /*!< WDCR  :[W] :Clear        */
/** 
 *  @}
 */ /* End of group SIWDxEN */
 /** 
 *  @defgroup  CGSTBYCR  CGSTBYCR Register
 *  @brief     Standby Control Register.
 *  @{
 */
/* STBY */
#define CGSTBYCR_STBY_RW_IDLE           ((uint32_t)0x00000000)    /*!< STBY :[R/W] :IDLE   */
#define CGSTBYCR_STBY_RW_STOP1          ((uint32_t)0x00000001)    /*!< STBY :[R/W] :STOP1  */
#define CGSTBYCR_STBY_RW_STOP2          ((uint32_t)0x00000002)    /*!< STBY :[R/W] :STOP2  */
/** 
 *  @}
 */ /* End of group CGSTBYCR */
/** 
 *  @defgroup  CGWUPHCR  CGWUPHCR Register
 *  @brief     HOSC warming up Register.
 *  @{
 */
/* WUPT  */
#define CGWUPHCR_WUPT_HIGH_MASK         ((uint32_t)0xFFF00000)    /*!< WUPT    :High Bit Mask.  */
/* WUCLK */
#define CGWUPHCR_WUCLK_MASK             ((uint32_t)0x00000100)    /*!< WUCLK   :Mask.           */
#define CGWUPHCR_WUCLK_RW_IHOSC         ((uint32_t)0x00000000)    /*!< WUCLK   :[R/W]  :IHOSC   */
#define CGWUPHCR_WUCLK_RW_EHOSC         ((uint32_t)0x00000100)    /*!< WUCLK   :[R/W]  :EHOSC   */
/** 
 *  @}
 */ /* End of group CGWUPHCR */
/** 
 *  @defgroup  CGPLL0SEL  CGPLL0SEL Register
 *  @brief     fsys PLL Select Register.
 *  @{
 */
/* PLL0ST  */
#define CGPLL0SEL_PLL0ST_MASK           ((uint32_t)0x00000004)    /*!< PLL0ST  :Mask.           */
#define CGPLL0SEL_PLL0ST_R_FOSC         ((uint32_t)0x00000000)    /*!< PLL0ST  :[R]     :fosc   */
#define CGPLL0SEL_PLL0ST_R_FPLL         ((uint32_t)0x00000004)    /*!< PLL0ST  :[R]     :fpll   */
/* PLL0SEL */
#define CGPLL0SEL_PLL0SEL_MASK          ((uint32_t)0x00000002)    /*!< PLL0SEL :Mask.           */
#define CGPLL0SEL_PLL0SEL_RW_FOSC       ((uint32_t)0x00000000)    /*!< PLL0SEL :[R/W]   :fosc   */
#define CGPLL0SEL_PLL0SEL_RW_FPLL       ((uint32_t)0x00000002)    /*!< PLL0SEL :[R/W]   :fpll   */
/* PLL0ON  */
#define CGPLL0SEL_PLL0ON_MASK           ((uint32_t)0x00000001)    /*!< PLL0ON  :Mask.           */
#define CGPLL0SEL_PLL0ON_RW_DISABLE     ((uint32_t)0x00000000)    /*!< PLL0ON  :[R/W]   :Disable*/
#define CGPLL0SEL_PLL0ON_RW_ENABLE      ((uint32_t)0x00000001)    /*!< PLL0ON  :[R/W]   :Enable */
/** 
 *  @}
 */ /* End of group CGPLL0SEL */
 /** 
 *  @defgroup  CGOSCCR  CGOSCCR Register
 *  @brief     CG Control Register.
 *  @{
 */
/* IHOSC2F  */
#define CGOSCCR_IHOSC2F_MASK            ((uint32_t)0x00080000)    /*!< IHOSC2F   :Mask                  */
#define CGOSCCR_IHOSC2F_R_STOP          ((uint32_t)0x00000000)    /*!< IHOSC2F   :[R]   :Stop           */
#define CGOSCCR_IHOSC2F_R_RUNNING       ((uint32_t)0x00080000)    /*!< IHOSC2F   :[R]   :Running        */
/* OSCF     */
#define CGOSCCR_OSCF_MASK               ((uint32_t)0x00000200)    /*!< OSCF      :Mask                  */
#define CGOSCCR_OSCF_R_IHOSC            ((uint32_t)0x00000000)    /*!< OSCF      :[R]   :Internal HOSC  */
#define CGOSCCR_OSCF_R_EHOSC            ((uint32_t)0x00000200)    /*!< OSCF      :[R]   :External HOSC  */
/* OSCSEL   */
#define CGOSCCR_OSCSEL_MASK             ((uint32_t)0x00000100)    /*!< OSCSEL    :Mask                  */
#define CGOSCCR_OSCSEL_RW_IHOSC         ((uint32_t)0x00000000)    /*!< OSCSEL    :[R/W] :Internal HOSC  */
#define CGOSCCR_OSCSEL_RW_EHOSC         ((uint32_t)0x00000100)    /*!< OSCSEL    :[R/W] :External HOSC  */
/* IHOSC2EN */
#define CGOSCCR_IHOSC2EN_MASK           ((uint32_t)0x00000008)    /*!< IHOSC2EN  :Mask                  */
#define CGOSCCR_IHOSC2EN_RW_DISABLE     ((uint32_t)0x00000000)    /*!< IHOSC2EN  :[R/W] :Disable        */
#define CGOSCCR_IHOSC2EN_RW_ENABLE      ((uint32_t)0x00000008)    /*!< IHOSC2EN  :[R/W] :Enable         */
/* EOSCEN   */
#define CGOSCCR_EOSCEN_MASK             ((uint32_t)0x00000006)    /*!< EOSCEN    :Mask                  */
#define CGOSCCR_EOSCEN_RW_UNUSE         ((uint32_t)0x00000000)    /*!< EOSCEN    :[R/W] :Unuse          */
#define CGOSCCR_EOSCEN_RW_EHOSC         ((uint32_t)0x00000002)    /*!< EOSCEN    :[R/W] :External HOSC  */
#define CGOSCCR_EOSCEN_RW_ECLK          ((uint32_t)0x00000004)    /*!< EOSCEN    :[R/W] :External Clock */
/* IHOSC1EN */
#define CGOSCCR_IHOSC1EN_MASK           ((uint32_t)0x00000001)    /*!< IHOSC1EN  :Mask                  */
#define CGOSCCR_IHOSC1EN_RW_DISABLE     ((uint32_t)0x00000000)    /*!< IHOSC1EN  :[R/W] :Disable        */
#define CGOSCCR_IHOSC1EN_RW_ENABLE      ((uint32_t)0x00000001)    /*!< IHOSC1EN  :[R/W] :Enable         */
/** 
 *  @}
 */ /* End of group CGOSCCR */
 
 /** 
 *  @defgroup  FCSR0  FCSR0 Register
 *  @brief     Flash status Register 0.
 *  @{
 */
/*RDYBSY */
#define FCSR0_RDYBSY_R_BUSY             ((uint32_t)0x00000000)    /*!< WRDYBSY :[R] :Busy      */
#define FCSR0_RDYBSY_R_READY            ((uint32_t)0x00000001)    /*!< WRDYBSY :[R] :Ready     */
/** 
 *  @}
 */ /* End of group FCSR0 */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

/** 
 *  @}
 */ /* End of group CG_STOP1_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_STOP1_Private_variables CG_STOP1 Private Variables
 *  @{
 */
static uint32_t proc     = CFG_PROC_DEFAULT;      /*!< Processing mode.        */
static uint32_t counter = CFG_LED_BLINK_TIME;      /* Blink Time Counter        */
/**
 *  @}
 */ /* End of group CG_STOP1_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_STOP1_Private_fuctions CG_STOP1 Private Fuctions
 *  @{
 */
static void GPIO_Config(void);
static void wdt_disable(void);
static void pll_disable(void);
static void internal_hosc_enable(void);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 2 (LED2) */
  TSB_PJ_DATA_PJ2 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 2 (LED2) */
  TSB_PJ_CR_PJ2C = GPIO_DISABLE;
  /* (6) Enable the peripheral clock of PORT A */
  TSB_CG_FSYSMENA_IPMENA00 = FSYS_ENABLE;
  /* (7) Select output mode on PORT A pin 0 (SWS) */
  TSB_PA_CR_PA0C = GPIO_ENABLE;
  TSB_PA_DATA_PA0 = GPIO_ENABLE;   /*SW select 1:SW7-4 */
  /* (8) Select input mode on PORT A pin 2 (SW4) */
  TSB_PA_IE_PA2IE = GPIO_ENABLE;
  /* (9) Select input mode on PORT A pin 4 (SW6) */
  TSB_PA_IE_PA4IE = GPIO_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  WDT Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void wdt_disable(void)
{
    TSB_SIWD0->EN = SIWDxEN_WDTE_RW_DISABLE;
    TSB_SIWD0->CR = SIWDxCR_WDCR_W_DISABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  PLL Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void pll_disable(void)
{
    uint32_t work;

    /* [CGPLL0SEL]<PLL0SEL>: fosc           */
    work = (uint32_t)(TSB_CG->PLL0SEL & ~CGPLL0SEL_PLL0SEL_MASK);
    TSB_CG->PLL0SEL = (uint32_t)(work | CGPLL0SEL_PLL0SEL_RW_FOSC);
    /* [CGPLL0SEL]<PLL0ST> : wait for fosc  */
    while ((TSB_CG->PLL0SEL & CGPLL0SEL_PLL0ST_MASK) != CGPLL0SEL_PLL0ST_R_FOSC) 
    {
        /* no processing */
    }
    /* [CGPLL0SEL]<PLL0ON> : PLL Disable    */
    work = (uint32_t)(TSB_CG->PLL0SEL & ~CGPLL0SEL_PLL0ON_MASK);
    TSB_CG->PLL0SEL = (uint32_t)(work | CGPLL0SEL_PLL0ON_RW_DISABLE);
}
/*--------------------------------------------------*/
/** 
  * @brief  Internal HOSC Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void internal_hosc_enable(void)
{
    uint32_t work;

    /* [CGOSCCR]<IHOSC1EN> : HOSC Enable    */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_IHOSC1EN_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_IHOSC1EN_RW_ENABLE);
    /* [CGOSCCR]<OSCSEL>   : IHOSC          */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_OSCSEL_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_OSCSEL_RW_IHOSC);
    /* [CGOSCCR]<OSCF>     : Read(wait for IHOSC) */
    while ((TSB_CG->OSCCR & CGOSCCR_OSCF_MASK) != CGOSCCR_OSCF_R_IHOSC) 
    {
        /* no processing */
    }
}

/**
 *  @}
 */ /* End of group CG_STOP1_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_STOP1_Exported_functions CG_STOP1 Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  uint32_t period;    /* Period for Systick */
  uint32_t work;
  /*----------------------*/
  /* Port Setting         */
  /*----------------------*/
  GPIO_Config();
  /*--- IBIMC033: Enable ---*/
  TSB_IB->IMC033 = (uint8_t)(0x05);
  
  /*------------------------*/
  /* Normal Mode Processing */
  /*------------------------*/
  while(1){
    /* Period for 10msec SYSTICK */
    SystemCoreClockUpdate();
    period = SystemCoreClock / 100;
    if (SysTick_Config(period ))    /* systick interrupt cycle setting */
    {
    }
  
    while(TSB_PA_DATA_PA4 == 0)
    {
        /* no processing */
    }
    
    /*------------------------*/
    /* STOP1 Mode Processing  */
    /*------------------------*/
    /*----------------------*/
    /* WDT Disable          */
    /*----------------------*/
    wdt_disable();
    /*----------------------*/
    /* Wait FC status Ready */
    /*----------------------*/
    while((TSB_FC->SR0 && FCSR0_RDYBSY_R_READY) != FCSR0_RDYBSY_R_READY)
    {
        /* no processing */
    }
    /*----------------------*/
    /* Internal HOSC Select */
    /*----------------------*/
    /* [CGWUPHCR]<WUCLK> : IHOSC */
    work = (uint32_t)(TSB_CG->WUPHCR & ~CGWUPHCR_WUCLK_MASK);
    TSB_CG->WUPHCR = (uint32_t)(work | CGWUPHCR_WUCLK_RW_IHOSC);
    /*----------------------*/
    /* Internal HOSC Warm-up Time Set*/
    /*----------------------*/
    /*-----------------------------------------------*/
    /* "1"counter  (s) = 1 / IHOSC                   */
    /* "1"counter (us) = (10^6) / IHOSC              */
    /* "x"counter (us) = time                        */
    /*-----------------------------------------------*/
    /* x : time = 1 : (10^6) / IHOSC                 */
    /*-----------------------------------------------*/
    {
        uint64_t x = (uint64_t)((uint64_t)(CFG_IHOSC_WARM_UP_TIME) * (uint64_t)(CFG_IHOSC_CLOCK));
        x = (uint64_t)(x / (uint64_t)(1000000));
        if (x > (uint64_t)(0xFFFF))
        {
            /* invalid value */
        }
        work = (uint32_t)x;
    }
    work  &= (uint32_t)(0xFFFFFFF0);
    work <<= 16;
    work  |= (uint32_t)(TSB_CG->WUPHCR & ~CGWUPHCR_WUPT_HIGH_MASK);
    TSB_CG->WUPHCR = work;
    /*----------------------*/
    /* Set STBY             */
    /*----------------------*/
    /* [CGSTBYCR]<STBY>    :STOP1           */
    TSB_CG->STBYCR = CGSTBYCR_STBY_RW_STOP1;
    /*----------------------*/
    /* PLL Disable          */
    /*----------------------*/
    pll_disable();
    /*----------------------*/
    /* Internal HOSC Enable */
    /*----------------------*/
    internal_hosc_enable();
    /*----------------------*/
    /* External HOSC Disable*/
    /*----------------------*/
    /* [CGOSCCR]<EOSCEN>   : EHOSC Disable  */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_EOSCEN_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_EOSCEN_RW_UNUSE);
    /*----------------------*/
    /* OFD Internal HOSC Disable*/
    /*----------------------*/
    /* [CGOSCCR]<IHOSC2EN> : OFD IHOSC Disable  */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_IHOSC2EN_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_IHOSC2EN_RW_DISABLE);
    /*----------------------*/
    /* External HOSC Disable Check*/
    /*----------------------*/
    /* [CGOSCCR]<EOSCEN>   : Read  */
    while ((TSB_CG->OSCCR & CGOSCCR_EOSCEN_MASK) != CGOSCCR_EOSCEN_RW_UNUSE) 
    {
        /* no processing */
    }
    /*----------------------*/
    /* OFD Internal HOSC Disable Wait*/
    /*----------------------*/
    /* [CGOSCCR]<IHOSC2F>  : Read(wait for OFD IHOSC Disable)  */
    while ((TSB_CG->OSCCR & CGOSCCR_IHOSC2F_MASK) != CGOSCCR_IHOSC2F_R_STOP) 
    {
        /* no processing */
    }
    /* LED status STOP1 Set */
    TSB_PJ_CR_PJ0C = GPIO_DISABLE;        /* switch off led0 on PJ0 */
    TSB_PJ_CR_PJ2C = GPIO_ENABLE;         /* set led2 on PJ2          */
    TSB_PJ_DATA_PJ2 = 1;
    /* Interrupt Enable */
    NVIC_EnableIRQ(INT00_IRQn);     /* Interrupt pin 02 enable */
    /* State Change STOP1 Mode */
    __WFI();
    
    while(proc != CFG_PROC_NORMAL_START)
    {
        /* no processing */
    }
  }
}
/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
    if (counter-- == 0) 
    {
      /* The end of the counting down period, the data is
         inverted and stored in a register PJ0           */
      TSB_PJ_DATA_PJ0 ^= 1;     /* led0 blink for 1sec period */
      counter = CFG_LED_BLINK_TIME;
    }else{
        /* no processing */
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  INT00 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   0: Interrupt Pin0
  */
/*--------------------------------------------------*/
void INT00_IRQHandler(void)
{
    /*--- IBIMC033: INTxCLR ---*/
    TSB_IB->IMC033 = (uint8_t)(0xC5);
    /* Dummy read is need. */
    {
        __IO uint8_t imc = TSB_IB->IMC033;
    }
    /* Process STOP1 -> NORMAL_START */
    proc = CFG_PROC_NORMAL_START;
    /* LED status NORMAL Set */
    TSB_PJ_CR_PJ2C = GPIO_DISABLE;        /* switch off led2 on PJ2 */
    TSB_PJ_CR_PJ0C = GPIO_ENABLE;         /* set led0 on PJ0        */
}
/**
 *  @}
 */ /* End of group CG_STOP1_Exported_functions */

/**
 *  @}
 */ /* End of group CG_STOP1 */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
