/** 
 *******************************************************************************
 * @file    main.c
 * @brief   CRC_MEMORY_RAM Sample Application.
 * @version V1.0.0.0
 * $Date:: 2019-05-10 13:13:22 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */
#include "uart_io.h"
#include <stdio.h>
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup CRC_MEMORY_RAM CRC_MEMORY_RAM Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CRC_MEMORY_RAM_Private_macro CRC_MEMORY_RAM Private Macro
 *  @{
 */
/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */

 
/** 
 *  @name  CRCTYP CRCTYP Macro Definition.
 *  @brief CRCTYP Register Macro Definition.
 *  @{
 */
#define CRCTYP_INIT              ((uint32_t)0x00000000)     /*!< CFMT=00,DBIT=00 */
#define CRCTYP_8BIT              ((uint32_t)0x00000001)     /*!< CFMT=00,DBIT=01 */
#define CRCTYP_16BIT             ((uint32_t)0x00000002)     /*!< CFMT=00,DBIT=10 */
/**
 *  @}
 */ /* End of name CRCTYP Macro Definition */
 
/** 
 *  @name  CRCCLC CRCCLC Macro Definition.
 *  @brief CRCCLC Register Macro Definition.
 *  @{
 */
#define CRCCLC_16_INIT           ((uint32_t)0x0000FFFF)     /*!< CRC16 Initial Value  */
/**
 *  @}
 */ /* End of name CRCCLC Macro Definition */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

 /** 
 *  @defgroup  MEMORY_CONFIG MEMORY CONFIG
 *  @brief     MEMORY CONFIG
 *  @{
 */
#define MEMORY_TOP                      ((uint32_t)0x20000867)      /*!< Top                     */
#define MEMORY_SIZE                     ((uint32_t)0x00000009)      /*!< Size                    */
/** 
 *  @}
 */ /* End of group MEMORY_CONFIG */
 
/** 
 *  @}
 */ /* End of group CRC_MEMORY_RAM_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Public Member                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CRC_MEMORY_RAM_Public_variables CRC_MEMORY_RAM Public Variables
 *  @{
 */
uint16_t    crc_data;
#if defined ( __CC_ARM   )      /* RealView Compiler */
uint8_t comp_data[] __attribute__((at(MEMORY_TOP)))  = "123456789" ;
#elif defined ( __ICCARM__ )    /*IAR Compiler */
const uint8_t org_data[] = "123456789" ;
#endif
/**
 *  @}
 */ /* End of group CRC_MEMORY_RAM_Public_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CRC_MEMORY_RAM_Private_fuctions CRC_MEMORY_RAM Private Fuctions
 *  @{
 */
static void GPIO_LED_Config(void);
static void CRC_Config(void);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_LED_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_LED_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 1 (LED1) */
  TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 1 (LED1) */
  TSB_PJ_CR_PJ1C = GPIO_ENABLE;
  /* (6) Turn off PORT J pin 2 (LED2) */
  TSB_PJ_DATA_PJ2 = GPIO_DISABLE;
  /* (7) Select output mode on PORT J pin 2 (LED2) */
  TSB_PJ_CR_PJ2C = GPIO_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  CRC_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void CRC_Config(void)
{
  /* (1) Enable the peripheral clock of CRC  */
  TSB_CG_FSYSENA_IPENA00 = FSYS_ENABLE;
  
  /* Configure CRC */
  /* (1) CRC Control Register Setting, CRC16, 32bit data Width
         CFMT=00,DBIT=00                                        */
  /* (2) CRC Initial value Rsult Register,
         CRCCLC=0x0000FFFF                                      */
  TSB_CRC->TYP   = CRCTYP_INIT;    /* (1) */
  TSB_CRC->CLC   = CRCCLC_16_INIT;  /* (2) */
}

/**
 *  @}
 */ /* End of group TSPI_Private_functions */

/**
 *  @}
 */ /* End of group CRC_MEMORY_RAM_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CRC_MEMORY_RAM_Exported_functions CRC_MEMORY_RAM Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    uint32_t i;
    uint8_t *ptr = (uint8_t *)MEMORY_TOP;
    uint32_t length = MEMORY_SIZE;
    uint32_t work = 0x00;
#if defined ( __ICCARM__ )    /*IAR Compiler */
    for(i=0; i<length; ++i)
    {
        *ptr++ = org_data[i];
    }
    ptr = (uint8_t *)MEMORY_TOP;
#endif
    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    GPIO_LED_Config();
    /*------------------------*/
    /* UART Setting            */
    /*------------------------*/
    uart_io_initialize();
    /*------------------------*/
    /* CRC Setting */
    /*------------------------*/ 
    CRC_Config();
    
    /* CRC  Calculation */
    ptr = (uint8_t *)MEMORY_TOP;
    for(i=0; i < length; ++i)
    {
        work = ((work << 8U) | ((uint32_t)*ptr));
        ptr++;
        if((i % 4)==3)
        {
            TSB_CRC->DIN = work;
             work = 0x00;
        }
    }
    if((length % 4 )== 0)
    {
        /* no process */
    }
    else if((length % 4 )== 1)
    {
        TSB_CRC->TYP   = CRCTYP_8BIT;
        TSB_CRC->DIN = work;
    }
    else if((length % 4 )== 2)
    {
        TSB_CRC->TYP   = CRCTYP_16BIT;
        TSB_CRC->DIN = work;
    }
    else if((length % 4 )== 3)
    {
        uint32_t work2 = work >> 8U;
        TSB_CRC->TYP   = CRCTYP_16BIT;
        TSB_CRC->DIN = work2;
        work2 = work & 0xFFU;
        TSB_CRC->TYP   = CRCTYP_8BIT;
        TSB_CRC->DIN = work2;
    }
    
    crc_data = (uint16_t)TSB_CRC->CLC;
    {
        work = (uint32_t)crc_data;
        TSB_CRC->TYP   = CRCTYP_16BIT;
        TSB_CRC->DIN = work;
        if(TSB_CRC->CLC == 0x00)
        {
            TSB_PJ_DATA_PJ0 = 1;     /* led0 on */
        }
        else
        {
            TSB_PJ_DATA_PJ2 = 1;     /* led2 on */
        }
        printf("START = 0x%08x\r\n", MEMORY_TOP);
        printf("END   = 0x%08x\r\n", (MEMORY_TOP+MEMORY_SIZE-1));
        printf("CRC   = 0x%04x\r\n",crc_data);
    }
    while(1){
    }
}

/**
 *  @}
 */ /* End of group CRC_MEMORY_RAM_Exported_functions */

/**
 *  @}
 */ /* End of group CRC_MEMORY_RAM */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
