/** 
 *******************************************************************************
 * @file    main.c
 * @brief   DNF Sample Application.
 * @version V1.0.0.1
 * $Date:: 2019-05-10 15:19:26 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup DNF DNF Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DNF_Private_macro DNF Private Macro
 *  @{
 */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */
/** 
 *  @defgroup NFCKCR_NFCKS NFCKS
 *  @brief    [Bit2-0]: Noise Filter Clock Select.
 *  @{
 */
#define DNF_NFCKCR_NFCKS_STOP           ((uint32_t)0x00000000)        /*!< Clock Control circuit stop.                                       */
#define DNF_NFCKCR_NFCKS_CLOCK_2        ((uint32_t)0x00000001)        /*!< fc/2   fc=20MHz: 0.7us 40MHz: 0.35us 80MHz: 0.175us 100MHz:0.14us */
#define DNF_NFCKCR_NFCKS_CLOCK_4        ((uint32_t)0x00000002)        /*!< fc/4   fc=20MHz: 1.4us 40MHz: 0.7us  80MHz: 0.35us  100MHz:0.28us */
#define DNF_NFCKCR_NFCKS_CLOCK_8        ((uint32_t)0x00000003)        /*!< fc/8   fc=20MHz: 2.8us 40MHz: 1.4us  80MHz: 0.7us   100MHz:0.56us */
#define DNF_NFCKCR_NFCKS_CLOCK_16       ((uint32_t)0x00000004)        /*!< fc/16  fc=20MHz: 5.6us 40MHz: 2.8us  80MHz: 1.4us   100MHz:1.12us */
#define DNF_NFCKCR_NFCKS_CLOCK_32       ((uint32_t)0x00000005)        /*!< fc/32  fc=20MHz:11.2us 40MHz: 5.6us  80MHz: 2.8us   100MHz:2.24us */
#define DNF_NFCKCR_NFCKS_CLOCK_64       ((uint32_t)0x00000006)        /*!< fc/64  fc=20MHz:22.4us 40MHz:11.2us  80MHz: 5.6us   100MHz:4.48us */
#define DNF_NFCKCR_NFCKS_CLOCK_128      ((uint32_t)0x00000007)        /*!< fc/128 fc=20MHz:44.8us 40MHz:22.4us  80MHz:11.2us   100MHz:8.96us */
/**
 *  @}
 */ /* End of group NFCKCR_NFCKS */
/** 
 *  @}
 */ /* End of group DNF_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DNF_Private_fuctions DNF Private Fuctions
 *  @{
 */
static void GPIO_Config(void);
static void DNF_Config(void);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Enable the peripheral clock of PORT A */
  TSB_CG_FSYSMENA_IPMENA00 = FSYS_ENABLE;
  /* (5) Select output mode on PORT A pin 0 (SWS) */
  TSB_PA_CR_PA0C = GPIO_ENABLE;
  TSB_PA_DATA_PA0 = GPIO_ENABLE;   /*SW select 1:SW7-4 */
  /* (6) Select input mode on PORT A pin 2 (SW4) */
  TSB_PA_IE_PA2IE = GPIO_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  DNF_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void DNF_Config(void)
{
  /* (1) Enable the peripheral clock of DNF UnitA */
  TSB_CG_FCEN_FCIPEN26 = FSYS_ENABLE;
  /* (2) Enable the peripheral clock of DNF UnitB */
  TSB_CG_FCEN_FCIPEN27 = FSYS_ENABLE;
  /* (3) Enable the peripheral clock of DNF UnitC */
  TSB_CG_FCEN_FCIPEN28 = FSYS_ENABLE;
  /* (4) Noise Filter Clock Seting For fc/2 clock output */
  TSB_DNFA->CKCR = DNF_NFCKCR_NFCKS_CLOCK_2;
  /* (5) Noise Filter Clock Enable For INT00 */
  TSB_DNFA->ENCR = (1U << 0);
}

/**
 *  @}
 */ /* End of group DNF_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DNF_Exported_functions DNF Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_Config();
  /*------------------------*/
  /* DNF Setting           */
  /*------------------------*/
  DNF_Config();
  /*--- IBIMC033: Enable ---*/
  TSB_IB->IMC033 = (uint8_t)(0x05);  
  /* Interrupt Enable */
  NVIC_EnableIRQ(INT00_IRQn);     /* Interrupt pin 00 enable */
  while(1)
  {
  }
}
/*--------------------------------------------------*/
/** 
  * @brief  INT00 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   0: Interrupt pin 0
  */
/*--------------------------------------------------*/
void INT00_IRQHandler(void)
{
    /*--- IBIMC033: INTxCLR ---*/
    TSB_IB->IMC033 = (uint8_t)(0xC5);
    /* Dummy read is need. */
    {
        __IO uint8_t imc = TSB_IB->IMC033;
    }
    TSB_PJ_DATA_PJ0 ^= 1;                  /* Toggle led0 PJ0            */
}

/**
 *  @}
 */ /* End of group DNF_Exported_functions */

/**
 *  @}
 */ /* End of group DNF */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
