/** 
 *******************************************************************************
 * @file    main.c
 * @brief   FLASH_CODE Sample Application.
 * @version V1.0.0.2
 * $Date:: 2019-05-10 14:12:14 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */
#include <stdio.h>
#include <string.h>
#include "fc.h"
/**
 *  @addtogroup Example
 *  @{
 */
extern void copy_fc_func(void);
extern void copy_user_program(void);
extern uint32_t rewrite_user_program(void);
extern const char data_a[18];
extern const char data_b[18];
volatile uint32_t LedPattern;
/** 
 *  @defgroup FLASH_CODE FLASH_CODE Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_CODE_Private_macro FLASH_CODE Private Macro
 *  @{
 */
 /* OSCF     */
#define CGOSCCR_OSCF_MASK               ((uint32_t)0x00000200)    /*!< OSCF      :Mask                  */
#define CGOSCCR_OSCF_R_IHOSC            ((uint32_t)0x00000000)    /*!< OSCF      :[R]   :Internal HOSC  */
#define CGOSCCR_OSCF_R_EHOSC            ((uint32_t)0x00000200)    /*!< OSCF      :[R]   :External HOSC  */
/* OSCSEL   */
#define CGOSCCR_OSCSEL_MASK             ((uint32_t)0x00000100)    /*!< OSCSEL    :Mask                  */
#define CGOSCCR_OSCSEL_RW_IHOSC         ((uint32_t)0x00000000)    /*!< OSCSEL    :[R/W] :Internal HOSC  */
#define CGOSCCR_OSCSEL_RW_EHOSC         ((uint32_t)0x00000100)    /*!< OSCSEL    :[R/W] :External HOSC  */
/* IHOSC1EN */
#define CGOSCCR_IHOSC1EN_MASK           ((uint32_t)0x00000001)    /*!< IHOSC1EN  :Mask                  */
#define CGOSCCR_IHOSC1EN_RW_DISABLE     ((uint32_t)0x00000000)    /*!< IHOSC1EN  :[R/W] :Disable        */
#define CGOSCCR_IHOSC1EN_RW_ENABLE      ((uint32_t)0x00000001)    /*!< IHOSC1EN  :[R/W] :Enable         */

 /** 
 *  @defgroup  TXZ_RESULT TXZ_RESULT
 *  @brief     Result
 *  @{
 */
#define TXZ_SUCCESS                     ((uint32_t)0x00000000)    /*!< Success                 */
#define TXZ_ERROR                       ((uint32_t)0x00000001)    /*!< Error                   */
/** 
 *  @}
 */ /* End of group TXZ_RESULT */
 /** 
 *  @defgroup  TXZ_STATE TXZ_STATE
 *  @brief     Status
 *  @{
 */
#define TXZ_DONE                        ((uint32_t)0x00000000)    /*!< Done                    */
#define TXZ_BUSY                        ((uint32_t)0x00000001)    /*!< Busy                    */
/** 
 *  @}
 */ /* End of group TXZ_STATE */

 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

/** 
 *  @}
 */ /* End of group FLASH_CODE_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_CODE_Private_define FLASH_CODE Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,          /*!< Idle.             */
    PROC_START,         /*!< Start.            */
    PROC_RAMTRANSFER,   /*!< RAM transferring. */
    PROC_BLOCK_ERASING, /*!< Erasing.          */
    PROC_AREA_ERASING,  /*!< Erasing.          */
    PROC_PAGE_ERASING,  /*!< Erasing.          */
    PROC_REWRITING,     /*!< Rewriting.        */
    PROC_FINISHED,      /*!< Finished.         */
    PROC_ERROR          /*!< Error.            */
} Processing;

/** 
 *  @enum   fc_sr0_t
 *  @brief  Enumerated type definition of the FCSR0 register.
 */

/** 
 *  @}
 */ /* End of group FLASH_CODE_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_CODE_Private_variables FLASH_CODE Private Variables
 *  @{
 */
static Processing proc = PROC_IDLE;;         /*!< Processing mode.         */
static uint32_t count = 0;
/**
 *  @}
 */ /* End of group FLASH_CODE_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_CODE_Private_fuctions FLASH_CODE Private Fuctions
 *  @{
 */
static void internal_hosc_enable(void);
static void GPIO_Config(void);
/*--------------------------------------------------*/
/** 
  * @brief  Internal HOSC Enable
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void internal_hosc_enable(void)
{
    uint32_t work;

    /* [CGOSCCR]<IHOSC1EN> : HOSC Enable    */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_IHOSC1EN_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_IHOSC1EN_RW_ENABLE);
}
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 1 (LED1) */
  TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 1 (LED1) */
  TSB_PJ_CR_PJ1C = GPIO_ENABLE;
  /* (6) Turn off PORT J pin 2 (LED2) */
  TSB_PJ_DATA_PJ2 = GPIO_DISABLE;
  /* (7) Select output mode on PORT J pin 2 (LED2) */
  TSB_PJ_CR_PJ2C = GPIO_ENABLE;
  /* (8) Turn off PORT J pin 3 (LED3) */
  TSB_PJ_DATA_PJ3 = GPIO_DISABLE;
  /* (9) Select output mode on PORT J pin 3 (LED3) */
  TSB_PJ_CR_PJ3C = GPIO_ENABLE;
  /* (10) Enable the peripheral clock of PORT A */
  TSB_CG_FSYSMENA_IPMENA00 = FSYS_ENABLE;
  /* (11) Select output mode on PORT A pin 0 (SWS) */
  TSB_PA_CR_PA0C = GPIO_ENABLE;
  TSB_PA_DATA_PA0 = GPIO_ENABLE;   /*SW select 1:SW7-4 */
  /* (12) Select input mode on PORT A pin 2 (SW4) */
  TSB_PA_IE_PA2IE = GPIO_ENABLE;
  /* (13) Select input mode on PORT A pin 1 (SW5) */
  TSB_PA_IE_PA1IE = GPIO_ENABLE;
}

/**
 *  @}
 */ /* End of group FLASH_CODE_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_CODE_Exported_functions FLASH_CODE Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  char cmp_data[18] = "Excecute Data A \n";
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_Config();
  /*--- IBIMC033: Enable ---*/
  TSB_IB->IMC033 = (uint8_t)(0x05);
  
  internal_hosc_enable();         /* internal HOSC enable */
  NVIC_EnableIRQ(INT00_IRQn);     /* interrupt pin0 enable */
  while(1){
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(proc != PROC_ERROR)
    {
        switch(proc)
        {
        case PROC_IDLE:
            if(strncmp(cmp_data, data_a, sizeof(cmp_data)) == 0) {
                TSB_PJ_DATA_PJ1  = 0;     /* led1 turn off              */
                TSB_PJ_DATA_PJ3  = 0;     /* led3 turn off              */
                TSB_PJ_DATA_PJ0  = 1;     /* led0 turn on               */
                TSB_PJ_DATA_PJ2  = 1;     /* led2 turn on               */
            }
            else if(strncmp(cmp_data, data_b, sizeof(cmp_data)) == 0) {
                TSB_PJ_DATA_PJ0  = 0;     /* led0 turn off              */
                TSB_PJ_DATA_PJ2  = 0;     /* led2 turn off              */
                TSB_PJ_DATA_PJ1  = 1;     /* led1 turn on               */
                TSB_PJ_DATA_PJ3  = 1;     /* led3 turn on               */
            }
            proc = PROC_RAMTRANSFER;
            break;
        case PROC_RAMTRANSFER:
            copy_fc_func();
            copy_user_program();
            break;
        case PROC_PAGE_ERASING:
            fc_erase_page_code_flash(FC_CODE_FLASH_PAGE8, 1);
            fc_erase_page_code_flash(FC_CODE_FLASH_PAGE16, 1);
            proc = PROC_REWRITING;
            break;
        case PROC_BLOCK_ERASING:
            fc_erase_block_code_flash(FC_CODE_FLASH_BLOCK1, 1);
            fc_erase_block_code_flash(FC_CODE_FLASH_BLOCK2, 1);
            proc = PROC_REWRITING;
            break;
        case PROC_AREA_ERASING:
            fc_erase_area_code_flash(FC_CODE_FLASH_AREA0);
            proc = PROC_FINISHED;
            break;
        case PROC_REWRITING:
            if(rewrite_user_program() == TXZ_ERROR){
                proc = PROC_ERROR;
            }
            else {
                proc = PROC_FINISHED;
            }
            break;
        case PROC_FINISHED:
            /*--- IBIMC033: Enable ---*/
            TSB_IB->IMC033 = (uint8_t)(0x05);
            NVIC_EnableIRQ(INT00_IRQn);     /* interrupt pin0 enable */
            proc = PROC_IDLE;
            break;
        case PROC_ERROR:
            /* no processing */
            break;
        default:
            proc = PROC_ERROR;
            break;
        }
    }
  }
}

/*--------------------------------------------------*/
/** 
  * @brief  INT00 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   0: Interrupt Pin0
  */
/*--------------------------------------------------*/
void INT00_IRQHandler(void)
{
    NVIC_DisableIRQ(INT00_IRQn);     /* interrupt pin0 disable */
    /*--- IBIMC033: INTxCLR ---*/
    TSB_IB->IMC033 = (uint8_t)(0xC0);
    /* Dummy read is need. */
    {
        __IO uint8_t imc = TSB_IB->IMC033;
    }
    switch(count)
    {
        case 0:
          proc = PROC_PAGE_ERASING;
          count++;
          break;
        case 1:
          proc = PROC_BLOCK_ERASING;
          count=0;
          break;
#if 0
        case 2:
          proc = PROC_AREA_ERASING;
          count=0;
          break;
#endif
        default:
          break;
    }
}
/**
 *  @}
 */ /* End of group FLASH_CODE_Exported_functions */

/**
 *  @}
 */ /* End of group FLASH_CODE */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
