/** 
 *******************************************************************************
 * @file    f2ram.c
 * @brief   FLASH_DATA Sample Application.
 * @version V1.0.0.1
 * $Date:: 2019-05-10 14:14:06 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */
#include "dfc.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup FLASH_DATA FLASH_DATA Sample Appli
 *  @{
 */

#if defined ( __CC_ARM   )      /* RealView Compiler */
extern uint32_t Load$$FLASH_CODE_RAM$$Base;
extern uint32_t Image$$FLASH_CODE_RAM$$Base;
extern uint32_t Load$$FLASH_CODE_RAM$$Length;
extern uint32_t Image$$FLASH_DEMO_A$$Base;
extern uint32_t Load$$FLASH_DEMO_B$$Base;
extern uint32_t Image$$RAM_DEMO_A$$Base;
extern uint32_t Image$$RAM_DEMO_B$$Base;
extern uint32_t Image$$FRAM_DEMO_A$$Base;
extern uint32_t Image$$FRAM_DEMO_B$$Base;
extern uint32_t Image$$FRAM_DEMO_A$$Length;
extern uint32_t Image$$FRAM_DEMO_B$$Length;
#elif defined ( __ICCARM__ )    /* IAR Compiler */
#pragma section = "FLASH_CODE_RAM"
#pragma section = "FLASH_CODE_ROM"
#pragma section = "FLASH_DEMO_A"
#pragma section = "FLASH_DEMO_B"
#pragma section = "RAM_DEMO_A"
#pragma section = "RAM_DEMO_B"
#pragma section = "FRAM_DEMO_A"
#pragma section = "FRAM_DEMO_B"
#endif


/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_DATA_Private_macro FLASH_DATA Private Macro
 *  @{
 */

 /** 
 *  @defgroup  TXZ_RESULT TXZ_RESULT
 *  @brief     Result
 *  @{
 */
#define TXZ_SUCCESS                     ((uint32_t)0x00000000)    /*!< Success                 */
#define TXZ_ERROR                       ((uint32_t)0x00000001)    /*!< Error                   */
/** 
 *  @}
 */ /* End of group TXZ_RESULT */
 /** 
 *  @defgroup  TXZ_STATE TXZ_STATE
 *  @brief     Status
 *  @{
 */
#define TXZ_DONE                        ((uint32_t)0x00000000)    /*!< Done                    */
#define TXZ_BUSY                        ((uint32_t)0x00000001)    /*!< Busy                    */
/** 
 *  @}
 */ /* End of group TXZ_STATE */

/** 
 *  @}
 */ /* End of group FLASH_DATA_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_DATA_Private_define FLASH_DATA Private Define
 *  @{
 */
#if defined ( __CC_ARM   )      /* RealView Compiler */
#define FLASH_API_ROM           (uint32_t *)&Load$$FLASH_CODE_RAM$$Base

#define FLASH_API_RAM           (uint32_t *)&Image$$FLASH_CODE_RAM$$Base
#define SIZE_FLASH_API          (uint32_t)&Load$$FLASH_CODE_RAM$$Length

#define DEMO_A_RAM              (uint32_t *)&Image$$RAM_DEMO_A$$Base
#define DEMO_B_RAM              (uint32_t *)&Image$$RAM_DEMO_B$$Base

#define DEMO_A_FRAM              (uint32_t *)&Image$$FRAM_DEMO_A$$Base
#define DEMO_B_FRAM              (uint32_t *)&Image$$FRAM_DEMO_B$$Base

#elif defined ( __ICCARM__ )    /* IAR Compiler */
#define FLASH_API_ROM           ((uint32_t *)__section_begin("FLASH_CODE_ROM"))

#define FLASH_API_RAM           ((uint32_t *)__section_begin("FLASH_CODE_RAM"))
#define SIZE_FLASH_API          ((uint32_t)__section_size("FLASH_CODE_ROM"))


#define DEMO_A_RAM              ((uint32_t *)__section_begin("RAM_DEMO_A"))
#define DEMO_B_RAM              ((uint32_t *)__section_begin("RAM_DEMO_B"))

#define DEMO_A_FRAM              ((uint32_t *)__section_begin("FRAM_DEMO_A"))
#define DEMO_B_FRAM              ((uint32_t *)__section_begin("FRAM_DEMO_B"))
#endif

/* code start address */
#define DEMO_START_ADDR         ((uint32_t)DEMO_A_FLASH)

/* DEMO RAM SIZE */
#define DEMO_SIZE         ((uint32_t)0x400)

/** 
 *  @}
 */ /* End of group FLASH_DATA_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_DATA_Private_fuctions FLASH_DATA Private Fuctions
 *  @{
 */
static void Copy_Routine(uint32_t * dest, uint32_t * source, uint32_t size);
void (* startup) (void);
uint32_t gLoadAddr;


/*--------------------------------------------------*/
/** 
  * @brief  copy 32-bit data from source to dest
  * @param  the address of source and dast, the data size
  * @retval None.
  * @note   -
  */
/*--------------------------------------------------*/
static void Copy_Routine(uint32_t * dest, uint32_t * source, uint32_t size)
{
    uint32_t *dest_addr, *source_addr, tmpsize;
    uint32_t i, tmps, tmpd, mask;

    dest_addr = dest;
    source_addr = source;

    tmpsize = size >> 2U;
    for (i = 0U; i < tmpsize; i++) {    /* 32bits copy */
        *dest_addr = *source_addr;
        dest_addr++;
        source_addr++;
    }
    if (size & 0x00000003U) {   /* if the last data size is not 0(maybe 1,2 or 3), copy the last data */
        mask = 0xFFFFFF00U;
        i = size & 0x00000003U;
        tmps = *source_addr;
        tmpd = *dest_addr;
        while (i - 1U) {
            mask = mask << 8U;
            i--;
        }
        tmps = tmps & (~mask);
        tmpd = tmpd & (mask);
        *dest_addr = tmps + tmpd;       /* 32bits copy, but only change the bytes need to be changed */
    } else {
        /* Do nothing */
    }
}
/**
 *  @}
 */ /* End of group FLASH_DATA_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_DATA_Exported_functions FLASH_DATA Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  copy flash control functions.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void copy_fc_func(void)
{
    Copy_Routine(FLASH_API_RAM, FLASH_API_ROM, SIZE_FLASH_API);     /* copy flash API to RAM */
}

/*--------------------------------------------------*/
/** 
  * @brief  copy fram to ram.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void copy_fram_ram(void)
{
    Copy_Routine(DEMO_A_RAM, DEMO_A_FRAM, DEMO_SIZE);     /* copy flash RAM to RAM */
    Copy_Routine(DEMO_B_RAM, DEMO_B_FRAM, DEMO_SIZE);     /* copy flash RAM to RAM */
}

/*--------------------------------------------------*/
/** 
  * @brief  rewrite user program.
  * @param  -
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t rewrite_user_ram(void)
{
    uint32_t retval = TXZ_SUCCESS;

    if(fc_write_data_flash(DEMO_B_RAM, DEMO_A_FRAM, DEMO_SIZE) == TXZ_ERROR){
        retval = TXZ_ERROR;
    }
    else {
        if(fc_write_data_flash(DEMO_A_RAM, DEMO_B_FRAM, DEMO_SIZE) == TXZ_ERROR){
            retval = TXZ_ERROR;
        }
        else {
            /* do nothing */
        }
    }
    return (retval);

}

/**
 *  @}
 */ /* End of group FLASH_DATA_Exported_functions */

/**
 *  @}
 */ /* End of group FLASH_DATA */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
