/** 
 *******************************************************************************
 * @file    main.c
 * @brief   FLASH_USER_INFORMATION Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-05-10 15:21:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */
#include "fc.h"
#include "uart_io.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup FLASH_USER_INFORMATION FLASH_USER_INFORMATION Sample Appli
 *  @{
 */
extern void copy_fc_func(void);

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_USER_INFORMATION_Private_define FLASH_USER_INFORMATION Private Define
 *  @{
 */

/** 
 *  @defgroup SWPollingConfiguration SW Polling Time Configuration
 *  @brief    SW Polling Time(ms).
 *  @{
 */
#define CFG_SW_POLLING_TIME     ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group SWPollingConfiguration */

/** 
 *  @}
 */ /* End of group FLASH_USER_INFORMATION_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_USER_INFORMATION_Private_define FLASH_USER_INFORMATION Private Define
 *  @{
 */

#define MAIN_NULL       ((void *)0)     /*!< Null Pointer  */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */
/* OSCF     */
#define CGOSCCR_OSCF_MASK               ((uint32_t)0x00000200)    /*!< OSCF      :Mask                  */
#define CGOSCCR_OSCF_R_IHOSC            ((uint32_t)0x00000000)    /*!< OSCF      :[R]   :Internal HOSC  */
#define CGOSCCR_OSCF_R_EHOSC            ((uint32_t)0x00000200)    /*!< OSCF      :[R]   :External HOSC  */
/* OSCSEL   */
#define CGOSCCR_OSCSEL_MASK             ((uint32_t)0x00000100)    /*!< OSCSEL    :Mask                  */
#define CGOSCCR_OSCSEL_RW_IHOSC         ((uint32_t)0x00000000)    /*!< OSCSEL    :[R/W] :Internal HOSC  */
#define CGOSCCR_OSCSEL_RW_EHOSC         ((uint32_t)0x00000100)    /*!< OSCSEL    :[R/W] :External HOSC  */
/* IHOSC1EN */
#define CGOSCCR_IHOSC1EN_MASK           ((uint32_t)0x00000001)    /*!< IHOSC1EN  :Mask                  */
#define CGOSCCR_IHOSC1EN_RW_DISABLE     ((uint32_t)0x00000000)    /*!< IHOSC1EN  :[R/W] :Disable        */
#define CGOSCCR_IHOSC1EN_RW_ENABLE      ((uint32_t)0x00000001)    /*!< IHOSC1EN  :[R/W] :Enable         */

/** 
 *  @defgroup  TXZ_RESULT TXZ_RESULT
 *  @brief     Result
 *  @{
 */
#define TXZ_SUCCESS                     ((uint32_t)0x00000000)    /*!< Success                 */
#define TXZ_ERROR                       ((uint32_t)0x00000001)    /*!< Error                   */
/** 
 *  @}
 */ /* End of group TXZ_RESULT */
 /** 
 *  @defgroup  TXZ_STATE TXZ_STATE
 *  @brief     Status
 *  @{
 */
#define TXZ_DONE                        ((uint32_t)0x00000000)    /*!< Done                    */
#define TXZ_BUSY                        ((uint32_t)0x00000001)    /*!< Busy                    */
/** 
 *  @}
 */ /* End of group TXZ_STATE */

 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */
 /** 
 *  @defgroup  SW_CHATARING SW_CHATARING
 *  @brief     SW Chatatinf
 *  @{
 */
#define SW_CHATARING                    ((uint32_t)0x0000000A)    /*!< chataring               */
#define SW_ON_STATE                     ((uint32_t)0x00000001)    /*!< On State                */
/** 
 *  @}
 */ /* End of group SW_CHATARING */
/** 
 *  @}
 */ /* End of group FLASH_USER_INFORMATION_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_USER_INFORMATION_Private_define FLASH_USER_INFORMATION Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,          /*!< Idle.             */
    PROC_START,         /*!< Start.            */
    PROC_RAMTRANSFER,   /*!< RAM transferring. */
    PROC_ERASING,       /*!< Erasing.          */
    PROC_WRITING,       /*!< writing.        */
    PROC_FINISHED,      /*!< Finished.         */
    PROC_ERROR          /*!< Error.            */
} Processing;
#define USERINFOSIZE      (128)        /*<! User Information data size */

/** 
 *  @}
 */ /* End of group FLASH_USER_INFORMATION_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_USER_INFORMATION_Private_variables FLASH_USER_INFORMATION Private Variables
 *  @{
 */
static Processing proc;         /*!< Processing mode.         */
static char UserInformationData[USERINFOSIZE] =
{
	"Toshiba TMPM4KQ User Information Example"
};                             /*!< User Information Data  */
static char GetUserInformationData[USERINFOSIZE];
static uint32_t sw_new = 0;
static uint32_t sw_old = 0;
static uint32_t sw_state = 0;
static uint32_t sw_chataring = 0;
/**
 *  @}
 */ /* End of group FLASH_USER_INFORMATION_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_USER_INFORMATION_Private_fuctions FLASH_USER_INFORMATION Private Fuctions
 *  @{
 */
static void internal_hosc_enable(void);
static void GPIO_SW_Config(void);
/*--------------------------------------------------*/
/** 
  * @brief  Internal HOSC Enable
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void internal_hosc_enable(void)
{
    uint32_t work;

    /* [CGOSCCR]<IHOSC1EN> : HOSC Enable    */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_IHOSC1EN_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_IHOSC1EN_RW_ENABLE);
}

/*--------------------------------------------------*/
/** 
  * @brief  GPIO_SW_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_SW_Config(void)
{
  /* (1) Enable the peripheral clock of PORT A */
  TSB_CG_FSYSMENA_IPMENA00 = FSYS_ENABLE;
  /* (2) Select output mode on PORT A pin 0 (SWS) */
  TSB_PA_CR_PA0C = GPIO_ENABLE;
  TSB_PA_DATA_PA0 = GPIO_ENABLE;   /*SW select 1:SW7-4 */
  /* (3) Select input mode on PORT A pin 2 (SW4) */
  TSB_PA_IE_PA2IE = GPIO_ENABLE;
}

/**
 *  @}
 */ /* End of group FLASH_USER_INFORMATION_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_USER_INFORMATION_Exported_functions FLASH_USER_INFORMATION Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/

int main(void)
{
    uint32_t period;    /* Period for Systick */
    
    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    GPIO_SW_Config();
    uart_io_initialize();
    
    internal_hosc_enable();         /* internal HOSC enable */
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    /* Period for 10msec SYSTICK */
    SystemCoreClockUpdate();
    period = SystemCoreClock / 100;

    while(proc != PROC_ERROR)
    {
        switch(proc)
        {
        case PROC_IDLE:
            printf("******************** \n");
            printf("FLASH User Information Area Write Example\n");
            printf("******************** \n");
            printf("Please press the S4 \n");
            
            proc = PROC_START;

            if (SysTick_Config(period ))    /* systick interrupt cycle setting & Interrupt Enable */
            {
            }
            break;
        case PROC_START:
            /* SW waiting. */
            break;
        case PROC_RAMTRANSFER:
            printf("RAM transferring. \n");
            copy_fc_func();
            proc = PROC_ERASING;
            break;
        case PROC_ERASING:
            /* Interrupt Disable */
            SysTick->CTRL  = SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_TICKINT_Msk; /* Disable */

            if(fc_erase_user_information_area() == TXZ_ERROR){
                proc = PROC_ERROR;
            }
            else {
                printf("Erasing. \n");
                proc = PROC_WRITING;
            }
            break;
        case PROC_WRITING:
            if(fc_write_user_information_area((uint32_t *)UserInformationData, USERINFOSIZE) == TXZ_ERROR){
                proc = PROC_ERROR;
            }
            else {
                printf("writing. \n");
                proc = PROC_FINISHED;
            }
            break;
        case PROC_FINISHED:
            fc_read_user_information_area((uint32_t *)GetUserInformationData, USERINFOSIZE);
            printf("Finished. \n");
            printf("check a variable GetUserInformationData. \n");
            proc = PROC_IDLE;
            break;
        case PROC_ERROR:
            printf("writing error. \n");
            /* no processing */
            break;
        default:
            proc = PROC_ERROR;
            break;
        }
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
    sw_new = TSB_PA_DATA_PA2;
    if(sw_new == sw_old)
    {
        if(++sw_chataring >= SW_CHATARING)
        {
            sw_chataring = 0;
            if(sw_new != sw_state)
            {
                sw_state = sw_new;
                if(sw_state == SW_ON_STATE)
                {
                    proc = PROC_RAMTRANSFER;
                }
            }
        }
    }
    else
    {
        sw_old = sw_new;
        sw_chataring = 0;
    }
}
/**
 *  @}
 */ /* End of group FLASH_USER_INFORMATION_Exported_functions */

/**
 *  @}
 */ /* End of group FLASH_USER_INFORMATION */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
