/** 
 *******************************************************************************
 * @file    main.c
 * @brief   I2C_SLAVE_TRANS Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-06-25 15:17:07 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup I2C_SLAVE_TRANS I2C_SLAVE_TRANS Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_SLAVE_TRANS_Private_macro I2C_SLAVE_TRANS Private Macro
 *  @{
 */
/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */
/** 
 *  @defgroup  I2CError I2C Error 
 *  @brief    Error  .
 *  @{
 */
#define ERROR_I2C       ((uint32_t)(1))
/** 
 *  @}
 */ /* End of group I2CError */
/** 
 *  @defgroup  I2C_DataLength I2C Data Length
 *  @brief    I2C Data Length.
 *  @{
 */
#define CFG_I2C_DATA_LENGTH     (16)                /*!< Length of base data.          */
/** 
 *  @}
 */ /* End of group I2C_DataLength */
 
/** 
 *  @defgroup  I2C_ADDRESS I2C Address
 *  @brief    I2C Frequency.
 *  @{
 */
#define CFG_I2C_SELF_ADDR       ((uint8_t)0x40)     /*!< Self address for test.        */
#define CFG_I2C_SLAVE_ADDR      ((uint8_t)0x40)     /*!< Slave address for test.       */
/** 
 *  @}
 */ /* End of group I2C_ADDRESS */
/** 
 *  @name  I2CxSR Macro Definition.
 *  @brief I2CxSR Register Macro Definition.
 *  @{
 */
#define I2CxSR_MST              ((uint32_t)0x00000080)     /*!< MST                     */
#define I2CxSR_TRX              ((uint32_t)0x00000040)     /*!< TRX                     */
#define I2CxSR_BB               ((uint32_t)0x00000020)     /*!< BB                      */
#define I2CxSR_PIN              ((uint32_t)0x00000010)     /*!< PIN                     */
#define I2CxSR_AL               ((uint32_t)0x00000008)     /*!< AL                      */
#define I2CxSR_AAS              ((uint32_t)0x00000004)     /*!< AAS                     */
#define I2CxSR_AD0              ((uint32_t)0x00000002)     /*!< AD0                     */
#define I2CxSR_LRB              ((uint32_t)0x00000001)     /*!< LRB                     */
/**
 *  @}
 */ /* End of name I2CxSR Macro Definition */
/** 
 *  @name  I2CxOP Macro Definition.
 *  @brief I2CxOP Register Macro Definition.
 *  @{
 */
#define I2CxOP_DISAL            ((uint32_t)0x00000080)     /*!< DISAL                   */
#define I2CxOP_SA2ST            ((uint32_t)0x00000040)     /*!< SA2ST                   */
#define I2CxOP_SAST             ((uint32_t)0x00000020)     /*!< SAST                    */
#define I2CxOP_NFSEL            ((uint32_t)0x00000010)     /*!< NFSEL                   */
#define I2CxOP_RSTA             ((uint32_t)0x00000008)     /*!< RSTA                    */
#define I2CxOP_GCDI             ((uint32_t)0x00000004)     /*!< GDDI                    */
#define I2CxOP_SREN             ((uint32_t)0x00000002)     /*!< SREN                    */
#define I2CxOP_MFACK            ((uint32_t)0x00000001)     /*!< MFACK                   */
#define I2CxOP_INIT             ((uint32_t)0x00000000)     /*!< Master Initial Settings.  */
#define I2CxOP_SLAVE_INIT       ((uint32_t)0x00000084)     /*!< Slave Initial Settings.   */
/**
 *  @}
 */ /* End of name I2CxOP Macro Definition */
/** 
 *  @name  I2CxCR1 Macro Definition.
 *  @brief I2CxCR1 Register Macro Definition.
 *  @{
 */
#define I2CxCR1_ACK             ((uint32_t)0x00000010)     /*!< ACK         */
#define I2CxCR1_NOACK           ((uint32_t)0x00000008)     /*!< NOACK       */
#define I2CxCR1_BC              ((uint32_t)0x000000E0)     /*!< BC          */

/**
 *  @}
 */ /* End of name I2CxCR1 Macro Definition */
/** 
 *  @name  I2CxCR2 Macro Definition.
 *  @brief I2CxCR2 Register Macro Definition.
 *  @{
 */
#define I2CxCR2_PIN_CLEAR       ((uint32_t)0x00000010)     /*!< PIN=1                                  */
#define I2CxCR2_I2CM_DISABLE    ((uint32_t)0x00000000)     /*!< I2CM=0                                 */
#define I2CxCR2_I2CM_ENABLE     ((uint32_t)0x00000008)     /*!< I2CM=1                                 */
#define I2CxCR2_SWRES_10        ((uint32_t)0x00000002)     /*!< SWRES=10                               */
#define I2CxCR2_SWRES_01        ((uint32_t)0x00000001)     /*!< SWRES=01                               */
#define I2CxCR2_START_CONDITION ((uint32_t)0x000000F8)     /*!< MST=1,TRX=1,BB=1,PIN=1,I2CM=1          */
#define I2CxCR2_STOP_CONDITION  ((uint32_t)0x000000D8)     /*!< MST=1,TRX=1,BB=0,PIN=1,I2CM=1          */
#define I2CxCR2_INIT            ((uint32_t)0x00000008)     /*!< MST=0,TRX=0,BB=0,PIN=0,I2CM=1,SWRES=00 */

/**
 *  @}
 */ /* End of name I2CxCR2 Macro Definition */

/** 
 *  @name  I2CxAR Macro Definition.
 *  @brief I2CxAR Register Macro Definition.
 *  @{
 */
#define I2CxAR_ALS              ((uint32_t)0x00000001)      /*!< ALS.                   */
#define I2CxAR_INIT             ((uint32_t)0x00000000)      /*!< Initial Settings.      */
#define I2CxAR2_INIT            ((uint32_t)0x00000000)      /*!< Initial Settings.      */

/**
 *  @}
 */ /* End of name I2CxAR Macro Definition */
/** 
 *  @name  I2CxIE Macro Definition.
 *  @brief I2CxIE Register Macro Definition.
 *  @{
 */
#define I2CxIE_SELPINCD         ((uint32_t)0x00000040)     /*!< SELPINCD                */
#define I2CxIE_DMARI2CTX        ((uint32_t)0x00000020)     /*!< DMARI2CTX               */
#define I2CxIE_DMARI2CRX        ((uint32_t)0x00000010)     /*!< DMARI2CRX               */
#define I2CxIE_INTNACK          ((uint32_t)0x00000008)     /*!< NACK                    */
#define I2CxIE_I2C              ((uint32_t)0x00000001)     /*!< INTI2C                  */
#define I2CxIE_CLEAR            ((uint32_t)0x00000000)     /*!< All Clear Setting       */

/**
 *  @}
 */ /* End of name I2CxIE Macro Definition */
 
 /** 
 *  @name  I2CxDBR Macro Definition.
 *  @brief I2CxDBR Register Macro Definition.
 *  @{
 */
#define I2CxDBR_DB_MASK         ((uint32_t)0x000000FF)      /* !< DB 7-0 bits mask.     */
/**
 *  @}
 */ /* End of name I2CxDBR Macro Definition */
 /** 
 *  @name  I2CxST Macro Definition.
 *  @brief I2CxST Register Macro Definition.
 *  @{
 */
#define I2CxST_NACK             ((uint32_t)0x00000008)    /*!< NACK Interrupt Status.   */
#define I2CxST_I2CBF            ((uint32_t)0x00000004)    /*!< I2CBF Interrupt Status.  */
#define I2CxST_I2CAL            ((uint32_t)0x00000002)    /*!< I2CAL Interrupt Status.  */
#define I2CxST_I2C              ((uint32_t)0x00000001)    /*!< I2C Interrupt Status.    */
#define I2CxST_CLEAR            ((uint32_t)0x0000000F)    /*!< All Bits Clear.          */
/**
 *  @}
 */ /* End of name I2CxST Macro Definition */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

/** 
 *  @}
 */ /* End of group I2C_SLAVE_TRANS_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_SLAVE_TRANS_Private_variables I2C_SLAVE_TRANS Private Variables
 *  @{
 */
static uint8_t send = 0;
static const uint8_t tx_data[] = "TMPM4KQ\n";
static uint8_t error = 0;
static uint32_t counter  = CFG_LED_BLINK_TIME;    /* Blink Time Counter        */
/**
 *  @}
 */ /* End of group I2C_SLAVE_TRANS_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_SLAVE_TRANS_Private_fuctions I2C_SLAVE_TRANS Private Fuctions
 *  @{
 */
static void GPIO_LED_Config(void);
static void GPIO_SW_Config(void);
static void GPIO_I2C_Slave_Config(void);
static void I2C_Slave_Config(void);
static void I2C_stop_condition(void);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_LED_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_LED_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 1 (LED1) */
  TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 1 (LED1) */
  TSB_PJ_CR_PJ1C = GPIO_DISABLE;
  /* (6) Turn off PORT J pin 2 (LED2) */
  TSB_PJ_DATA_PJ2 = GPIO_DISABLE;
  /* (7) Select output mode on PORT J pin 2 (LED2) */
  TSB_PJ_CR_PJ2C = GPIO_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_SW_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_SW_Config(void)
{
  /* (1) Enable the peripheral clock of PORT A */
  TSB_CG_FSYSMENA_IPMENA00 = FSYS_ENABLE;
  /* (2) Select output mode on PORT A pin 0 (SWS) */
  TSB_PA_CR_PA0C = GPIO_ENABLE;
  TSB_PA_DATA_PA0 = GPIO_ENABLE;   /*SW select 1:SW7-4 */
  /* (3) Select input mode on PORT A pin 2 (SW4) */
  TSB_PA_IE_PA2IE = GPIO_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  GPIO_I2C_Slave_Config
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_I2C_Slave_Config(void)
{
  /* (1) Enable the peripheral clock of PORT U */
  TSB_CG_FSYSMENA_IPMENA16 = FSYS_ENABLE;
  /* (2) Select function mode on PORT U pin 0  */
  TSB_PU_FR3_PU0F3 = GPIO_ENABLE;
  /* (3) Select output mode on PORT U pin 0    */
  TSB_PU_CR_PU0C = GPIO_ENABLE;
  /* (5) Select input mode on PORT U pin 0     */
  TSB_PU_IE_PU0IE = GPIO_ENABLE;
  /* (6) Select function mode on PORT U pin 1  */
  TSB_PU_FR3_PU1F3 = GPIO_ENABLE;
  /* (7) Select output mode on PORT U pin 1    */
  TSB_PU_CR_PU1C = GPIO_ENABLE;
  /* (8) Select input mode on PORT U pin1      */
  TSB_PU_IE_PU1IE = GPIO_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  I2C_Slave_Config
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void I2C_Slave_Config(void)
{  
  /* (1) Enable the peripheral clock of I2C ch1 */
  TSB_CG_FSYSMENA_IPMENA26 = FSYS_ENABLE;
  
  /* Configure I2C1, slave */
  /* (2) Timing register value is fast Mode @800kHz with I2CCLK = 80MHz, PRCK=5,
            BC=000,ACK=1,NOACK=0,SCK=0 (sck) */
  /* (3) Set Expand  Function Register  DISAL=1,SA2ST=0,SAST=0,NFSEL=0,GCDI=1,SREN=0,MFACK=0 */
  /* (4) Set Control Register 2  processing request disable */
  /* (5) Set Control Register 2  MST=0,TRX=0,BB=0,PIN=0,I2CM=1,SWRES=00 */
  /* (6) Set address recognizing mode1 ALS=00 */
  /* (7) Set own address 2 address = 0x00 */
  /* (8) Set Interrupt Enable Register SELPINCD=0,DMARI2CTX=0,DMARI2CRX=0,INTNACK=1,INTI2CBF=0,INTI2CAL=0,INTI2C=1 */

  TSB_I2C1->PRS = 0x5U;                    /* (2) */
  TSB_I2C1->CR1 = (I2CxCR1_ACK | 0);      /* (2) */
  while(TSB_I2C1->SR == I2CxSR_BB )
  {
    
  }
  TSB_I2C1->OP  = I2CxOP_SLAVE_INIT;        /* (3) */
  TSB_I2C1->CR2 = (I2CxCR2_INIT | I2CxCR2_PIN_CLEAR); /* (4) */
  TSB_I2C1->CR2 = I2CxCR2_INIT;                       /* (5) */
  
  TSB_I2C1->AR  = (CFG_I2C_SELF_ADDR & ~I2CxAR_ALS);  /* (6) */
  TSB_I2C1->AR2 = I2CxAR2_INIT;                       /* (7) */
  TSB_I2C1->IE  = (I2CxIE_I2C | I2CxIE_INTNACK);      /* (8) */
  
}
/*--------------------------------------------------*/
/**
  * @brief     Generate start condition
  * @param     -
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
static void I2C_stop_condition(void)
{
    TSB_I2C1->CR2 = I2CxCR2_STOP_CONDITION;
}
/**
 *  @}
 */ /* End of group I2C_Private_functions */

/**
 *  @}
 */ /* End of group I2C_SLAVE_TRANS_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_SLAVE_TRANS_Exported_functions I2C_SLAVE_TRANS Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  uint32_t period;    /* Period for Systick */
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_LED_Config();
  GPIO_SW_Config();
  
  GPIO_I2C_Slave_Config();
  /*--- IBIMC033: Enable ---*/
  TSB_IB->IMC033 = (uint8_t)(0x05);
  
  /* Period for 10msec SYSTICK */
  SystemCoreClockUpdate();
  period = SystemCoreClock / 100;
  if (SysTick_Config(period ))    /* systick interrupt cycle setting */
  {
  }
  I2C_Slave_Config();
  /* Interrupt Enable */
  NVIC_EnableIRQ(INT00_IRQn);          /* interrupt pin 0 enable        */
  NVIC_EnableIRQ(INTI2C1_IRQn);        /* I2C ch1 interrupt enaable     */
  NVIC_EnableIRQ(INTI2C1NA_IRQn);    /* I2C ch1 nack interrupt enable */
  /*------------------------*/
  /* Normal Mode Processing */
  /*------------------------*/
  while(1){
  }
}
/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
  if(error == ERROR_I2C){
    if (counter-- == 0) 
    {
      /* The end of the counting down period, the data is
         inverted and stored in a register PJ1           */
      TSB_PJ_DATA_PJ1 ^= 1;     /* led1 blink for 1sec period */
      counter = CFG_LED_BLINK_TIME;
    }else{
        /* no processing */
    }
  }
}
/*--------------------------------------------------*/
/** 
  * @brief  INT00 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   0: Interrupt Pin0
  */
/*--------------------------------------------------*/
void INT00_IRQHandler(void)
{
    /*--- IBIMC033: INTxCLR ---*/
    TSB_IB->IMC033 = (uint8_t)(0xC5);
    /* Dummy read is need. */
    {
        __IO uint8_t imc = TSB_IB->IMC033;
    }
    NVIC_DisableIRQ(INT00_IRQn);     /* interrupt pin 0 disable */
    TSB_PJ_DATA_PJ2 = 1;             /*  Turn on led2 On PJ2 */
}
/*--------------------------------------------------*/
/** 
  * @brief  INTI2C1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   75: I2C ch1 Transmission and reception interrupt
  */
/*--------------------------------------------------*/
void INTI2C1_IRQHandler(void)
{
    if((TSB_I2C1->ST & I2CxST_NACK) != I2CxST_NACK) /* When INTI2C1NA is occurred, this handler is do nothing */
    {
      if((TSB_I2C1->SR & I2CxSR_LRB) != I2CxSR_LRB) /* check the acknowledge bit */
      {
          if(send > (sizeof(tx_data)+1))   /* send data length + start condition data */
         {
             error = ERROR_I2C;
          }else{
            /* clear transfer complete flag and fill DBR with a new char */
           TSB_I2C1->ST = I2CxST_I2C; /* Clear transfer complete flag */
            TSB_I2C1->DBR = (tx_data[send++] & I2CxDBR_DB_MASK);
         }
      }else{
         error = ERROR_I2C;
     }
   }
}
/*--------------------------------------------------*/
/** 
  * @brief  INTI2C1NA IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   79: I2C ch1 No ACK interrupt
  */
/*--------------------------------------------------*/
void INTI2C1NA_IRQHandler(void)
{
    if(send >= (sizeof(tx_data) + 1))   /* send data length + start condition data */
    {
        send = 0;
        TSB_I2C1->ST = I2CxST_I2C; /* Clear complete flag */
        TSB_PJ_DATA_PJ2 = 0;         /* Turn Off led2 on PJ2           */
        TSB_PJ_DATA_PJ0 = 1;         /* Turn On  led0 on PJ0           */
        I2C_stop_condition();
    }else{
        error = ERROR_I2C;
    }
}
/**
 *  @}
 */ /* End of group I2C_SLAVE_TRANS_Exported_functions */

/**
 *  @}
 */ /* End of group I2C_SLAVE_TRANS */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
