/** 
 *******************************************************************************
 * @file    main.c
 * @brief   LVD Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-05-10 15:07:20 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup LVD LVD Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_macro LVD Private Macro
 *  @{
 */
/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME      ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

#define LVD_VOLTAGE_26      ((uint32_t)0x00000000)          /*!< LVD voltage 2.6+-0.1V    */
#define LVD_VOLTAGE_27      ((uint32_t)0x00000010)          /*!< LVD voltage 2.7+-0.1V    */
#define LVD_VOLTAGE_28      ((uint32_t)0x00000020)          /*!< LVD voltage 2.8+-0.1V    */
#define LVD_VOLTAGE_29      ((uint32_t)0x00000030)          /*!< LVD voltage 2.9+-0.1V    */
#define LVD_VOLTAGE_38      ((uint32_t)0x00000040)          /*!< LVD voltage 3.8+-0.1V    */
#define LVD_VOLTAGE_40      ((uint32_t)0x00000050)          /*!< LVD voltage 4.0+-0.1V    */
#define LVD_VOLTAGE_42      ((uint32_t)0x00000060)          /*!< LVD voltage 4.2+-0.1V    */
#define LVD_VOLTAGE_44      ((uint32_t)0x00000070)          /*!< LVD voltage 4.4+-0.1V    */

#define LVD_SEL_INT         ((uint32_t)0x00000004)          /*!< LVD INTLVD               */
#define LVD_SEL_RESET       ((uint32_t)0x00000000)          /*!< LVD reset                */
#define LVD_OUT_ENABLE      ((uint32_t)0x00000002)          /*!< LVD output enable        */
#define LVD_OUT_DISABLE     ((uint32_t)0x00000000)          /*!< LVD output disable       */
#define LVD_ENABLE          ((uint32_t)0x00000001)          /*!< LVD enable               */
#define LVD_DISABLE         ((uint32_t)0x00000000)          /*!< LVD disable              */

#define LVD_ST_MASK         ((uint32_t)0x00000080)          /*!< LVD ST mask              */
#define LVD_UPPER           ((uint32_t)0x00000000)          /*!< LVD UPPER                */
#define LVD_LOWER           ((uint32_t)0x00000080)          /*!< LVD LOWER                */
#define LVD_UNKNOWN         ((uint32_t)0x000000FF)          /*!< LVD UNKNOWN              */

#define LVD_STATUS_CT       ((uint32_t)0x00000008)          /*!< LVD status count         */


#define LVD_SUCCESS         ((uint32_t)0x00000000)          /*!< LVD result Success       */
#define LVD_ERROR           ((uint32_t)0x00000001)          /*!< LVD result Error         */
/** 
 *  @}
 */ /* End of group LVD_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_variables LVD Private Variables
 *  @{
 */
static uint32_t counter  = CFG_LED_BLINK_TIME;    /* Blink Time Counter         */
uint32_t lvd_status_now;                          /*!< LVD status now           */
uint32_t lvd_status_old;                          /*!< LVD status old           */
uint32_t lvd_status_count = 0;                    /*!< LVD status count         */
uint32_t lvd_status = LVD_UPPER;                  /*!< LVD status               */
uint32_t lvd_led_pattern = 0;                     /*!< LVD LED pattern          */
/**
 *  @}
 */ /* End of group LVD_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_fuctions LVD Private Fuctions
 *  @{
 */
static void GPIO_Config(void);
static void init_LVD(void);
static void start_LVD(void);

/*--------------------------------------------------*/
/** 
  * @brief  GPIO_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 2 (LED2) */
  TSB_PJ_DATA_PJ2 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 2 (LED2) */
  TSB_PJ_CR_PJ2C = GPIO_ENABLE;
}

/*--------------------------------------------------*/
/**
  * @brief     Initialize the LVD object.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
static void init_LVD(void)
{
    TSB_LVD->CR &= (uint8_t)LVD_DISABLE;                 /* LVD Disable */
    TSB_LVD->CR  = (uint8_t)(LVD_VOLTAGE_40| LVD_SEL_RESET | LVD_OUT_DISABLE);              /* 4.0V RESET  Output Disable*/
}
/*--------------------------------------------------*/
/**
  * @brief     Start the LVD object.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
static void start_LVD(void)
{
    uint8_t tmp;
    tmp = TSB_LVD->CR;
    TSB_LVD->CR = (uint8_t)(tmp | LVD_ENABLE);                  /* LVD Enable */
}
/**
 *  @}
 */ /* End of group LVD_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Exported_functions LVD Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  uint32_t period = 0U;
  uint32_t res = LVD_ERROR;
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_Config();
  
  init_LVD();
  start_LVD();
  lvd_status = LVD_UNKNOWN;

  /* Period for 10msec SYSTICK */
  SystemCoreClockUpdate();
  period = SystemCoreClock / 100;
  if (SysTick_Config(period ))    /* systick interrupt cycle setting */
  {
  }

  for(;;){
      /*----------------------*/
      /* LVD Status Check            */
      /*----------------------*/
      lvd_status_now = (uint32_t)(TSB_LVD->CR & LVD_ST_MASK);

      if(lvd_status_now == lvd_status_old)
      {
          lvd_status_count++;
          if(lvd_status_count >= LVD_STATUS_CT)
          {
              if(lvd_status != lvd_status_now)
              {
                  lvd_status = lvd_status_now;
                  res = LVD_SUCCESS;
              }
              lvd_status_count = 0;
          }
      }
      else
      {
          lvd_status_old = lvd_status_now;
          lvd_status_count = 0;
      }
      if(res == LVD_SUCCESS)
      {
          if(lvd_status == LVD_UPPER)
          {
              lvd_led_pattern = 0;
          }
          else
          {
              lvd_led_pattern = 1;
          }
      }
      else
      {
          /* do nothing */
      }
  }
}
/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
    if (counter-- == 0) 
    {
      counter = CFG_LED_BLINK_TIME;
      if(lvd_led_pattern == 0){
        /* The end of the counting down period, the data is
           inverted and stored in a register PJ0           */
        TSB_PJ_DATA_PJ2 = 0;
        TSB_PJ_DATA_PJ0 = 1;     /* led0 turn on for 1sec period */
      }else{
        /* The end of the counting down period, the data is
           inverted and stored in a register PJ2           */
        TSB_PJ_DATA_PJ0 = 0;
        TSB_PJ_DATA_PJ2 ^= 1;     /* led2 blink for 1sec period */
      }
    }else{
        /* no processing */
    }
}

/**
 *  @}
 */ /* End of group LVD_Exported_functions */

/**
 *  @}
 */ /* End of group LVD */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
