/** 
 *******************************************************************************
 * @file    main.c
 * @brief   OFD Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-06-24 14:53:59 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup OFD OFD Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup OFD_Private_macro OFD Private Macro
 *  @{
 */
/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */
/** 
 *  @defgroup OFD_CFG OFD Configrations
 *  @brief    OFD Configrations.
 *  @{
 */
/*  fIHOSC2: Min   9.0MHz, TYP 10MHz, MAX  11.0MHz
    fEHOSC : Min   9.9MHz, TYP 10MHz, MAX  10.1MHz
    fc     : Min 158.4MHz, TYP160.0MHz, MAX 161.6MHz  */
#define OFD_LOWER_COUNT  ((uint32_t)0x1CCU)     /* (158.4/11)*32 = 460.8 : 460 = 0x1CC */
#define OFD_HIGHER_COUNT ((uint32_t)0x23EU)     /* (161.6/ 9)*32 = 574.5 : 574 = 0x23E */

#define OFD_LOWER_COUNT_EXTERNAL  ((uint32_t)0x01CU) /*  (9.9/11)*32 = 28.8 : 28 = 0x01C */
#define OFD_HIGHER_COUNT_EXTERNAL ((uint32_t)0x023U) /*  (10.1/9)*32 = 35.9 : 35 = 0x023 */

#define OSCCR_IHOSC2EN_MASK           ((uint8_t)0x00000008)    /*!< IHOSC2EN :[R/W]  :Mask     */
#define OSCCR_IHOSC2EN_RW_DISABLE     ((uint8_t)0x00000000)    /*!< IHOSC2EN :[R/W]  :Disable  */
#define OSCCR_IHOSC2EN_RW_ENABLE      ((uint8_t)0x00000008)    /*!< IHOSC2EN :[R/W]  :Enablc   */

#define OSCCR_IHOSC2F_MASK            ((uint8_t)0x00080000)    /*!< IHOSC2F  :[R]    :Mask     */
#define OSCCR_IHOSC2F_STOP            ((uint8_t)0x00000000)    /*!< IHOSC2F  :[R]    :Stop     */
#define OSCCR_IHOSC2F_RUNNING         ((uint8_t)0x00080000)    /*!< IHOSC2F  :[R]    :Running  */
#define RSTFLG1_OFD_CLEAR  ((uint32_t)0xFFFFFFF7)
/**
 *  @}
 */ /* End of group OFD_CFG*/
/** 
 *  @defgroup OFD_OFDCR1 OFDCR1 Register Setting
 *  @brief    OFDCR1 Register Macro Define.
 *  @{
 */
#define OFD_OFDCR1_OFDWEN_DISABLE      ((uint32_t)0x06U)     /* Regster Writing Disable */
#define OFD_OFDCR1_OFDWEN_ENABLE       ((uint32_t)0xF9U)     /* Regster Writing Enable  */
/**
 *  @}
 */ /* End of group OFD_OFDCR1*/
/** 
 *  @defgroup OFD_OFDRST OFDRST Register Setting
 *  @brief    OFDRST Register Macro Define.
 *  @{
 */
#define OFD_OFDRST_OFDRSTEN_DISABLE    ((uint32_t)0x0U)      /* OFD Ditection Disable    */
#define OFD_OFDRST_OFDRSTEN_ENABLE     ((uint32_t)0x1U)      /* OFD Ditection Enable     */
/**
 *  @}
 */ /* End of group OFD_OFDRST*/
/** 
 *  @defgroup OFD_OFDCR2 OFDCR2 Register Setting
 *  @brief    OFDCR2 Register Macro Define.
 *  @{
 */
#define OFD_OFDCR2_OFDEN_DISABLE       ((uint32_t)0x00U)     /* OFD Ditection Disable   */
#define OFD_OFDCR2_OFDEN_ENABLE        ((uint32_t)0xE4U)     /* OFD Ditection Enable    */
/**
 *  @}
 */ /* End of group OFD_OFDCR2*/
/** 
 *  @defgroup OFD_OFDMON OFDMON Register Setting
 *  @brief    OFDMON Register Macro Define.
 *  @{
 */
#define OFD_OFDMON_EOHSC              ((uint32_t)0x0U)     /* OFD Ditection Moniter for EOHSC   */
#define OFD_OFDMON_FC                 ((uint32_t)0x1U)     /* OFD Ditection Moniter for fc      */
/**
 *  @}
 */ /* End of group OFD_OFDMON*/
/** 
 *  @defgroup OFDSTAT_OFDBUSY OFDBUSY
 *  @brief    [Bit1]: OFD Running status.
 *  @{
 */
#define OFD_OFDSTAT_OFDBUSY_MASK         ((uint32_t)0x00000002)        /*!< For bit MASK.                 */
#define OFD_OFDSTAT_OFDBUSY_SUSPENDED    ((uint32_t)0x00000000)        /*!< OFD Susupended.               */
#define OFD_OFDSTAT_OFDBUSY_RUNNING      ((uint32_t)0x00000002)        /*!< OFD Running.                  */
/**
 *  @}
 */ /* End of group OFDSTAT_OFDBUSY */
/** 
 *  @defgroup OFDSTAT_FRQERR FRQERR
 *  @brief    [Bit0]: OFD Error Detection Flag.
 *  @{
 */
#define OFD_OFDSTAT_FRQERR_MASK          ((uint32_t)0x00000001)        /*!< For bit MASK.                 */
#define OFD_OFDSTAT_FRQERR_NOERROR       ((uint32_t)0x00000000)        /*!< Not Error occurred.           */
#define OFD_OFDSTAT_FRQERR_ERROR         ((uint32_t)0x00000001)        /*!< Error occurred.               */
/**
 *  @}
 */ /* End of group OFDSTAT_FRQERR */
/** 
 *  @}
 */ /* End of group OFD_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup OFD_Private_variables OFD Private Variables
 *  @{
 */
static uint32_t counter  = CFG_LED_BLINK_TIME;    /* Blink Time Counter        */
static uint32_t reset_flag = 0U;                  /* Reset Flag                */
static uint32_t err_flag = 0U;
/**
 *  @}
 */ /* End of group OFD_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup OFD_Private_fuctions OFD Private Fuctions
 *  @{
 */
static void GPIO_Config(void);
static void internal_hosc2_enable(void);
static void wait(int32_t count_p);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 2 (LED2) */
  TSB_PJ_DATA_PJ2 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 2 (LED2) */
  TSB_PJ_CR_PJ2C = GPIO_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  Internal HOSC Enable
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void internal_hosc2_enable(void)
{
    uint8_t work;

    work = (uint8_t)(TSB_CG->OSCCR & (uint8_t)~OSCCR_IHOSC2EN_MASK);
    TSB_CG->OSCCR = (uint8_t)(work | OSCCR_IHOSC2EN_RW_ENABLE);
    while ((TSB_CG->OSCCR & OSCCR_IHOSC2F_MASK) != OSCCR_IHOSC2F_RUNNING) 
    {
        /* no processing */
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Wait
  * @param  count_p
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void wait(int32_t count_p)
{
    int32_t i;
    for(i=0; i<count_p; i++){
        __NOP();
    }
}
/**
 *  @}
 */ /* End of group OFD_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup OFD_Exported_functions OFD Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    uint32_t period = 0U;
    uint32_t tmp =0;
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_Config();
  /* Enable the peripheral clock of OFD */
  TSB_CG_FSYSMENB_IPMENB14 = FSYS_ENABLE;
  /* Enable the peripheral clock of OFD fc */
  TSB_CG_FCEN_FCIPEN23  = FSYS_ENABLE;
  /* Period for 10msec SYSTICK */
  SystemCoreClockUpdate();
  period = SystemCoreClock / 100;
  if (SysTick_Config(period ))    /* systick interrupt cycle setting */
  {
  }
  /* OFD Reset flag Check */
  if((TSB_RLM->RSTFLG1 & 0x08) != 0U){
      reset_flag = 1;
      /* OFD Reset flag Clear */
      tmp = TSB_RLM->RSTFLG1;
      tmp &= RSTFLG1_OFD_CLEAR;
      TSB_RLM->RSTFLG1= (uint8_t)tmp;
  }else{
      /* IHOSC2 Enable */
       internal_hosc2_enable();
  }
  /*----------------------*/
  /* OFD                  */
  /*----------------------*/
  if(reset_flag == 0U)
  {
      /* Register Write Enable */
      TSB_OFD->CR1 = OFD_OFDCR1_OFDWEN_ENABLE;
      /* Reset Output Disable */
      TSB_OFD->RST = OFD_OFDRST_OFDRSTEN_DISABLE;
      /* EHOSC Monitor Enable */
      TSB_OFD->MON = OFD_OFDMON_EOHSC;
      /* Frequency Detection Set */
      TSB_OFD->MN0 = OFD_LOWER_COUNT_EXTERNAL;
      TSB_OFD->MX0 = OFD_HIGHER_COUNT_EXTERNAL;
      TSB_OFD->MN1 = OFD_LOWER_COUNT;
      TSB_OFD->MX1 = OFD_HIGHER_COUNT;
      /* Frequency Detection Enable */
      TSB_OFD->CR2 = OFD_OFDCR2_OFDEN_ENABLE;
      /* OFD Status Register Check */
      while((TSB_OFD->STAT & OFD_OFDSTAT_OFDBUSY_MASK) != OFD_OFDSTAT_OFDBUSY_RUNNING)
      {

      }
      wait(1500);
      if((TSB_OFD->STAT & OFD_OFDSTAT_FRQERR_MASK) == OFD_OFDSTAT_FRQERR_NOERROR)
      {
          err_flag = 0;
          /* Frequency Detection Disable */
          TSB_OFD->CR2 = OFD_OFDCR2_OFDEN_DISABLE;
          /* Reset Output Enable */
          TSB_OFD->RST = OFD_OFDRST_OFDRSTEN_ENABLE;
          /* Frequency Detection Enable */
          TSB_OFD->CR2 = OFD_OFDCR2_OFDEN_ENABLE;
          /* Register Write Disable */
          TSB_OFD->CR1 = OFD_OFDCR1_OFDWEN_DISABLE;             
       }else{
          err_flag = 1;
          /* Frequency Detection Disable */
          TSB_OFD->CR2 = OFD_OFDCR2_OFDEN_DISABLE;
          /* Reset Output Enable */
          TSB_OFD->RST = OFD_OFDRST_OFDRSTEN_ENABLE;
          /* Frequency Detection Enable */
          TSB_OFD->CR2 = OFD_OFDCR2_OFDEN_ENABLE;
           /* ----------------- */
           /* OFD Reset Occurre */
           /* ----------------- */
      }  
  }else{
      /* OFD Reset Occurred */
      err_flag = 1;
      reset_flag = 0U;
  }
  while(1)
  {
  }
}
/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
    if (counter-- == 0) 
    {
      counter = CFG_LED_BLINK_TIME;
      if(err_flag == 0){
        /* The end of the counting down period, the data is
           inverted and stored in a register PJ0           */
        TSB_PJ_DATA_PJ2 = 0;
        TSB_PJ_DATA_PJ0 ^= 1;     /* led0 blink for 1sec period */
      }else{
        /* The end of the counting down period, the data is
           inverted and stored in a register PJ2           */
        TSB_PJ_DATA_PJ0 = 0;
        TSB_PJ_DATA_PJ2 ^= 1;     /* led2 blink for 1sec period */
      }
    }else{
        /* no processing */
    }
}

/**
 *  @}
 */ /* End of group OFD_Exported_functions */

/**
 *  @}
 */ /* End of group OFD */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
