/** 
 *******************************************************************************
 * @file    main.c
 * @brief   PORT Sample Application.
 * @version V1.0.0.1
 * $Date:: 2019-05-10 14:46:32 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup PORT PORT Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup PORT_Private_macro PORT Private Macro
 *  @{
 */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

/** 
 *  @}
 */ /* End of group PORT_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup PORT_Private_fuctions PORT Private Fuctions
 *  @{
 */
static void GPIO_Config(void);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 1 (LED1) */
  TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 1 (LED1) */
  TSB_PJ_CR_PJ1C = GPIO_ENABLE;
  /* (6) Enable the peripheral clock of PORT E */
  TSB_CG_FSYSMENA_IPMENA04 = FSYS_ENABLE;
  /* (7) Select input mode on PORT E pin 5 */
  TSB_PE_IE_PE5IE = GPIO_ENABLE;
  /* (8) Enable the peripheral clock of PORT A */
  TSB_CG_FSYSMENA_IPMENA00 = FSYS_ENABLE;
  /* (9) Select output mode on PORT A pin 0 (SWS) */
  TSB_PA_CR_PA0C = GPIO_ENABLE;
  TSB_PA_DATA_PA0 = GPIO_ENABLE;   /*SW select 1:SW7-4 */
  /* (10) Select input mode on PORT A pin 1 (SW5) */
  TSB_PA_IE_PA1IE = GPIO_ENABLE;
}

/**
 *  @}
 */ /* End of group PORT_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup PORT_Exported_functions PORT Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_Config();
  /*--- INT05a(IMC042): Enable ---*/
  TSB_IB->IMC042 = (uint8_t)(0x07);
  /*--- INT11b(IMC053): Enable ---*/
  TSB_IB->IMC053 = (uint8_t)(0x07);     
  /* Interrupt Enable */
  NVIC_EnableIRQ(INT05_IRQn);     /* Interrupt pin 05 enable */
  NVIC_EnableIRQ(INT11_IRQn);     /* Interrupt pin 11 enable */
  while(1)
  {
  }
}
/*--------------------------------------------------*/
/** 
  * @brief  INT05 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   5: Interrupt pin 05a/05b
  */
/*--------------------------------------------------*/
void INT05_IRQHandler(void)
{
    /*--- IMC042: INTxCLR ---*/
    TSB_IB->IMC042 = (uint8_t)(0xC5);
    /* Dummy read is need. */
    {
        __IO uint8_t imc = TSB_IB->IMC042;
    }
    TSB_PJ_DATA_PJ0 ^= 1;                  /* Toggle led0 PJ0            */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT11 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   11: Interrupt pin 11a/11b
  */
/*--------------------------------------------------*/
void INT11_IRQHandler(void)
{
    /*--- IMC053: INTxCLR ---*/
    TSB_IB->IMC053 = (uint8_t)(0xC5);
    /* Dummy read is need. */
    {
        __IO uint8_t imc = TSB_IB->IMC053;
    }
    TSB_PJ_DATA_PJ1 ^= 1;                  /* Toggle led1 PJ1            */
}
/**
 *  @}
 */ /* End of group PORT_Exported_functions */

/**
 *  @}
 */ /* End of group PORT */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
