/** 
 *******************************************************************************
 * @file    main.c
 * @brief   SIWDT Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-05-10 10:56:19 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup SIWDT SIWDT Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIWDT_Private_macro SIWDT Private Macro
 *  @{
 */

// #define DEMO_CLEAR  /* When you comment out this line,  NMI interrupt is occurred by WDT */ 

/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(20))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */

/** 
 *  @defgroup ProcModeConfiguration Processing Mode Configuration
 *  @brief    Processing Mode Normal Start.
 *  @{
 */
#define CFG_PROC_DEFAULT               ((uint32_t)(0))
#define CFG_PROC_NORMAL_START          ((uint32_t)(1))
/** 
 *  @}
 */ /* End of group ProcModeConfiguration */

/** 
 *  @defgroup  SIWDxEN  SIWDxEN Register
 *  @brief     WD Enable Register.
 *  @{
 */
/* WDTE */
#define SIWDxEN_WDTE_RW_DISABLE         ((uint32_t)0x00000000)    /*!< WDTE  :[R/W] :Disable    */
#define SIWDxEN_WDTE_RW_ENABLE          ((uint32_t)0x00000001)    /*!< WDTE  :[R/W] :Enable     */
/* WDCR */
#define SIWDxCR_WDCR_W_DISABLE          ((uint32_t)0x000000B1)    /*!< WDCR  :[W] :Disable      */
#define SIWDxCR_WDCR_W_CLEAR            ((uint32_t)0x0000004E)    /*!< WDCR  :[W] :Clear        */
/** 
 *  @}
 */ /* End of group SIWDxEN */
 
/** 
 *  @defgroup  SIWDxMOD  SIWDxMOD Register
 *  @brief     WD Mode Register.
 *  @{
 */
/* WDTE */
#define SIWDxMOD_INIT                   ((uint32_t)0x00000500)    /*!< WDCLS=00,WDTP=101,WDCWD=00,INTF=0,RESCR=0    */
/** 
 *  @}
 */ /* End of group SIWDxMOD */
 /** 
 *  @defgroup  IMNFLGNMI IMNFLGNMI Register
 *  @brief     NMI Monitor Register.
 *  @{
 */
#define IMNFLGNMI_INT016FLG             ((uint32_t)0x00010000)    /*!< INT016FLG :[R] :INTWDT0      */ 
#define IBNIC00_INTPFLG                 ((uint32_t)0x00000010)    /*!< INTPFLG   :[R] :INTWDT0      */ 
#define IMNFLGNMI_INT000FLG             ((uint32_t)0x00000001)    /*!< INT000FLG :[R] :INTLVD      */     
/** 
 *  @}
 */ /* End of group IMNFLGNMI */
 /** 
 *  @defgroup  FCSR0  FCSR0 Register
 *  @brief     Flash status Register 0.
 *  @{
 */
/*RDYBSY */
#define FCSR0_RDYBSY_R_BUSY             ((uint32_t)0x00000000)    /*!< WRDYBSY :[R] :Busy      */
#define FCSR0_RDYBSY_R_READY            ((uint32_t)0x00000001)    /*!< WRDYBSY :[R] :Ready     */
/** 
 *  @}
 */ /* End of group FCSR0 */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

/** 
 *  @}
 */ /* End of group SIWDT_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIWDT_Private_variables SIWDT Private Variables
 *  @{
 */
static uint32_t counter  = CFG_LED_BLINK_TIME;    /* Blink Time Counter        */
/**
 *  @}
 */ /* End of group SIWDT_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIWDT_Private_fuctions SIWDT Private Fuctions
 *  @{
 */
static void GPIO_Config(void);
static void WDT_Config(void);
static void wdt_disable(void);
static void wdt_enable(void);
void Delay(uint32_t time);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 1 (LED1) */
  TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 1 (LED1) */
  TSB_PJ_CR_PJ1C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 2 (LED2) */
  TSB_PJ_DATA_PJ2 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 2 (LED2) */
  TSB_PJ_CR_PJ2C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 3 (LED3) */
  TSB_PJ_DATA_PJ3 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 3 (LED3) */
  TSB_PJ_CR_PJ3C = GPIO_ENABLE;

}

/*--------------------------------------------------*/
/** 
  * @brief  WDT_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void WDT_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_SIWD0->MOD = SIWDxMOD_INIT;
}

/*--------------------------------------------------*/
/** 
  * @brief  WDT Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void wdt_disable(void)
{
    TSB_SIWD0->EN = SIWDxEN_WDTE_RW_DISABLE;
    TSB_SIWD0->CR = SIWDxCR_WDCR_W_DISABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  WDT Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void wdt_enable(void)
{
    TSB_SIWD0->EN = SIWDxEN_WDTE_RW_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  Delay
  * @param  time : loop times (time * 800).
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void Delay(uint32_t time)
{
    volatile uint32_t i = 0U;
    for (i = 0U; i < (time * 800U); ++i);
}

/**
 *  @}
 */ /* End of group SIWDT_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIWDT_Exported_functions SIWDT Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  uint32_t period;    /* Period for Systick */
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_Config();
  /*----------------------*/
  /* WDT Disable          */
  /*----------------------*/
  wdt_disable();
  WDT_Config();
  /*------------------------*/
  /* Normal Mode Processing */
  /*------------------------*/
    /* Period for 10msec SYSTICK */
    SystemCoreClockUpdate();
    period = SystemCoreClock / 100;
    if (SysTick_Config(period ))    /* systick interrupt cycle setting */
    {
    }
   /*----------------------*/
  /* WDT Enable          */
  /*----------------------*/   
    wdt_enable();

    while (1) {
#ifdef DEMO_CLEAR
        Delay(500U);
        TSB_SIWD0->CR = SIWDxCR_WDCR_W_CLEAR;
#endif
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
    if (counter-- == 0) 
    {
      /* The end of the counting down period, the data is
         inverted and stored in a register PJ0- PJ3           */
      TSB_PJ_DATA_PJ0 ^= 1;     /* led0 blink for 1sec period */
      TSB_PJ_DATA_PJ1 ^= 1;     /* led1 blink for 1sec period */
      TSB_PJ_DATA_PJ2 ^= 1;     /* led2 blink for 1sec period */
      TSB_PJ_DATA_PJ3 ^= 1;     /* led3 blink for 1sec period */
      counter = CFG_LED_BLINK_TIME;
    }else{
        /* no processing */
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  NMI Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void NMI_Handler(void)
{
  uint32_t flag;
  flag = TSB_IMN->FLGNMI;
  if((flag & IMNFLGNMI_INT016FLG) == IMNFLGNMI_INT016FLG)
  {
    wdt_disable();
    /*--- IBNIC00: INTPCLR ---*/
    TSB_IB->NIC00 = 0x40;
    /* Dummy read is need. */
    {
        __IO uint8_t nic = TSB_IB->NIC00;
    }
    /* SysTick Disable */
    SysTick->CTRL = 0;
    /* LED turn off */
    TSB_PJ_DATA_PJ0 = 0;     /* led0 turn off */
    TSB_PJ_DATA_PJ1 = 0;     /* led1 turn off */
    TSB_PJ_DATA_PJ2 = 0;     /* led2 turn off */
    TSB_PJ_DATA_PJ3 = 0;     /* led3 turn off */
  }
}
/**
 *  @}
 */ /* End of group SIWDT_Exported_functions */

/**
 *  @}
 */ /* End of group SIWDT */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
