/** 
 *******************************************************************************
 * @file    main.c
 * @brief   T32A_PPG Sample Application.
 * @version V1.0.3.0
 * $Date:: 2019-06-28 09:19:54 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup T32A_PPG T32A_PPG Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup T32A_PPG_Private_macro T32A_PPG Private Macro
 *  @{
 */

/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */
/** 
 *  @name  T32AxMOD Macro Definition.
 *  @brief T32AxMOD Register Macro Definition.
 *  @{
 */
#define T32AxMOD_PPG_INIT       ((uint32_t)0x00000000)     /*!< HALT=0,MODE32=0 */
/**
 *  @}
 */ /* End of name T32AxMOD Macro Definition */
/** 
 *  @name  T32AxCRA Macro Definition.
 *  @brief T32AxCRA Register Macro Definition.
 *  @{
 */
#define T32AxCRA_PPG_INIT       ((uint32_t)0x10100700)     /*!< PRSCLA=001,CLKA=000,WBFA=1,UPDNA=00,RELDA=111,STOPA=000,STARTA=000 */
/**
 *  @}
 */ /* End of name T32AxCRA Macro Definition */
/** 
 *  @name  T32AxIMA Macro Definition.
 *  @brief T32AxIMA Register Macro Definition.
 *  @{
 */
#define T32AxIMA_PPG_INIT    ((uint32_t)0x00000000)     /*!< IMUFA=0,IMOFA=0,IMA1=0,IMA0=0 */
/**
 *  @}
 */ /* End of name T32AxIMA Macro Definition */
 /** 
 *  @name  T32AxOUTCRA0 Macro Definition.
 *  @brief T32AxOUTCRA0 Register Macro Definition.
 *  @{
 */
#define T32AxOUTCRA0_PPG_INIT    ((uint32_t)0x00000000)     /*!< OCRA=0 */
/**
 *  @}
 */ /* End of name T32AxOUTCRA0 Macro Definition */
 /** 
 *  @name  T32AxOUTCRA1 Macro Definition.
 *  @brief T32AxOUTCRA1 Register Macro Definition.
 *  @{
 */
#define T32AxOUTCRA1_PPG_INIT    ((uint32_t)0x00000009)     /*!< OCRCAPA1=00,OCRCAPA0=00,OCRCMPA1=10,OCRCMPA0=01  */
/**
 *  @}
 */ /* End of name T32AxOUTCRA1 Macro Definition */
/** 
 *  @name  T32AxRUNA Macro Definition.
 *  @brief T32AxRUNA Register Macro Definition.
 *  @{
 */
#define T32AxRUNA_UPCOUNT_START    ((uint32_t)0x00000003)     /*!< SFTSTPA=0,SFTSTAA=1, RUNA=1 */
#define T32AxRUNA_UPCOUNT_STOP     ((uint32_t)0x00000000)     /*!< SFTSTPA=0,SFTSTAA=1, RUNA=1 */
#define T32AxRUNA_RUNFLGA_MASK     ((uint32_t)0x00000010)     /*!< Mask for RUNFLGA */
/**
 *  @}
 */ /* End of name T32AxRUNA Macro Definition */
 
/** 
 *  @defgroup  T32AStartError T32A Start Error 
 *  @brief    T32A Start Error  .
 *  @{
 */
#define ERROR_T32A_START      ((uint32_t)(1))
/** 
 *  @}
 */ /* End of group T32AStartError */
 /** 
 *  @defgroup  T32ACounterError T32A Start Error 
 *  @brief    T32A Counter Error  .
 *  @{
 */
#define ERROR_T32A_COUNTER      ((uint32_t)(2))
/** 
 *  @}
 */ /* End of group T32ACounterError */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

/** 
 *  @}
 */ /* End of group T32A_PPG_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup T32A_PPG_Private_variables T32A_PPG Private Variables
 *  @{
 */
static uint32_t counter  = CFG_LED_BLINK_TIME;    /* Blink Time Counter        */
static uint32_t error    = 0;                     /* Error Flag         */
static uint32_t RisingTimingus[5U] = {0x7D0U, 0x1388U, 0x2710U, 0x3A98U, 0x4650U}; /* Low Time  10%, 25%, 50%, 75%, 90% */
static uint32_t ppg_count    = 0;                     /* For Timer Counter         */
/**
 *  @}
 */ /* End of group T32A_PPG_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup T32A_PPG_Private_fuctions T32A_PPG Private Fuctions
 *  @{
 */
static void GPIO_Config(void);
static void GPIO_T32A2_Configure(void);
void t32a_Calculator(uint32_t *p_value, uint32_t time, uint32_t prescaler, uint32_t prscl) ;
static void T32A2_Config(void);
static void T32A2_Start(void);
static void T32A2_Stop(void);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 1 (LED1) */
  TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 1 (LED1) */
  TSB_PJ_CR_PJ1C = GPIO_ENABLE;
  /* (6) Turn off PORT J pin 2 (LED2) */
  TSB_PJ_DATA_PJ2 = GPIO_DISABLE;
  /* (7) Select output mode on PORT J pin 2 (LED2) */
  TSB_PJ_CR_PJ2C = GPIO_ENABLE;
  /* (8) Turn off PORT J pin 3 (LED3) */
  TSB_PJ_DATA_PJ3 = GPIO_DISABLE;
  /* (9) Select output mode on PORT J pin 3 (LED3) */
  TSB_PJ_CR_PJ3C = GPIO_ENABLE;
  /* (10) Enable the peripheral clock of PORT A */
  TSB_CG_FSYSMENA_IPMENA00 = FSYS_ENABLE;
  /* (11) Select output mode on PORT A pin 0 (SWS) */
  TSB_PA_CR_PA0C = GPIO_ENABLE;
  TSB_PA_DATA_PA0 = GPIO_ENABLE;   /*SW select 1:SW7-4 */
  /* (12) Select input mode on PORT A pin 2 (SW4) */
  TSB_PA_IE_PA2IE = GPIO_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  GPIO_T32A2_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_T32A2_Configure(void)
{
  /* (1) Enable the peripheral clock of GPIO U */
  TSB_CG_FSYSMENA_IPMENA16 = FSYS_ENABLE;
  /* (2) Select function mode (T32A02OUTA) on PORT U pin 2    */
  TSB_PU_FR4_PU2F4 = GPIO_ENABLE;
  /* (3) Select output mode on PORT U pin 2    */
  TSB_PU_CR_PU2C = GPIO_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  T32A2_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void T32A2_Config(void)
{
  /* (1) Enable the peripheral clock of T32A02 */
  /* (2) Setting 16bit Timer Mode,
         HALT=0,MODE32=0                                                    */
  /* (3) Setting prescaler 1/2, counter relead  T32AxRGA0 compare match,
         PRSCLA=001,CLKA=000,WBFA=0,UPDNA=00,RELDA=111,STOPA=000,STARTA=000 */
  /* (4) Setting Reload Register to 0   */
  /* (5) Setting T32AxRGA1 for one cycle,
         Clock fsys 160.00MHz, Clock pht0mem 80.000MHz, prescaler 1/2, period 500us=2000Hz
         (80000000 / 2) / 2000   = 20000.00 => 20000    (0x4E20)  */
  /* (6) T32AxRGA1 compare match detection interrupt is enabled, 
         IMUFA=0,IMOFA=0,IMA1=0,IMA0=0                                     */
  /* (7) Setting T32AxOUTCRA0 to no change, 
         OCRA=00                                                           */
  /* (8) Setting T32AxOUTCRA1 to T32AxRGA1 compare match -> High, T32AxRGA0 compare match -> Low
         OCRCAPA1=00,OCRCAPA0=00,OCRCMPA1=10,OCRCMPA0=01                   */
  TSB_CG_FSYSMENA_IPMENA30 = FSYS_ENABLE;       /* (1) */
  TSB_T32A2->MOD   = T32AxMOD_PPG_INIT;         /* (2) */
  TSB_T32A2->CRA   = T32AxCRA_PPG_INIT;         /* (3) */
  TSB_T32A2->RELDA = 0x0;                       /* (4) */
  TSB_T32A2->RGA1  = 0x4E20;                    /* (5) */
  TSB_T32A2->IMA   = T32AxIMA_PPG_INIT;         /* (6) */
  TSB_T32A2->OUTCRA0 = T32AxOUTCRA0_PPG_INIT;   /* (7) */
  TSB_T32A2->OUTCRA1 = T32AxOUTCRA1_PPG_INIT;   /* (8) */
}
/*--------------------------------------------------*/
/** 
  * @brief  T32A2_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void T32A2_Start(void)
{
    /* Cehck the Timer is not running */
    if((TSB_T32A2->RUNA & T32AxRUNA_RUNFLGA_MASK) != T32AxRUNA_RUNFLGA_MASK)
    {
        /* TimerA Count Start SFTSTPA=0,SFTSTAA=1,RUNA=1 */
        TSB_T32A2->RUNA = T32AxRUNA_UPCOUNT_START;
    }else{
        /* set error */
        error = ERROR_T32A_START;
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  T32A2_Stop
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void T32A2_Stop(void)
{
    /* Timer Stop SFTSTPA=0,SFTSTAA=0,RUNA=0 */
    TSB_T32A2->RUNA = T32AxRUNA_UPCOUNT_STOP;
}
    
/**
 *  @}
 */ /* End of group T32A_PPG_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup T32A_PPG_Exported_functions T32A_PPG Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  uint32_t period;    /* Period for Systick */
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_Config();
  GPIO_T32A2_Configure();
  
  /*--- IBIMC033: Enable ---*/
  TSB_IB->IMC033 = (uint8_t)(0x05);
  
  /* Period for 10msec SYSTICK */
  SystemCoreClockUpdate();
  period = SystemCoreClock / 100;
  if (SysTick_Config(period ))    /* systick interrupt cycle setting */
  {
  }

  /*------------------------*/
  /* T32A CountUp Setting   */
  /*------------------------*/ 
  T32A2_Config();
  /* Interrupt Enable */
  NVIC_EnableIRQ(INT00_IRQn);     /* Interrupt pin 00 enable */
  /*------------------------*/
  /* Normal Mode Processing */
  /*------------------------*/
  while(1){

  }
}

/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
    if(error != 0)
    {
      if (counter-- == 0) 
      {
        /* The end of the counting down period, the data is
          inverted and stored in a register PJ0           */
       TSB_PJ_DATA_PJ0 ^= 1;     /* led0 blink for 1sec period */
       counter = CFG_LED_BLINK_TIME;
      }else{
          /* no processing */
      }
  }
}
/*--------------------------------------------------*/
/** 
  * @brief  INT00 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   0: Interrupt Pin0
  */
/*--------------------------------------------------*/
void INT00_IRQHandler(void)
{

    /*--- IBIMC033: INTxCLR ---*/
    TSB_IB->IMC033 = (uint8_t)(0xC5);
    /* Dummy read is need. */
    {
        __IO uint8_t imc = TSB_IB->IMC033;
    }
    /* Timer Stop */
    T32A2_Stop();
    /* Every INT00 occurs, the low time cycle is changed.                    */
    /* Low time cycle: 50us -> 125us -> 250us -> 375us -> 450us -> Return 50us */
    TSB_T32A2->RGA0 = RisingTimingus[ppg_count];
    ppg_count++;
    if(ppg_count==5)
    {
        ppg_count=0;
    }
    /* Timer Start */
    T32A2_Start();
}

/**
 *  @}
 */ /* End of group T32A_PPG_Exported_functions */

/**
 *  @}
 */ /* End of group T32A_PPG */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
