/** 
 *******************************************************************************
 * @file    main.c
 * @brief   T32A_UpCount Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-05-10 9:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup T32A_UpCount T32A_UpCount Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup T32A_UpCount_Private_macro T32A_UpCount Private Macro
 *  @{
 */

/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */
/** 
 *  @name  T32AxMOD Macro Definition.
 *  @brief T32AxMOD Register Macro Definition.
 *  @{
 */
#define T32AxMOD_UPCOUNT_INIT       ((uint32_t)0x00000000)     /*!< HALT=0,MODE32=0 */
/**
 *  @}
 */ /* End of name T32AxMOD Macro Definition */
/** 
 *  @name  T32AxCRA Macro Definition.
 *  @brief T32AxCRA Register Macro Definition.
 *  @{
 */
#define T32AxCRA_UPCOUNT_INIT       ((uint32_t)0x10000700)     /*!< PRSCLA=001,CLKA=000,WBFA=0,UPDNA=00,RELDA=111,STOPA=000,STARTA=000 */
/**
 *  @}
 */ /* End of name T32AxCRA Macro Definition */
/** 
 *  @name  T32AxIMA Macro Definition.
 *  @brief T32AxIMA Register Macro Definition.
 *  @{
 */
#define T32AxIMA_UPCOUNT_INIT    ((uint32_t)0x0000000D)     /*!< IMUFA=1,IMOFA=1, IMA1=0,IMA0=1 */
/**
 *  @}
 */ /* End of name T32AxIMA Macro Definition */
/** 
 *  @name  T32AxRUNA Macro Definition.
 *  @brief T32AxRUNA Register Macro Definition.
 *  @{
 */
#define T32AxRUNA_UPCOUNT_START    ((uint32_t)0x00000003)     /*!< SFTSTPA=0,SFTSTAA=1, RUNA=1 */
#define T32AxRUNA_UPCOUNT_STOP     ((uint32_t)0x00000003)     /*!< SFTSTPA=0,SFTSTAA=1, RUNA=1 */
#define T32AxRUNA_RUNFLGA_MASK     ((uint32_t)0x00000010)     /*!< Mask for RUNFLGA */
/**
 *  @}
 */ /* End of name T32AxRUNA Macro Definition */
 
/** 
 *  @defgroup  T32AStartError T32A Start Error 
 *  @brief    T32A Start Error  .
 *  @{
 */
#define ERROR_T32A_START      ((uint32_t)(1))
/** 
 *  @}
 */ /* End of group T32AStartError */
 /** 
 *  @defgroup  T32ACounterError T32A Start Error 
 *  @brief    T32A Counter Error  .
 *  @{
 */
#define ERROR_T32A_COUNTER      ((uint32_t)(2))
/** 
 *  @}
 */ /* End of group T32ACounterError */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

/** 
 *  @}
 */ /* End of group T32A_UpCount_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup T32A_UpCount_Private_variables T32A_UpCount Private Variables
 *  @{
 */
static uint32_t counter  = CFG_LED_BLINK_TIME;    /* Blink Time Counter        */
static uint32_t count    = 0;                     /* For Timer Counter         */
static uint32_t error    = 0;                     /* Error Flag         */
static uint32_t on = 0;                            /* Led flag */
/**
 *  @}
 */ /* End of group T32A_UpCount_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup T32A_UpCount_Private_fuctions T32A_UpCount Private Fuctions
 *  @{
 */
static void GPIO_Config(void);
static void GPIO_T32A2_Configure(void);
static void T32A2_Config(void);
static void T32A2_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 1 (LED1) */
  TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 1 (LED1) */
  TSB_PJ_CR_PJ1C = GPIO_ENABLE;
  /* (6) Turn off PORT J pin 2 (LED2) */
  TSB_PJ_DATA_PJ2 = GPIO_DISABLE;
  /* (7) Select output mode on PORT J pin 2 (LED2) */
  TSB_PJ_CR_PJ2C = GPIO_ENABLE;
  /* (8) Turn off PORT J pin 3 (LED3) */
  TSB_PJ_DATA_PJ3 = GPIO_DISABLE;
  /* (9) Select output mode on PORT J pin 3 (LED3) */
  TSB_PJ_CR_PJ3C = GPIO_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  GPIO_T32A2_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_T32A2_Configure(void)
{
  /* (1) Enable the peripheral clock of GPIO U */
  TSB_CG_FSYSMENA_IPMENA16 = FSYS_ENABLE;
  /* (2) Select function mode (T32A02OUTA) on PORT U pin 2    */
  TSB_PU_FR4_PU2F4 = GPIO_ENABLE;
  /* (3) Select output mode on PORT U pin 2    */
  TSB_PU_CR_PU2C = GPIO_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  T32A2_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void T32A2_Config(void)
{
  /* (1) Enable the peripheral clock of T32A02 */
  /* (2) Setting 16bit Timer Mode,
         HALT=0,MODE32=0                                                    */
  /* (3) Setting prescaler 1/2, counter relead  T32AxRGA0 compare match,
         PRSCLA=001,CLKA=000,WBFA=0,UPDNA=00,RELDA=111,STOPA=000,STARTA=000 */
  /* (4) Clear Timer Register 0                                             */
  /* (5) Clear Timer Register 1                                             */
  /* (6) Setting Reload Register,
         Clock fsys 160.00Hz, Clock pht0mem 80.000MHz, prescaler 1/2, period 1000us=1000Hz
         (80000000 / 2) / 1000   = 40000.0 => 40000    (0x9C40)  
         0xFFFF - 0x9C04 =  0x63BF */
  /* (7) T32AxRGA1 compare match detection interrupt is not MASK, 
         IMUFA=1,IMOFA=1, IMA1=0,IMA0=1                                     */
  TSB_CG_FSYSMENA_IPMENA30 = FSYS_ENABLE;   /* (1) */
  TSB_T32A2->STA = 0xF;
  TSB_T32A2->MOD  = T32AxMOD_UPCOUNT_INIT;  /* (2) */
  TSB_T32A2->CRA  = T32AxCRA_UPCOUNT_INIT;  /* (3) */
  TSB_T32A2->RGA0 = 0;                      /* (4) */
  TSB_T32A2->RGA1 = 0x9C40;                 /* (5) */
  TSB_T32A2->RELDA = 0x0;                   /* (6) */
  TSB_T32A2->IMA  = T32AxIMA_UPCOUNT_INIT;  /* (7) */
}
/*--------------------------------------------------*/
/** 
  * @brief  T32A2_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void T32A2_Start(void)
{
    /* Cehck the Timer is not running */
    if((TSB_T32A2->RUNA & T32AxRUNA_RUNFLGA_MASK) != T32AxRUNA_RUNFLGA_MASK)
    {
        /* TimerA Count Start SFTSTPA=0,SFTSTAA=1,RUNA=1 */
        TSB_T32A2->RUNA = T32AxRUNA_UPCOUNT_START;
    }else{
        /* set error */
        error = ERROR_T32A_START;
    }
}
    
/**
 *  @}
 */ /* End of group T32A_UpCount_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup T32A_UpCount_Exported_functions T32A_UpCount Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  uint32_t period;    /* Period for Systick */
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_Config();
  GPIO_T32A2_Configure();
  
  /* Period for 10msec SYSTICK */
  SystemCoreClockUpdate();
  period = SystemCoreClock / 100;
  if (SysTick_Config(period ))    /* systick interrupt cycle setting */
  {
  }
  
  /*------------------------*/
  /* T32A CountUp Setting   */
  /*------------------------*/ 
  T32A2_Config();
  /* Interrupt Enable */
  NVIC_EnableIRQ(INTT32A02AC_IRQn);   /* T32A ch2 TimmerA compare match detection */
  /* Timer Start */
  T32A2_Start();
  /*------------------------*/
  /* Normal Mode Processing */
  /*------------------------*/
  while(1){

  }
}

/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
    if(error != 0)
    {
      if (counter-- == 0) 
      {
        /* The end of the counting down period, the data is
          inverted and stored in a register PJ0           */
       TSB_PJ_DATA_PJ0 ^= 1;     /* led0 blink for 1sec period */
       counter = CFG_LED_BLINK_TIME;
      }else{
          /* no processing */
      }
  }
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A02AC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   95: T32A ch2 TimmerA/C compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A02AC_IRQHandler(void)
{
  if((TSB_T32A2->STA & 0x2) == 0x2)
  {
    TSB_T32A2->STA = 0x3;
    if(count < 1000) {   /* 1ms * 1000 = 1sec LED on */
        count++;
        on = 1;
    }else if((count >= 1000) && (count < 2000)){  /* 1ms * 1000 = 1sec LED off */
        count++;
        on = 0;
    }else{
            count = 0;
    }
    if(on == 1){
        TSB_PJ_DATA_PJ0 = GPIO_ENABLE;
        TSB_PJ_DATA_PJ1 = GPIO_ENABLE;
        TSB_PJ_DATA_PJ2 = GPIO_ENABLE;
        TSB_PJ_DATA_PJ3 = GPIO_ENABLE;
    }else{
        TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
        TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
        TSB_PJ_DATA_PJ2 = GPIO_DISABLE;
        TSB_PJ_DATA_PJ3 = GPIO_DISABLE;
    }
  }

}
/**
 *  @}
 */ /* End of group T32A_UpCount_Exported_functions */

/**
 *  @}
 */ /* End of group T32A_UpCount */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
