/** 
 *******************************************************************************
 * @file    trm.h
 * @brief   This file provides all the functions prototypes for TRM.
 * @version V1.0.0.0
 * $Date:: 2019-05-10 11:52:33 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __TRM_H
#define __TRM_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_sample_def.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup TRMOSC
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRMOSC_Private_define
 *  @{
 */

/** 
 *  @defgroup TRMOSC_Control  Trimming Control
 *  @brief    Trimming Control.
 *  @{
 */
#define  TRMOSC_CONTROL_ENABLE      (0x00000001UL)      /*!< Trimming Control enable   */
#define  TRMOSC_CONTROL_DISABLE     (0x00000000UL)      /*!< Trimming Control disable  */
/**
 *  @}
 */ /* End of group TRMOSC_Control */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */

/*----------------------------------*/
/** 
 * @brief  TRM handle structure definenition.
*/
/*----------------------------------*/
typedef struct {
    uint32_t TrimmingControl;       /*!< Value to TRIMEN   */
    uint32_t CoarseTrimmingValue;   /*!< Value to TRIMSETC */
    uint32_t FineTrimmingValue;     /*!< Value to TRIMSETF */
} TRM_ControlTypeDef;

/**
 *  @}
 */ /* End of group TRMOSC_Private_define */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRMOSC_Private_functions
 *  @{
 */

void SetTrmOsc(TRM_ControlTypeDef *ControlStruct);
uint32_t GetTrimmingControlValue(void);
uint32_t GetInitCoarseTrimValue(void);
uint32_t GetInitFineTrimValue(void);
uint32_t GetCoarseTrimValueSet(void);
uint32_t GetFineTrimValueSet(void);

/** 
 *  @}
 */ /* End of group TRMOSC_Private_functions */

/**
 *  @}
 */ /* End of group TRMOSC */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __TRM_H */

