/**
 *******************************************************************************
 * @file    trm_t32a.c
 * @brief   This file provides API functions for Periph driver interface.
 * @version V1.0.2.0
 * $Date:: 2019-05-10 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "trm_t32a.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TRMOSC TRMOSC Sample Appli
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_variables TRMOSC Private Variables
 *  @{
 */

static t32a_t   timer_sout;                /*!< Instance       :Timer Driver  */
static t32a_t   timer_refin;               /*!< Instance       :Timer Driver  */
/**
 *  @}
 */ /* End of group TRMOSC_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Exported_functions TRMOSC Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  Initialize T32A Signal Output Function
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void InitT32A_Sout(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Signal Output Timer         */
    /*------------------------------*/
    /*--- Construct (Register Allocation) ---*/
    /* Register Allocation */
    timer_sout.p_instance = BSP_T32A_SOUT;
    /*--- Initialize: Timer   ---*/
    /* Mode             */
    timer_sout.init_mode.mode.halt = T32A_DBG_HALT_RUN;
    timer_sout.init_mode.mode.mode = T32A_MODE_16;
    if (t32a_mode_init(&timer_sout) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*--- Initialize: Timer A ---*/
    {
        t32a_initial_setting_t *p_init = &timer_sout.init[T32A_TIMERA];

        /* Run Control      */
        p_init->runx.sftstp        = T32A_COUNT_STOP;
        p_init->runx.sftsta        = T32A_COUNT_DONT_START;
        p_init->runx.run           = T32A_RUN_ENABLE;
        /* Counter Control  */
        p_init->crx.prscl          = T32A_PRSCLx_1;
        p_init->crx.clk            = T32A_CLKx_PRSCLx;
        p_init->crx.wbf            = T32A_WBF_DISABLE;
        p_init->crx.updn           = T32A_COUNT_UP;
        p_init->crx.reld           = T32A_RELOAD_TREGx;
        p_init->crx.stop           = T32A_STOP_NON;
        p_init->crx.start          = T32A_START_NON;
        /* Output Control 0 */
        p_init->outcrx0.ocr        = T32A_OCR_DISABLE;
        /* Output Control 1 */
        p_init->outcrx1.ocrcap1    = T32A_OCRCAPx1_DISABLE;
        p_init->outcrx1.ocrcap0    = T32A_OCRCAPx0_DISABLE;
        p_init->outcrx1.ocrcmp1    = T32A_OCRCMPx1_INVERSION;
        p_init->outcrx1.ocrcmp0    = T32A_OCRCMPx0_INVERSION;
        /* Timer Counter 1  */
        p_init->rgx0.rgx0          = 0;
        p_init->rgx1.rgx1          = 1;
        /* Reload  */
        p_init->reldx.reld         = 0;
        /* Counter Capture  */
        p_init->tmrx.tmrx          = 0;
        /* Counter Capture Control A0 */
        p_init->capx0.capx0        = 0;
        /* Counter Capture Control A1 */
        p_init->capx1.capx1        = 0;
        /* Interrupt mask register Set */
        p_init->imx.imuf           = T32A_IMUFx_MASK_REQ;
        p_init->imx.imof           = T32A_IMOFx_MASK_REQ;
        p_init->imx.imx1           = T32A_IMx1_MASK_REQ;
        p_init->imx.imx0           = T32A_IMx0_MASK_REQ;
        /* DMA Request      */
        p_init->dma_req.dmaenx2    = T32A_DMAENx2_DISABLE;
        p_init->dma_req.dmaenx1    = T32A_DMAENx1_DISABLE;
        p_init->dma_req.dmaenx0    = T32A_DMAENx0_DISABLE;
        /* User ID          */
        p_init->id                 = 0;
        /* Handler          */
        p_init->handler_T   = T32A_NULL;
        p_init->handler_TC0 = T32A_NULL;
        p_init->handler_TC1 = T32A_NULL;
        if (t32a_timer_init(&timer_sout, T32A_TIMERA) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
        BSP_T32A_SOUT_RUNA_SFTSTAA = 1;
    }
    if (result != TXZ_SUCCESS)
    {
        /* no processing */
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Initialize T32A Reference Signal Input Function
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void InitT32A_RefIn(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*-------------------------------*/
    /*  Reference Signal Input Timer */
    /*-------------------------------*/
    /*--- Construct (Register Allocation) ---*/
    /* Register Allocation */
    timer_refin.p_instance = BSP_T32A_REFIN;
    /*--- Initialize: Timer   ---*/
    /* Mode             */
    timer_refin.init_mode.mode.halt = T32A_DBG_HALT_RUN;
    timer_refin.init_mode.mode.mode = T32A_MODE_16;
    if (t32a_mode_init(&timer_refin) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*--- Initialize: Timer A ---*/
    {
        t32a_initial_setting_t *p_init = &timer_refin.init[T32A_TIMERA];

        /* Run Control      */
        p_init->runx.sftstp        = T32A_COUNT_STOP;
        p_init->runx.sftsta        = T32A_COUNT_DONT_START;
        p_init->runx.run           = T32A_RUN_ENABLE;
        /* Counter Control  */
        p_init->crx.prscl          = T32A_PRSCLx_2;
        p_init->crx.clk            = T32A_CLKx_PRSCLx;
        p_init->crx.wbf            = T32A_WBF_DISABLE;
        p_init->crx.updn           = T32A_COUNT_UP;
        p_init->crx.reld           = T32A_RELOAD_NON;
        p_init->crx.stop           = T32A_STOP_EXTTRG_RISING_EDGE;
        p_init->crx.start          = T32A_START_EXTTRG_TRAILING_EDGE;
        /* Output Control 0 */
        p_init->outcrx0.ocr        = T32A_OCR_DISABLE;
        /* Output Control 1 */
        p_init->outcrx1.ocrcap1    = T32A_OCRCAPx1_DISABLE;
        p_init->outcrx1.ocrcap0    = T32A_OCRCAPx0_DISABLE;
        p_init->outcrx1.ocrcmp1    = T32A_OCRCMPx1_DISABLE;
        p_init->outcrx1.ocrcmp0    = T32A_OCRCMPx0_DISABLE;
        /* Timer Counter 1  */
        p_init->rgx0.rgx0          = 0;
        p_init->rgx1.rgx1          = 0;
        /* Reload  */
        p_init->reldx.reld         = 0;
        /* Counter Capture  */
        p_init->tmrx.tmrx          = 0;
        /* Capture Control */
        p_init->capcrx.capmx1      = T32A_CAPMx1_INx0_RISING_EDGE;
        p_init->capcrx.capmx0      = T32A_CAPMx0_INx0_TRAILING_EDGE;
        /* Counter Capture Control A0 */
        p_init->capx0.capx0        = 0;
        /* Counter Capture Control A1 */
        p_init->capx1.capx1        = 0;
        /* Interrupt mask register Set */
        p_init->imx.imuf           = T32A_IMUFx_MASK_REQ;
        p_init->imx.imof           = T32A_IMOFx_MASK_REQ;
        p_init->imx.imx1           = T32A_IMx1_MASK_REQ;
        p_init->imx.imx0           = T32A_IMx0_MASK_REQ;
        /* DMA Request      */
        p_init->dma_req.dmaenx2    = T32A_DMAENx2_DISABLE;
        p_init->dma_req.dmaenx1    = T32A_DMAENx1_DISABLE;
        p_init->dma_req.dmaenx0    = T32A_DMAENx0_DISABLE;
        /* User ID          */
        p_init->id                 = 0;
        /* Handler          */
        p_init->handler_T   = T32A_NULL;
        p_init->handler_TC0 = T32A_NULL;
        p_init->handler_TC1 = T32A_NULL;
    }
    if (result != TXZ_SUCCESS)
    {
        /* no processing */
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Start Capture
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void StartCapture(void)
{
    BSP_T32A_REFIN_RUNA_SFTSTPA = 1;
    if (t32a_deinit(&timer_refin, T32A_TIMERA) != TXZ_SUCCESS)
    {
        /* no processing */
    }
    TSB_IB->IMC019 = (uint8_t)(0x46);
    TSB_IB->IMC020 = (uint8_t)(0x46);
    NVIC_ClearPendingIRQ(BSP_INTT32A_REFIN_ACCAP0_IRQn);
    TSB_IB->IMC019 = (uint8_t)(0x07);
    if (t32a_timer_init(&timer_refin, T32A_TIMERA) != TXZ_SUCCESS)
    {
        /* no processing */
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Capture Data
  * @param  -
  * @retval Capture value
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t GetCaptureData(void)
{
    uint32_t retval = 0;

    for (;;)
    {
        while (NVIC_GetPendingIRQ(BSP_INTT32A_REFIN_ACCAP0_IRQn) == 0)
        {
            /* waiting falling edge ... */
        }
        TSB_IB->IMC019 = (uint8_t)(0x46);
        NVIC_ClearPendingIRQ(BSP_INTT32A_REFIN_ACCAP1_IRQn);
        TSB_IB->IMC020 = (uint8_t)(0x07);
        while (NVIC_GetPendingIRQ(BSP_INTT32A_REFIN_ACCAP1_IRQn) == 0)
        {
            /* waiting raising edge ... */          
        }
        retval = (timer_refin.p_instance->CAPA1 & T32A_CAPx1_MASK);
        break;
    }   
    return(retval); 
}

/*--------------------------------------------------*/
/** 
  * @brief  Stop Capture
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void StopCapture(void)
{
    BSP_T32A_REFIN_RUNA_SFTSTPA = 1;
    if (t32a_deinit(&timer_refin, T32A_TIMERA) != TXZ_SUCCESS)
    {
        /* no processing */
    }
    TSB_IB->IMC019 = (uint8_t)(0x46);
    TSB_IB->IMC020 = (uint8_t)(0x46);
    NVIC_ClearPendingIRQ(BSP_INTT32A_REFIN_ACCAP0_IRQn);
    NVIC_ClearPendingIRQ(BSP_INTT32A_REFIN_ACCAP1_IRQn);
}

/**
 *  @}
 */ /* End of group TRMOSC_Exported_functions */

/**
 *  @}
 */ /* End of group TRMOSC */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
