/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TSPI_RECEIVE_CRC Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-05-10 14:11:56 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */
#include <string.h>
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TSPI_RECEIVE_CRC TSPI_RECEIVE_CRC Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSPI_RECEIVE_CRC_Private_macro TSPI_RECEIVE_CRC Private Macro
 *  @{
 */
/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */
/** 
 *  @defgroup  ReceiveError Receive Error 
 *  @brief     Receive Error  .
 *  @{
 */
#define ERROR_TSPI_RECEIVE_CRC      ((uint32_t)(1))
/** 
 *  @}
 */ /* End of group ReceiveError */
 
/** 
 *  @defgroup DataLengtheConfiguration Data Length Configuration
 *  @brief    Data Length Configuration.
 *  @{
 */
#define CFG_DATA_LENGTH            ((uint32_t)16)
/** 
 *  @}
 */ /* End of group DataLengtheConfiguration */

/** 
 *  @defgroup TSPI_SW_Reset SW Reset
 *  @brief    Software Rest Macro Definition.
 *  @{
 */
#define TSPI_RESET10                      ((uint32_t)0x00000080)    /*!< RESET Pattarn 10 */
#define TSPI_RESET01                      ((uint32_t)0x00000040)    /*!< RESET Pattarn 01 */
/**
 *  @}
 */ /* End of group TSPI_SW_Reset */
 

/** 
 *  @defgroup TSPI_Enable  TSPI Enable/Disable Control
 *  @brief    Enable/Disable TSPIE Macro Definition.
 *  @{
 */
#define TSPI_DISABLE                      ((uint32_t)0x00000000)    /*!< Disable */
#define TSPI_ENABLE                       ((uint32_t)0x00000001)    /*!< Enable  */
/**
 *  @}
 */ /* End of group TSPI_Enable */

/** 
 *  @name  TSPIxCR1 Macro Definition.
 *  @brief TSPIxCR1 Register Macro Definition.
 *  @{
 */
#define TSPIxCR1_TRXE_ENABLE        ((uint32_t)0x00004000)     /*!< TRXE=1                                                     */
#define TSPIxCR1_TRXE_DISABLE_MASK  ((uint32_t)0xFFFFBFFF)     /*!< Mask for TRXE=0                                            */
#define TSPIxCR1_MASTER_TX_INIT     ((uint32_t)0x00001500)     /*!< TRGEN=0,TRXE=0,TSPIMS=0,MSTR=1,TMMD=01,CSSEL=1,FC=00000000 */
#define TSPIxCR1_SLAVE_RX_INIT      ((uint32_t)0x00000900)     /*!< TRGEN=0,TRXE=0,TSPIMS=0,MSTR=0,TMMD=10,CSSEL=1,FC=00000000 */
/**
 *  @}
 */ /* End of name TSPIxCR1 Macro Definition */
/** 
 *  @name  TSPIxCR2 Macro Definition.
 *  @brief TSPIxCR2 Register Macro Definition.
 *  @{
 */
#define TSPIxCR2_MASTER_TX_INIT  ((uint32_t)0x00E10140)     /*!< TIDLE=11,TXDEMP=1,RXDLY=1,TIL=0000,RIL=0001,INTTXFE=0,INTTXWE=1,INTRXFE=0,INTRXWE=0,INTERR=0,DMATE=0,DMARE=0 */
#define TSPIxCR2_SLAVE_RX_INIT   ((uint32_t)0x00E10110)     /*!< TIDLE=11,TXDEMP=1,RXDLY=1,TIL=0000,RIL=0001,INTTXFE=0,INTTXWE=0,INTRXFE=0,INTRXWE=1,INTERR=0,DMATE=0,DMARE=0 */
/**
 *  @}
 */ /* End of name TSPIxCR2 Macro Definition */

/** 
 *  @name  TSPIxFMT0 Macro Definition.
 *  @brief TSPIxFMT0 Register Macro Definition.
 *  @{
 */
#define TSPIxFMT0_MASTER_TX_INIT  ((uint32_t)0x8800C400)     /*!< DIR=1,FL=001000,FINT=0000,CS3POL=0,CS2POL=0,CS1POL=0,CS0POL=0,CKPHA=1,CKPOL=1,CSINT=001,CSSCKDL=0000,SCKCSDL=0000 */
#define TSPIxFMT0_SLAVE_RX_INIT   ((uint32_t)0x8800C400)     /*!< DIR=1,FL=001000,FINT=0000,CS3POL=0,CS2POL=0,CS1POL=0,CS0POL=0,CKPHA=1,CKPOL=1,CSINT=001,CSSCKDL=0000,SCKCSDL=0000 */
/**
 *  @}
 */ /* End of name TSPIxFMT0 Macro Definition */
/** 
 *  @defgroup TSPI_State  State Flag
 *  @brief    State Flag Macro Definisiton.
 *  @{
 */
#define TSPI_TX_RUN                      ((uint32_t)0x00800000)    /*!< Active Sending Data */
#define TSPI_RX_RUN                      ((uint32_t)0x00000080)    /*!< Active Rending Data */
/**
 *  @}
 */ /* End of group TSPI_State */

/** 
 *  @defgroup TSPI_Done  Transmitting/Receiving Complete Flag
 *  @brief    Transmitting/Receiving Complete Flag Macro Definisiton.
 *  @{
 */
#define TSPI_TX_DONE_CLR        ((uint32_t)0x00400000)    /*!< Send Data Complete Flag Clear */
#define TSPI_RX_DONE_CLR        ((uint32_t)0x00000040)    /*!< Read Data Complete Flag Clear */
/**
 *  @}
 */ /* End of group TSPI_Done */
/** 
 *  @name  CRCTYP CRCTYP Macro Definition.
 *  @brief CRCTYP Register Macro Definition.
 *  @{
 */
#define CRCTYP_INIT              ((uint32_t)0x00000000)     /*!< CFMT=00,DBIT=00 */
/**
 *  @}
 */ /* End of name CRCTYP Macro Definition */
 
/** 
 *  @name  CRCCLC CRCCLC Macro Definition.
 *  @brief CRCCLC Register Macro Definition.
 *  @{
 */
#define CRCCLC_16_INIT           ((uint32_t)0x0000FFFF)     /*!< CRC16 Initial Value  */
/**
 *  @}
 */ /* End of name CRCCLC Macro Definition */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

/** 
 *  @}
 */ /* End of group TSPI_RECEIVE_CRC_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSPI_RECEIVE_CRC_Private_variables TSPI_RECEIVE_CRC Private Variables
 *  @{
 */
static uint32_t error    = 0;      /*!< error flag.        */
static uint32_t cmp_flag    = 0;      /*!< Receive cpmplete flag.        */
static uint8_t read = 0;
static uint8_t rx_data[CFG_DATA_LENGTH];
static uint8_t rx_length = 9;
static uint8_t stringtosend[10] = "TMPM4KQ";
static uint32_t counter  = CFG_LED_BLINK_TIME;    /* Blink Time Counter        */
/**
 *  @}
 */ /* End of group TSPI_RECEIVE_CRC_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSPI_RECEIVE_CRC_Private_fuctions TSPI_RECEIVE_CRC Private Fuctions
 *  @{
 */
static void GPIO_LED_Config(void);
static void GPIO_SW_Config(void);
static void GPIO_TSPI_Config(void);
static void TSPI_Config(void);
static void CRC_Config(void);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_LED_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_LED_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 1 (LED1) */
  TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 1 (LED1) */
  TSB_PJ_CR_PJ1C = GPIO_ENABLE;
  /* (6) Turn off PORT J pin 2 (LED2) */
  TSB_PJ_DATA_PJ2 = GPIO_DISABLE;
  /* (7) Select output mode on PORT J pin 2 (LED2) */
  TSB_PJ_CR_PJ2C = GPIO_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_SW_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_SW_Config(void)
{
  /* (1) Enable the peripheral clock of PORT A */
  TSB_CG_FSYSMENA_IPMENA00 = FSYS_ENABLE;
  /* (2) Select output mode on PORT A pin 0 (SWS) */
  TSB_PA_CR_PA0C = GPIO_ENABLE;
  TSB_PA_DATA_PA0 = GPIO_ENABLE;   /*SW select 1:SW7-4 */
  /* (3) Select input mode on PORT A pin 2 (SW4) */
  TSB_PA_IE_PA2IE = GPIO_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_TSPI_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_TSPI_Config(void)
{
  /* (1) Enable the peripheral clock of PORT G */
  TSB_CG_FSYSMENA_IPMENA06 = FSYS_ENABLE;
  /* (2) Select function mode (TSPI1CSIN) on PORT G pin 3    */
  TSB_PG_FR1_PG3F1 = GPIO_ENABLE;
  /* (3) Select input mode on PORT G pin 3    */
  TSB_PG_IE_PG3IE = GPIO_ENABLE;
  /* (4) Select function mode (TSPI1RXD) on PORT G pin 4    */
  TSB_PG_FR1_PG4F1 = GPIO_ENABLE;
  /* (5) Select input mode on PORT G pin 4    */
  TSB_PG_IE_PG4IE = GPIO_ENABLE;
  /* (6) Select function mode (TSPI1TXD) on PORT G pin 5    */
  TSB_PG_FR1_PG5F1 = GPIO_ENABLE;
  /* (7) Select output mode on PORT G pin 5    */
  TSB_PG_CR_PG5C = GPIO_ENABLE;
  /* (8) Select function mode (TSPI1SCK) on PORT G pin 6    */
  TSB_PG_FR1_PG6F1 = GPIO_ENABLE;
  /* (9) Select input mode on PORT G pin 6    */
  TSB_PG_IE_PG6IE = GPIO_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  TSPI_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TSPI_Config(void)
{
  /* (1) Enable the peripheral clock of TSPI1  */
  TSB_CG_FSYSMENA_IPMENA20 = FSYS_ENABLE;
  
  TSB_TSPI1->CR0 = (TSPI_RESET10 | TSPI_ENABLE);
  TSB_TSPI1->CR0 = (TSPI_RESET01 | TSPI_ENABLE);
  __NOP();
  __NOP();
  /* Configure SPI1 in slave */
  /* (1) Slave and SPI mode selection,
         TRGEN=0,TRXE=0,TSPIMS=0,MSTR=0,TMMD=10,CSSEL=1,FC=00000000 */
  /* (2) Receive complete interrupt is enabled, 
         TIDLE=11,TXDEMP=1,RXDLY=1,TIL=0000,RIL=0001,INTTXFE=0,INTTXWE=0,INTRXFE=0,INTRXWE=1,INTERR=0,DMATE=0,DMARE=0 */
  /* (3) Clock select fsys 160MHz,T0 80MHz, Tx 40MHz (1/2),TSPIxSCK 20MHz (N=2) */
  /* (4) CPOL and CPHA at 1 (rising first edge), 8-bit data frame, MSB first,
         DIR=1,FL=001000,FINT=0000,CS3POL=0,CS2POL=0,CS1POL=0,CS0POL=0,CKPHA=1,CKPOL=1,CSINT=001,CSSCKDL=0000,SCKCSDL=0000 */
  TSB_TSPI1->CR1   = TSPIxCR1_SLAVE_RX_INIT;  /* (1) */
  TSB_TSPI1->CR2   = TSPIxCR2_SLAVE_RX_INIT;  /* (2) */
  TSB_TSPI1->BR     = 0x12;                   /* (3) */
  TSB_TSPI1->FMTR0 = TSPIxFMT0_SLAVE_RX_INIT; /* (4) */
  /* (5) Receive interrupt Enable    */
  NVIC_EnableIRQ(INTSC1RX_IRQn); /* Enable INTSC1RX_IRQn */
}

/*--------------------------------------------------*/
/** 
  * @brief  CRC_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void CRC_Config(void)
{
  /* (1) Enable the peripheral clock of CRC  */
  TSB_CG_FSYSENA_IPENA00 = FSYS_ENABLE;
  
  /* Configure CRC */
  /* (1) CRC Control Register Setting, CRC16, 32bit data Width
         CFMT=00,DBIT=00                                        */
  /* (2) CRC Initial value Rsult Register,
         CRCCLC=0x0000FFFF                                      */
  TSB_CRC->TYP   = CRCTYP_INIT;    /* (1) */
  TSB_CRC->CLC   = CRCCLC_16_INIT;  /* (2) */
}

/**
 *  @}
 */ /* End of group TSPI_Private_functions */

/**
 *  @}
 */ /* End of group TSPI_RECEIVE_CRC_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSPI_RECEIVE_CRC_Exported_functions TSPI_RECEIVE_CRC Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  int32_t i;
  uint32_t period;    /* Period for Systick */
  uint32_t data1 = 0;
  uint32_t data2 = 0;
  uint32_t val[2]  = {0,0};
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_LED_Config();
  GPIO_SW_Config();
  GPIO_TSPI_Config();
  /*--- IBIMC033: Enable ---*/
  TSB_IB->IMC033 = (uint8_t)(0x05);
  /* Period for 10msec SYSTICK */
  SystemCoreClockUpdate();
  period = SystemCoreClock / 100;
  if (SysTick_Config(period ))    /* systick interrupt cycle setting */
  {
  }
  /*------------------------*/
  /* CRC Setting */
  /*------------------------*/ 
  CRC_Config();
  /* CRC  Input data Setting */
  for(i = 0; i<4; i++){
      data1 = ((data1 << 8U) |  (uint32_t)(stringtosend[i]));
  }
  for(i = 4; i<7; i++){
      data2 = ((data2 << 8U) |  (uint32_t)(stringtosend[i]));
  }
  data2 = (data2 << 8U);
  TSB_CRC->DIN = data1;
  TSB_CRC->DIN = data2;
  val[0] = TSB_CRC->CLC;
  val[1] = TSB_CRC->CLC;
  stringtosend[7] = (uint8_t)(val[1] >> 8);
  stringtosend[8] = (uint8_t)(val[1]  & 0x000000FF);
  /* Interrupt Enable */
  NVIC_EnableIRQ(INT00_IRQn);     /* interrupt pin 0 enable */
 while(1){
  cmp_flag = 0;
  /*------------------------*/
  /* TSPI Receiver Setting           */
  /*------------------------*/ 
  TSPI_Config();
  /* Interrupt Enable */
  /*------------------------*/
  /* Wait Data Receive */
  /*------------------------*/
  while(cmp_flag == 0){
    
  }
  if ( strncmp((char *)stringtosend, (char *)rx_data, rx_length) == 0 ) {
    TSB_PJ_DATA_PJ2 = 0;         /* Turn off led2 on PJ2           */
    TSB_PJ_DATA_PJ0 = 1;         /* Turn on  led0 on PJ0           */
  }else{
    error = ERROR_TSPI_RECEIVE_CRC;    /* Report an error */
  }
  for(i =0; i < CFG_DATA_LENGTH; i++)
  {
      rx_data[i] = 0;
  }
  TSB_PJ_DATA_PJ2 = 0;          /*  Turn off led2 on PJ2 */
 }
}
/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
  if(error == ERROR_TSPI_RECEIVE_CRC){
    if (counter-- == 0) 
    {
      /* The end of the counting down period, the data is
         inverted and stored in a register PJ1           */
      TSB_PJ_DATA_PJ1 ^= 1;     /* led1 blink for 1sec period */
      counter = CFG_LED_BLINK_TIME;
    }else{
        /* no processing */
    }
  }
}
/*--------------------------------------------------*/
/** 
  * @brief  INT00 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   0: Interrupt Pin0
  */
/*--------------------------------------------------*/
void INT00_IRQHandler(void)
{
    uint32_t tmp=0;
    /*--- IBIMC033: INTxCLR ---*/
    TSB_IB->IMC033 = (uint8_t)(0xC5);
    /* Dummy read is need. */
    {
        __IO uint8_t imc = TSB_IB->IMC033;
    }
    NVIC_DisableIRQ(INT00_IRQn);     /* interrupt pin 0 disable */
    TSB_PJ_DATA_PJ2 = 1;             /*  Turn on led2 on PJ2 */
    /* Wait TSPI transmission */
    /*--- TSPIxCR1 ---*/
    /* Write to TRXE(=1). */
    tmp = TSB_TSPI1->CR1;
    TSB_TSPI1->CR1 = (tmp | TSPIxCR1_TRXE_ENABLE);
}
/*--------------------------------------------------*/
/** 
  * @brief  INTSC1RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   62: TSPI/UART ch1 Reception interrupt
  */
/*--------------------------------------------------*/
void INTSC1RX_IRQHandler(void)
{
  uint32_t tmp=0;

  if((TSB_TSPI1->SR & TSPI_RX_RUN) != TSPI_RX_RUN)
  {
    if(read >= rx_length)
    {
      read=0;
      TSB_TSPI1->SR = TSPI_RX_DONE_CLR; /* Clear transfer complete flag */
      /*--- TSPIxCR1 ---*/
      /* Write to TRXE(=0). */
      tmp = TSB_TSPI1->CR1;
      TSB_TSPI1->CR1 = (tmp & TSPIxCR1_TRXE_DISABLE_MASK);
      cmp_flag = 1;
    }
    else
    {
      /* Reading Receive data */
      rx_data[read++] = TSB_TSPI1->DR;
    }
  }
  else
  {
    error = ERROR_TSPI_RECEIVE_CRC;    /* Report an error */
    NVIC_DisableIRQ(INTSC1RX_IRQn); /* Disable INTSC1RX_IRQn */
  }
}
/**
 *  @}
 */ /* End of group TSPI_RECEIVE_CRC_Exported_functions */

/**
 *  @}
 */ /* End of group TSPI_RECEIVE_CRC */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
