/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TSPI_TRANS_DMA Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-06-25 12:12:42 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */
#include "tspi_dma.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TSPI_TRANS_DMA TSPI_TRANS_DMA Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSPI_TRANS_DMA_Private_macro TSPI_TRANS_DMA Private Macro
 *  @{
 */
/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */
/** 
 *  @defgroup  TransmitError Transmit Error 
 *  @brief    Transmit Error  .
 *  @{
 */
#define ERROR_TSPI_TRANSMIT       ((uint32_t)(1))
/** 
 *  @}
 */ /* End of group TransmitError */
/** 
 *  @defgroup DataLengtheConfiguration Data Length Configuration
 *  @brief    Data Length Configuration.
 *  @{
 */
#define CFG_DATA_LENGTH            ((uint32_t)16)
/** 
 *  @}
 */ /* End of group DataLengtheConfiguration */
/** 
 *  @defgroup TSPI_SW_Reset SW Reset
 *  @brief    Software Rest Macro Definition.
 *  @{
 */
#define TSPI_RESET10                      ((uint32_t)0x00000080)    /*!< RESET Pattarn 10 */
#define TSPI_RESET01                      ((uint32_t)0x00000040)    /*!< RESET Pattarn 01 */
/**
 *  @}
 */ /* End of group TSPI_SW_Reset */
/** 
 *  @defgroup TSPI_Enable  TSPI Enable/Disable Control
 *  @brief    Enable/Disable TSPIE Macro Definition.
 *  @{
 */
#define TSPI_DISABLE                      ((uint32_t)0x00000000)    /*!< Disable */
#define TSPI_ENABLE                       ((uint32_t)0x00000001)    /*!< Enable  */
/**
 *  @}
 */ /* End of group TSPI_Enable */
/** 
 *  @name  TSPIxCR1 Macro Definition.
 *  @brief TSPIxCR1 Register Macro Definition.
 *  @{
 */
#define TSPIxCR1_TRXE_ENABLE        ((uint32_t)0x00004000)     /*!< TRXE=1                                                     */
#define TSPIxCR1_TRXE_DISABLE_MASK  ((uint32_t)0xFFFFBFFF)     /*!< Mask for TRXE=0                                            */
#define TSPIxCR1_MASTER_TX_INIT     ((uint32_t)0x00001500)     /*!< TRGEN=0,TRXE=0,TSPIMS=0,MSTR=1,TMMD=01,CSSEL=0,FC=00000000 */
#define TSPIxCR1_SLAVE_RX_INIT      ((uint32_t)0x00000900)     /*!< TRGEN=0,TRXE=0,TSPIMS=0,MSTR=0,TMMD=10,CSSEL=0,FC=00000000 */
/**
 *  @}
 */ /* End of name TSPIxCR1 Macro Definition */
/** 
 *  @name  TSPIxCR2 Macro Definition.
 *  @brief TSPIxCR2 Register Macro Definition.
 *  @{
 */
#define TSPIxCR2_MASTER_TX_INIT  ((uint32_t)0x00E10142)     /*!< TIDLE=11,TXDEMP=1,RXDLY=1,TIL=0000,RIL=0001,INTTXFE=0,INTTXWE=1,INTRXFE=0,INTRXWE=0,INTERR=0,DMATE=1,DMARE=0 */
#define TSPIxCR2_SLAVE_RX_INIT   ((uint32_t)0x00E10111)     /*!< TIDLE=11,TXDEMP=1,RXDLY=1,TIL=0000,RIL=0001,INTTXFE=0,INTTXWE=0,INTRXFE=0,INTRXWE=1,INTERR=0,DMATE=0,DMARE=1 */
/**
 *  @}
 */ /* End of name TSPIxCR2 Macro Definition */
/** 
 *  @name  TSPIxFMT0 Macro Definition.
 *  @brief TSPIxFMT0 Register Macro Definition.
 *  @{
 */
#define TSPIxFMT0_MASTER_TX_INIT  ((uint32_t)0x8800C400)     /*!< DIR=1,FL=001000,FINT=0000,CS3POL=0,CS2POL=0,CS1POL=0,CS0POL=0,CKPHA=1,CKPOL=1,CSINT=001,CSSCKDL=0000,SCKCSDL=0000 */
#define TSPIxFMT0_SLAVE_RX_INIT   ((uint32_t)0x8800C400)     /*!< DIR=1,FL=001000,FINT=0000,CS3POL=0,CS2POL=0,CS1POL=0,CS0POL=0,CKPHA=1,CKPOL=1,CSINT=001,CSSCKDL=0000,SCKCSDL=0000 */
/**
 *  @}
 */ /* End of name TSPIxFMT0 Macro Definition */
/** 
 *  @defgroup TSPI_TxState  Transmitting State Flag
 *  @brief    Transmitting State Flag Macro Definisiton.
 *  @{
 */
#define TSPI_TX_RUN                      ((uint32_t)0x00800000)    /*!< Active Sending Data */
/**
 *  @}
 */ /* End of group TSPI_TxState */
/** 
 *  @defgroup TSPI_TxDone  Transmitting Complete Flag
 *  @brief    Transmitting Complete Flag Macro Definisiton.
 *  @{
 */
#define TSPI_TX_DONE_CLR        ((uint32_t)0x00400000)    /*!< Send Data Complete Flag Clear */
/**
 *  @}
 */ /* End of group TSPI_TxDone */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */
/** 
 *  @}
 */ /* End of group TSPI_TRANS_DMA_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSPI_TRANS_DMA_Private_define TSPI_TRANS_DMA Private Define
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_START,             /*!< Start.                       */
    PROC_STOP,              /*!< Stop.                        */
    PROC_SW_WAIT,           /*!< Switch wait                  */
    PROC_DMA_TRANS,         /*!< Dma transmiting.             */
    PROC_ERROR              /*!< Error.                       */
} Processing;
/** 
 *  @}
 */ /* End of group TSPI_TRANS_DMA_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSPI_TRANS_DMA_Private_variables TSPI_TRANS_DMA Private Variables
 *  @{
 */
uint8_t rx_data[CFG_DATA_LENGTH];
static uint32_t error    = 0;      /*!< error flag.        */
static uint32_t counter  = CFG_LED_BLINK_TIME;    /* Blink Time Counter        */
static Processing proc;      /*!< Processing mode.        */
/**
 *  @}
 */ /* End of group TSPI_TRANS_DMA_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSPI_TRANS_DMA_Private_fuctions TSPI_TRANS_DMA Private Fuctions
 *  @{
 */
static void GPIO_LED_Config(void);
static void GPIO_SW_Config(void);
static void GPIO_TSPI_Config(void);
static void TSPI_Config(void);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_LED_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_LED_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 1 (LED1) */
  TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 1 (LED1) */
  TSB_PJ_CR_PJ1C = GPIO_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_SW_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_SW_Config(void)
{
  /* (1) Enable the peripheral clock of PORT A */
  TSB_CG_FSYSMENA_IPMENA00 = FSYS_ENABLE;
  /* (2) Select output mode on PORT A pin 0 (SWS) */
  TSB_PA_CR_PA0C = GPIO_ENABLE;
  TSB_PA_DATA_PA0 = GPIO_ENABLE;   /*SW select 1:SW7-4 */
  /* (3) Select input mode on PORT A pin 2 (SW4) */
  TSB_PA_IE_PA2IE = GPIO_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_TSPI_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_TSPI_Config(void)
{
  /* (1) Enable the peripheral clock of PORT G */
  TSB_CG_FSYSMENA_IPMENA06 = FSYS_ENABLE;
  /* (2) Select function mode (TSPI1CS1) on PORT G pin 1    */
  TSB_PG_FR1_PG1F1 = GPIO_ENABLE;
  /* (3) Select output mode on PORT G pin 1    */
  TSB_PG_CR_PG1C = GPIO_ENABLE;

  /* (4) Select function mode (TSPI1TXD) on PORT G pin 5    */
  TSB_PG_FR1_PG5F1 = GPIO_ENABLE;
  /* (5) Select output mode on PORT G pin 5    */
  TSB_PG_CR_PG5C = GPIO_ENABLE;
  /* (6) Select function mode (TSPI1SCK) on PORT G pin 6    */
  TSB_PG_FR1_PG6F1   = GPIO_ENABLE;
  /* (7) Select output mode on PORT G pin 6    */
  TSB_PG_CR_PG6C = GPIO_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  TSPI_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TSPI_Config(void)
{
  /* (1) Enable the peripheral clock of TSPI1  */
  TSB_CG_FSYSMENA_IPMENA20 = FSYS_ENABLE;
  
  TSB_TSPI1->CR0 = (TSPI_RESET10 | TSPI_ENABLE);
  TSB_TSPI1->CR0 = (TSPI_RESET01 | TSPI_ENABLE);
  __NOP();
  __NOP();
  /* Configure SPI1 in master */
  /* (1) Master and SPI mode selection,
         TRGEN=0,TRXE=0,TSPIMS=0,MSTR=1,TMMD=01,CSSEL=1,FC=00000000 */
  /* (2) Transmit complete interrupt is enabled,
         TIDLE=11,TXDEMP=1,RXDLY=1,TIL=0000,RIL=0001,INTTXFE=0,INTTXWE=1,INTRXFE=0,INTRXWE=0,INTERR=0,DMATE=1,DMARE=0 */
  /* (3) Clock select fsys 160MHz,T0 80MHz, Tx 40MHz (1/2),TSPIxSCK 20MHz (N=2) */
  /* (4) CPOL and CPHA at 1 (rising first edge), 8-bit data frame, MSB first,
         DIR=1,FL=001000,FINT=0000,CS3POL=0,CS2POL=0,CS1POL=0,CS0POL=0,CKPHA=1,CKPOL=1,CSINT=001,CSSCKDL=0000,SCKCSDL=0000 */
  TSB_TSPI1->CR1   = TSPIxCR1_MASTER_TX_INIT;  /* (1) */
  TSB_TSPI1->CR2   = TSPIxCR2_MASTER_TX_INIT;  /* (2) */
  TSB_TSPI1->BR     = 0x12;                    /* (3) */
  TSB_TSPI1->FMTR0 = TSPIxFMT0_MASTER_TX_INIT; /* (4) */
}
/**
 *  @}
 */ /* End of group TSPI_TRANS_DMA_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSPI_TRANS_DMA_Exported_functions TSPI_TRANS_DMA Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  uint32_t period;    /* Period for Systick */
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_LED_Config();
  GPIO_SW_Config();
  GPIO_TSPI_Config();
  /*--- IBIMC033: Enable ---*/
  TSB_IB->IMC033 = (uint8_t)(0x05);
  /* Period for 10msec SYSTICK */
  SystemCoreClockUpdate();
  period = SystemCoreClock / 100;
  if (SysTick_Config(period ))    /* systick interrupt cycle setting */
  {
  }
  
  DMA_Config();
  TSPI_Config();
  /* Interrupt Enable */
  NVIC_EnableIRQ(INT00_IRQn);     /* interrupt pin 0 enable */
  /*------------------------*/
  /* Normal Mode Processing */
  /*------------------------*/
  while(proc != PROC_ERROR){
	switch(proc)
	{
		/*--- Start ---*/
        case PROC_START:
            {
                proc = PROC_SW_WAIT;
            }
            break;
        /*--- Stop ---*/
        case PROC_STOP:
            {
                /* DMA  */
                NVIC_DisableIRQ(INTDMAATC_IRQn);
                NVIC_DisableIRQ(INTDMAAERR_IRQn);
                error = ERROR_TSPI_TRANSMIT;    /* Report an error */
                proc = PROC_ERROR;
            }
            break;
        /*--- Wait to Switch ---*/
        case PROC_SW_WAIT:
            {
                /* no processing */
            }
            break;
        /*--- DMA Trans ---*/
        case PROC_DMA_TRANS:
            {
                TSPI_DMA_Trans(8);
                TSB_PJ_DATA_PJ0 = 1;              /* Turn on led0 on PJ0          */
                proc = PROC_START;
            }
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* no processing */
            break;
        /*--- Default ---*/
        default:
            error = ERROR_TSPI_TRANSMIT;    /* Report an error */
            proc = PROC_ERROR;
            break;
        }
  }
}
/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
  if(error == ERROR_TSPI_TRANSMIT){
    if (counter-- == 0) 
    {
      /* The end of the counting down period, the data is
         inverted and stored in a register PJ1           */
      TSB_PJ_DATA_PJ1 ^= 1;     /* led1 blink for 1sec period */
      counter = CFG_LED_BLINK_TIME;
    }else{
        /* no processing */
    }
  }
}
/*--------------------------------------------------*/
/** 
  * @brief  INT00 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   0: Interrupt Pin0
  */
/*--------------------------------------------------*/
void INT00_IRQHandler(void)
{
    uint32_t tmp=0;
    /*--- IBIMC033: INTxCLR ---*/
    TSB_IB->IMC033 = (uint8_t)(0xC5);
    /* Dummy read is need. */
    {
        __IO uint8_t imc = TSB_IB->IMC033;
    }
    NVIC_DisableIRQ(INT00_IRQn);     /* interrupt pin 0 disable */
    
     proc = PROC_DMA_TRANS;
    /*--- TSPIxCR1 ---*/
    /* Write to TRXE(=1). */
    tmp = TSB_TSPI1->CR1;
    TSB_TSPI1->CR1 = (tmp | TSPIxCR1_TRXE_ENABLE);
}
/**
 *  @}
 */ /* End of group TSPI_TRANS_DMA_Exported_functions */

/**
 *  @}
 */ /* End of group TSPI_TRANS_DMA */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
