/** 
 *******************************************************************************
 * @file    main.c
 * @brief   UART_RECEIVE Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-06-25 10:41:42 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */
#include <string.h>
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup UART_RECEIVE UART_RECEIVE Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_RECEIVE_Private_macro UART_RECEIVE Private Macro
 *  @{
 */
/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */
/** 
 *  @defgroup BaudRateConfiguration BaudRate Configuration
 *  @brief    BaudRate .
 *  @{
 */
#define CFG_BAUDRATE             ((uint32_t)(115200))
#define CFG_BAUDRATE_SET_VAL     ((uint32_t)(0x00A6002B))
/** 
 *  @}
 */ /* End of group BaudRateConfiguration */
/** 
 *  @defgroup DataLengtheConfiguration Data Length Configuration
 *  @brief    Data Length Configuration.
 *  @{
 */
#define CFG_DATA_LENGTH            ((uint32_t)16)
/** 
 *  @}
 */ /* End of group DataLengtheConfiguration */
/** 
 *  @defgroup  ReceiveError Receive Error 
 *  @brief    Receive Error  .
 *  @{
 */
#define ERROR_UART_RECEIVE       ((uint32_t)(1))
/** 
 *  @}
 */ /* End of group ReceiveError */

/** 
 *  @defgroup UART_DataConfiguration  Data Configuration
 *  @brief    Data Configuration Macro Definisiton.
 *  @{
 */
 #define UART_DATA_DIRECTION_MSB         ((uint32_t)0x00000020)    /*!< MSB first */
#define UART_DATA_DIRECTION_LSB         ((uint32_t)0x00000000)    /*!< LSB first */
#define UART_STOP_BIT_1                 ((uint32_t)0x00000000)    /*!< 1 bit */
#define UART_PARITY_BIT_ODD             ((uint32_t)0x00000000)    /*!< Odd Parity  */
#define UART_PARITY_DISABLE             ((uint32_t)0x00000000)    /*!< Disable */
#define UART_DATA_LENGTH_8              ((uint32_t)0x00000001)    /*!< 8 bit */
 /**
 *  @}
 */ /* End of group UART_DataLength */

/** 
 *  @defgroup UART_RxInterrupt  Rx Interrpt
 *  @brief    Available Receive Interrupt Macro Definisiton.
 *  @{
 */
#define UART_RX_INT_DISABLE             ((uint32_t)0x00000000)    /*!< Not Available. */
#define UART_RX_INT_ENABLE              ((uint32_t)0x00000010)    /*!< Available.     */
/**
 *  @}
 */ /* End of group UART_RxInterrupt */
/** 
 *  @defgroup UART_RxState  Rx State
 *  @brief    Receive State Macro Definisiton.
 *  @{
 */
#define UART_RX_STATE_RUN               ((uint32_t)0x00000080)    /*!< Run      */
/**
 *  @}
 */ /* End of group UART_RxState */
/** 
 *  @defgroup UART_RxDone  Rx Done
 *  @brief    Receive Done Macro Definisiton.
 *  @{
 */
#define UART_RX_DONE                    ((uint32_t)0x00000040)    /*!< Receive Done */
/**
 *  @}
 */ /* End of group UART_RxDone */
 
 /** 
 *  @defgroup UART_RxEnable  Receiving Enable
 *  @brief    Receiving Enable Macro Definisiton.
 *  @{
 */
#define UART_RXE_ENABLE                 ((uint32_t)0x00000001)    /*!< Receiving Enable */
#define UART_RXE_DISABLE                ((uint32_t)0x00000000)    /*!< Receiving Disable */
/**
 *  @}
 */ /* End of group UART_RxEnable */

/** 
 *  @defgroup UART_Prescaler  Prescaler
 *  @brief    Prescaler Macro Definisiton.
 *  @{
 */
#define UART_PLESCALER_1                ((uint32_t)0x00000000)    /*!< 1/1   */
/**
 *  @}
 */ /* End of group UART_Prescaler */
 
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

/** 
 *  @}
 */ /* End of group UART_RECEIVE_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_RECEIVE_Private_variables UART_RECEIVE Private Variables
 *  @{
 */
static uint32_t error       = 0;      /*!< error flag.        */
static uint32_t cmp_flag    = 0;      /*!< Receive complete flag.        */
static uint32_t read_flag   = 0;      /*!< Receive data flag.           */
static uint32_t counter  = CFG_LED_BLINK_TIME;    /* Blink Time Counter        */
static uint8_t rx_data[CFG_DATA_LENGTH ];
static uint32_t read        = 0;
static uint8_t rx_length    = 8;
static uint8_t cmp_length   = 6;
static const uint8_t stringtosend[] = "TMPM4KQ\n";
/**
 *  @}
 */ /* End of group UART_RECEIVE_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_RECEIVE_Private_fuctions UART_RECEIVE Private Fuctions
 *  @{
 */
static void GPIO_LED_Config(void);
static void GPIO_UART_Config(void);
static void UART_Config(void);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_LED_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_LED_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 1 (LED1) */
  TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 1 (LED1) */
  TSB_PJ_CR_PJ1C = GPIO_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  GPIO_UART_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_UART_Config(void)
{
  /* (1) Enable the peripheral clock of PORT U */
  TSB_CG_FSYSMENA_IPMENA16 = FSYS_ENABLE;
  /* (2) Select function mode on PORT U pin 5    */
  TSB_PU_FR1_PU5F1 = GPIO_ENABLE;
  /* (3) Select output mode on PORT U pin 5    */
  TSB_PU_CR_PU5C = GPIO_ENABLE;
  TSB_PU_CR_PU6C = GPIO_DISABLE;
  /* (4) Select function mode on PORT U pin 6    */
  TSB_PU_FR1_PU6F1 = GPIO_ENABLE;
  /* (5) Select input mode on PORT U pin 6    */
  TSB_PU_IE_PU5IE = GPIO_DISABLE;
  TSB_PU_IE_PU6IE = GPIO_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  UART_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void UART_Config(void)
{
  /* (1) Enable the peripheral clock of UART1  */
  TSB_CG_FSYSMENA_IPMENA22 = FSYS_ENABLE;
  
  TSB_UART1->SWRST = 0x2U;
  TSB_UART1->SWRST = 0x1U;
  
  while(TSB_UART1->SWRST==  0x80)
  {
  }
    /* (2) oversampling by 115200 baud */
  TSB_UART1->BRD = CFG_BAUDRATE_SET_VAL;
  TSB_UART1->CLK = (UART_PLESCALER_1 << 4);
  /* (3) 8 data bit, 1 start bit, 1 stop bit, no parity */
  TSB_UART1->CR0 = (UART_DATA_DIRECTION_LSB | UART_STOP_BIT_1 | UART_PARITY_BIT_ODD | \
                    UART_PARITY_DISABLE | UART_DATA_LENGTH_8);
  /* (4) Transmit interrupt Enable    */
  TSB_UART1->CR1 = (UART_RX_INT_ENABLE);
  NVIC_EnableIRQ(INTSC1RX_IRQn); /* Enable UART1_IRQn */
  /*--- UARTxTRANS ---*/
  /* Write to RXE(=1). */
  TSB_UART1->TRANS = UART_RXE_ENABLE;
}

/**
 *  @}
 */ /* End of group UART_Private_functions */

/**
 *  @}
 */ /* End of group UART_RECEIVE_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_RECEIVE_Exported_functions UART_RECEIVE Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  uint32_t period;    /* Period for Systick */
  int32_t i;
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_LED_Config();
  GPIO_UART_Config();
  /*--- IBIMC033: Enable ---*/
  TSB_IB->IMC033 = (uint8_t)(0x05);
  
  /* Period for 10msec SYSTICK */
  SystemCoreClockUpdate();
  period = SystemCoreClock / 100;
  if (SysTick_Config(period ))    /* systick interrupt cycle setting */
  {
  }
  
  UART_Config();
  /*------------------------*/
  /* Wait Data Receive */
  /*------------------------*/
  while(cmp_flag == 0){
 
  }
  read = 0;
  cmp_flag = 0;
  /*------------------------*/
  /* Compare Receiving Data */
  /*------------------------*/  
  if ( strncmp((char *)stringtosend, (char *)rx_data, cmp_length) == 0 ) {
    TSB_PJ_DATA_PJ0 = 1;
    for(i =0; i < CFG_DATA_LENGTH; i++)
    {
        rx_data[i] = 0;
    }
  }else{
   error = ERROR_UART_RECEIVE;    /* Report an error */
   for(i =0; i < CFG_DATA_LENGTH; i++)
    {
        rx_data[i] = 0;
    }
  }
  /*--- UARTxTRANS ---*/
  /* Write to RXE(=1). */
  TSB_UART1->TRANS = UART_RXE_ENABLE;
  /*------------------------*/
  /*          Loop                         */
  /*------------------------*/ 
  while(1){
     if(error == ERROR_UART_RECEIVE)
     {
        if(read_flag == 1)
        {
            error = 0;   /* When led1 is toggled,  something is input next,  LED1 is turned off.  */
        }
    }
  }  
}

/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
  if(error != 0){
    if (counter-- == 0) 
    {
      /* The end of the counting down period, the data is
         inverted and stored in a register PJ1           */
      TSB_PJ_DATA_PJ1 ^= 1;     /* led1 blink for 1sec period */
      counter = CFG_LED_BLINK_TIME;
    }else{
        /* no processing */
    }
  }else{
      TSB_PJ_DATA_PJ1 = 0;
  }
}


/*--------------------------------------------------*/
/** 
  * @brief  INTSC1RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   62: TSPI/UART ch1 Reception interrupt
  */
/*--------------------------------------------------*/
void INTSC1RX_IRQHandler(void)
{
  /*--- UARTxTRANS ---*/
  /* Write to RXE(=0). */
   TSB_UART1->TRANS = UART_RXE_DISABLE;
  if((TSB_UART1->SR & UART_RX_STATE_RUN) != UART_RX_STATE_RUN)
  {
    if(read >= rx_length ){
        cmp_flag = 1;
        read_flag = 0;
        TSB_UART1->SR = UART_RX_DONE; /* Clear Receive complete flag */
    }else{
        rx_data[read++] = (uint8_t)(TSB_UART1->DR);
        /*--- UARTxTRANS ---*/
        /* Write to RXE(=1). */
        TSB_UART1->TRANS = UART_RXE_ENABLE;
        read_flag = 1;
    }
  }
  else
  {
    error = ERROR_UART_RECEIVE;    /* Report an error */
  }
}
/**
 *  @}
 */ /* End of group UART_RECEIVE_Exported_functions */

/**
 *  @}
 */ /* End of group UART_RECEIVE */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
