/** 
 *******************************************************************************
 * @file    main.c
 * @brief   UART_TRANS_DMA Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-06-25 13:09:51 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */
#include "uart_io_dma.h"
/**
 *  @addtogroup Example
 *  @{
 */
/** 
 *  @defgroup UART_TRANS_DMA UART_TRANS_DMA Sample Appli
 *  @{
 */
 
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_TRANS_DMA_Private_macro UART_TRANS_DMA Private Macro
 *  @{
 */
/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */
/** 
 *  @defgroup BaudRateConfiguration BaudRate Configuration
 *  @brief    BaudRate .
 *  @{
 */
#define CFG_BAUDRATE             ((uint32_t)(115200))
#define CFG_BAUDRATE_SET_VAL     ((uint32_t)(0x00A6002B))
/** 
 *  @}
 */ /* End of group BaudRateConfiguration */
/** 
 *  @defgroup  TransmitError Transmit Error 
 *  @brief    Transmit Error  .
 *  @{
 */
#define ERROR_UART_TRANSMIT       ((uint32_t)(1))
/** 
 *  @}
 */ /* End of group TransmitError */
/** 
 *  @defgroup DataLengtheConfiguration Data Length Configuration
 *  @brief    Data Length Configuration.
 *  @{
 */
#define CFG_DATA_LENGTH            ((uint32_t)16)
/** 
 *  @}
 */ /* End of group DataLengtheConfiguration */
/** 
 *  @defgroup UART_DataConfiguration  Data Configuration
 *  @brief    Data Configuration Macro Definisiton.
 *  @{
 */
#define UART_DATA_DIRECTION_LSB         ((uint32_t)0x00000000)    /*!< LSB first */
#define UART_STOP_BIT_1                 ((uint32_t)0x00000000)    /*!< 1 bit */
#define UART_PARITY_BIT_ODD             ((uint32_t)0x00000000)    /*!< Odd Parity  */
#define UART_PARITY_DISABLE             ((uint32_t)0x00000000)    /*!< Disable */
#define UART_DATA_LENGTH_8              ((uint32_t)0x00000001)    /*!< 8 bit */
 /**
 *  @}
 */ /* End of group UART_DataLength */
/** 
 *  @defgroup UART_TxInterrupt  Tx Interrpt
 *  @brief    Available Transmit Interrupt Macro Definisiton.
 *  @{
 */
#define UART_TX_DMATE                   ((uint32_t)0x00000002)    /*!< Available.     */
/**
 *  @}
 */ /* End of group UART_TxInterrupt */
/** 
 *  @defgroup UART_TxEnable  Transmitting Enable
 *  @brief    Transmitting Enable Macro Definisiton.
 *  @{
 */
#define UART_TXE_ENABLE                 ((uint32_t)0x00000002)    /*!< Transmitting Enable */
#define UART_TXE_DISABLE                ((uint32_t)0x00000000)    /*!< Transmitting Disable */
/**
 *  @}
 */ /* End of group UART_TxEnable */
/** 
 *  @defgroup UART_Prescaler  Prescaler
 *  @brief    Prescaler Macro Definisiton.
 *  @{
 */
#define UART_PLESCALER_1                ((uint32_t)0x00000000)    /*!< 1/1   */
/**
 *  @}
 */ /* End of group UART_Prescaler */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */
/** 
 *  @}
 */ /* End of group UART_TRANS_DMA_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_TRANS_DMA_Private_define UART_TRANS_DMA Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_START,             /*!< Start.                       */
    PROC_STOP,              /*!< Stop.                        */
    PROC_SW_WAIT,           /*!< Switch wait                  */
    PROC_DMA_TRANS,         /*!< Dma transmiting.             */
    PROC_ERROR              /*!< Error.                       */
} Processing;
/** 
 *  @}
 */ /* End of group UART_TRANS_DMA_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_TRANS_DMA_Private_variables UART_TRANS_DMA Private Variables
 *  @{
 */
uint8_t rx_data[CFG_DATA_LENGTH];
static uint32_t error    = 0;      /*!< error flag.        */
static uint32_t counter  = CFG_LED_BLINK_TIME;    /* Blink Time Counter        */
static Processing proc;      /*!< Processing mode.        */
/**
 *  @}
 */ /* End of group UART_TRANS_DMA_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_TRANS_DMA_Private_fuctions UART_TRANS_DMA Private Fuctions
 *  @{
 */
static void GPIO_LED_Config(void);
static void GPIO_SW_Config(void);
static void GPIO_UART_Config(void);
static void UART_Config(void);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_LED_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_LED_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 1 (LED1) */
  TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 1 (LED1) */
  TSB_PJ_CR_PJ1C = GPIO_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_SW_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_SW_Config(void)
{
  /* (1) Enable the peripheral clock of PORT A */
  TSB_CG_FSYSMENA_IPMENA00 = FSYS_ENABLE;
  /* (2) Select output mode on PORT A pin 0 (SWS) */
  TSB_PA_CR_PA0C = GPIO_ENABLE;
  TSB_PA_DATA_PA0 = GPIO_ENABLE;   /*SW select 1:SW7-4 */
  /* (3) Select input mode on PORT A pin 2 (SW4) */
  TSB_PA_IE_PA2IE = GPIO_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_UART_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_UART_Config(void)
{
  /* (1) Enable the peripheral clock of PORT U */
  TSB_CG_FSYSMENA_IPMENA16 = FSYS_ENABLE;
  /* (2) Select function mode on PORT U pin 5    */
  TSB_PU_FR1_PU5F1 = GPIO_ENABLE;
  /* (3) Select output mode on PORT U pin 5    */
  TSB_PU_CR_PU5C = GPIO_ENABLE;
  /* (4) Select function mode on PORT U pin 6    */
  TSB_PU_FR1_PU6F1 = GPIO_ENABLE;
  /* (5) Select input mode on PORT U pin 6    */
  TSB_PU_IE_PU6IE = GPIO_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  UART_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void UART_Config(void)
{
  /* (1) Enable the peripheral clock of UART1  */
  TSB_CG_FSYSMENA_IPMENA22 = FSYS_ENABLE;
  
  TSB_UART1->SWRST = 0x2U;
  TSB_UART1->SWRST = 0x1U;
  
  while(TSB_UART1->SWRST==  0x80)
  {
  }
  /* (2) oversampling by 115200 baud */
  /*  Fsysme = 80MHz, KEN = 1:Enable, N+(64-K)/64 division K:0x2B N:0x2B */
  TSB_UART1->BRD = CFG_BAUDRATE_SET_VAL;
  TSB_UART1->CLK = (UART_PLESCALER_1 << 4);
  /* (3) 8 data bit, 1 start bit, 1 stop bit, no parity */
  TSB_UART1->CR0 = (UART_DATA_DIRECTION_LSB | UART_STOP_BIT_1 | UART_PARITY_BIT_ODD | \
                    UART_PARITY_DISABLE | UART_DATA_LENGTH_8);
  /* (4) Transmit interrupt Enable    */
  TSB_UART1->CR1 = (UART_TX_DMATE);
}

/**
 *  @}
 */ /* End of group UART_TRANS_DMA_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_TRANS_DMA_Exported_functions UART_TRANS_DMA Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  uint32_t period;    /* Period for Systick */
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_LED_Config();
  GPIO_SW_Config();
  GPIO_UART_Config();
  /*--- IBIMC033: Enable ---*/
  TSB_IB->IMC033 = (uint8_t)(0x05);
  /* Period for 10msec SYSTICK */
  SystemCoreClockUpdate();
  period = SystemCoreClock / 100;
  if (SysTick_Config(period ))    /* systick interrupt cycle setting */
  {
  }
  
  DMA_Config();
  UART_Config();
  /* Interrupt Enable */
  NVIC_EnableIRQ(INT00_IRQn);                       /* interrupt pin 0 enable */
  proc = PROC_START;
  /*------------------------*/
  /* Loop */
  /*------------------------*/
  while(proc != PROC_ERROR){
	switch(proc)
        {
        /*--- Start ---*/
        case PROC_START:
            {
                proc = PROC_SW_WAIT;
                /* Interrupt Enable */
                NVIC_EnableIRQ(INT00_IRQn);                       /* interrupt pin 0 enable */
            }
            break;
        /*--- Stop ---*/
        case PROC_STOP:
            {
                /* DMA  */
                NVIC_DisableIRQ(INTDMAATC_IRQn);
                NVIC_DisableIRQ(INTDMAAERR_IRQn);
                proc = PROC_ERROR;
            }
            break;
        /*--- Wait to Switch ---*/
        case PROC_SW_WAIT:
            {
                /* no processing */
            }
            break;
        /*--- DMA Trans ---*/
        case PROC_DMA_TRANS:
            {
                UART_DMA_Trans(8);
                proc = PROC_START;
            }
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* no processing */
            break;
        /*--- Default ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
  }
}
/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
  if(error == ERROR_UART_TRANSMIT){
    if (counter-- == 0) 
    {
      /* The end of the counting down period, the data is
         inverted and stored in a register PJ1           */
      TSB_PJ_DATA_PJ1 ^= 1;     /* led1 blink for 1sec period */
      counter = CFG_LED_BLINK_TIME;
    }else{
        /* no processing */
    }
  }
}
/*--------------------------------------------------*/
/** 
  * @brief  INT00 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   0: Interrupt Pin0
  */
/*--------------------------------------------------*/
void INT00_IRQHandler(void)
{
    /*--- IBIMC033: INTxCLR ---*/
    TSB_IB->IMC033 = (uint8_t)(0xC5);
    /* Dummy read is need. */
    {
        __IO uint8_t imc = TSB_IB->IMC033;
    }
    
    NVIC_DisableIRQ(INT00_IRQn);     /* interrupt pin 0 disable */
    
    proc = PROC_DMA_TRANS;
}
/**
 *  @}
 */ /* End of group UART_TRANS_DMA_Exported_functions */

/**
 *  @}
 */ /* End of group UART_TRANS_DMA */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
