/** 
 *******************************************************************************
 * @file    uart_io.c
 * @brief   Uart Input/Output
 * @version V1.0.1.0
 * $Date:: 2019-06-25 13:15:47 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "uart_io.h"

#if defined(__UART_IO_H)
/**
 *  @addtogroup Sample_Appli
 *  @{
 */

/** 
 *  @addtogroup UART_IO_IO
 *  @{
 */

/** 
 *  @defgroup UART_IO_IO_PRIVATE  Uart IO Private
 *  @brief    Uart IO Private Group.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/*----------------------*/
/*  UART Setting        */
/*----------------------*/
/*-------------------------------*/
/* Demo bord                     */
/*-------------------------------*/
    #define UART_IO_IO_CFG_INTTX_IRQ      (INTSC1TX_IRQn)               /*   TX :NVIC Interrupt Enable       */
    #define UART_IO_IO_CFG_INTRX_IRQ      (INTSC1RX_IRQn)               /*   RX :NVIC Interrupt Enable       */
    #define UART_IO_IO_CFG_INTERR_IRQ     (INTSC1ERR_IRQn)              /*   ERR :NVIC Interrupt Enable      */
    #define UART_IO_IO_CFG_UART_FSYSEN    (TSB_CG_FSYSMENA_IPMENA22)    /*   UART1 :Fsys Enable              */
    #define UART_IO_IO_CFG_PORT_FSYSEN    (TSB_CG_FSYSMENA_IPMENA16)    /*   PORTC :Fsys Enable              */
    #define UART_IO_IO_CFG_TX_IE          (TSB_PU_IE_PU5IE)             /*   TX :Input Control Register      */
    #define UART_IO_IO_CFG_TX_CR          (TSB_PU_CR_PU5C)              /*   TX :Output Control Register     */
    #define UART_IO_IO_CFG_TX_OD          (TSB_PU_OD_PU5OD)             /*   TX :Open-drain Control Register */
    #define UART_IO_IO_CFG_TX_PUP         (TSB_PU_PUP_PU5UP)            /*   TX :Pull-up Control Register    */
    #define UART_IO_IO_CFG_TX_PDN         (TSB_PU_PDN_PU5DN)            /*   TX :Pull-down Control Register  */
    #define UART_IO_IO_CFG_TX_DATA        (TSB_PU_DATA_PU5)             /*   TX :Data Register               */
    #define UART_IO_IO_CFG_TX_FR          (TSB_PU_FR1_PU5F1)            /*   TX :Function Register           */
    #define UART_IO_IO_CFG_RX_IE          (TSB_PU_IE_PU6IE)             /*   RX :Input Control Register      */
    #define UART_IO_IO_CFG_RX_CR          (TSB_PU_CR_PU6C)              /*   RX :Output Control Register     */
    #define UART_IO_IO_CFG_RX_OD          (TSB_PU_OD_PU6OD)             /*   RX :Open-drain Control Register */
    #define UART_IO_IO_CFG_RX_PUP         (TSB_PU_PUP_PU6UP)            /*   RX :Pull-up Control Register    */
    #define UART_IO_IO_CFG_RX_PDN         (TSB_PU_PDN_PU6DN)            /*   RX :Pull-down Control Register  */
    #define UART_IO_IO_CFG_RX_DATA        (TSB_PU_DATA_PU6)             /*   RX :Data Register               */
    #define UART_IO_IO_CFG_RX_FR          (TSB_PU_FR1_PU6F1)            /*   RX :Function Register           */

/*----------------------*/
/*  for Debug           */
/*----------------------*/
#if 0
    #define UART_IO_IO_CFG_UART_IO
#endif

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @name  NULL Pointer
 *  @brief NULL Pointer.
 *  @{
 */
#define UART_IO_IO_NULL           ((void *)0)
/**
 *  @}
 */ /* End of name NULL Pointer */
/** 
 *  @defgroup BaudRateConfiguration BaudRate Configuration
 *  @brief    BaudRate .
 *  @{
 */
#define CFG_BAUDRATE             ((uint32_t)(115200))
#define CFG_BAUDRATE_SET_VAL     ((uint32_t)(0x00A6002B))
#define CFG_BAUDRATE_SET_VAL_10M ((uint32_t)(0x00A50005))
/** 
 *  @}
 */ /* End of group BaudRateConfiguration */
/** 
 *  @defgroup UART_DataConfiguration  Data Configuration
 *  @brief    Data Configuration Macro Definisiton.
 *  @{
 */
#define UART_DATA_DIRECTION_LSB         ((uint32_t)0x00000000)    /*!< LSB first */
#define UART_STOP_BIT_1                 ((uint32_t)0x00000000)    /*!< 1 bit */
#define UART_PARITY_BIT_ODD             ((uint32_t)0x00000000)    /*!< Odd Parity  */
#define UART_PARITY_DISABLE             ((uint32_t)0x00000000)    /*!< Disable */
#define UART_DATA_LENGTH_8              ((uint32_t)0x00000001)    /*!< 8 bit */
 /**
 *  @}
 */ /* End of group UART_DataLength */
/** 
 *  @defgroup UART_Prescaler  Prescaler
 *  @brief    Prescaler Macro Definisiton.
 *  @{
 */
#define UART_PLESCALER_1                ((uint32_t)0x00000000)    /*!< 1/1   */
/**
 *  @}
 */ /* End of group UART_Prescaler */
/** 
 *  @defgroup UART_TxInterrupt  Tx Interrpt
 *  @brief    Available Transmit Interrupt Macro Definisiton.
 *  @{
 */
#define UART_TX_INT_ENABLE              ((uint32_t)0x00000040)    /*!< Available.     */
/**
 *  @}
 */ /* End of group UART_TxInterrupt */
/** 
 *  @defgroup UART_TxState  Tx State
 *  @brief    Transmitting State Macro Definisiton.
 *  @{
 */
#define UART_TX_STATE_RUN               ((uint32_t)0x00008000)    /*!< Run       */
/**
 *  @}
 */ /* End of group UART_TxState */
/** 
 *  @defgroup UART_RxInterrupt  Rx Interrpt
 *  @brief    Available Receive Interrupt Macro Definisiton.
 *  @{
 */
#define UART_RX_INT_ENABLE              ((uint32_t)0x00000010)    /*!< Available.     */
/**
 *  @}
 */ /* End of group UART_RxInterrupt */
/** 
 *  @defgroup UARTxDR  UARTxDR Register
 *  @brief    UARTxDR Register Definition.
 *  @{
 */
/* DR */
#define UARTxDR_DR_8BIT_MASK      ((uint32_t)0x000000FF)    /*!< DR     :Mask for 8bit */
/**
 *  @}
 */ /* End of group UARTxDR */
/** 
 *  @defgroup UARTxSR  UARTxSR Register
 *  @brief    UARTxSR Register Definition.
 *  @{
 */
/* TXEND */
#define UARTxSR_TXEND_MASK        ((uint32_t)0x00004000)    /*!< TEXND :Mask.               */
#define UARTxSR_TXEND_R_END       ((uint32_t)0x00004000)    /*!< TXEND :[read]  Transfer done. */
#define UARTxSR_TXEND_W_CLEAR     ((uint32_t)0x00004000)    /*!< TXEND :[write] Clear Flag. */
/* TXFF */
#define UARTxSR_TXFF_MASK         ((uint32_t)0x00002000)    /*!< TXFF  :Mask.               */
#define UARTxSR_TXFF_R_REACHED    ((uint32_t)0x00002000)    /*!< TXFF  :[read]  Reached the transfer level. */
#define UARTxSR_TXFF_W_CLEAR      ((uint32_t)0x00002000)    /*!< TXFF  :[write] Clear Flag. */
/* TLVL */
#define UARTxSR_TLVL_MASK         ((uint32_t)0x00000F00)    /*!< TLVL  :Mask.               */
/* RXEND */
#define UARTxSR_RXEND_MASK        ((uint32_t)0x00000040)    /*!< RXEND :Mask.               */
#define UARTxSR_RXEND_R_END       ((uint32_t)0x00000040)    /*!< RXEND :[read]  Receive done. */
#define UARTxSR_RXEND_W_CLEAR     ((uint32_t)0x00000040)    /*!< RXEND :[write] Clear Flag. */
/* RXFF */
#define UARTxSR_RXFF_MASK         ((uint32_t)0x00000020)    /*!< RXFF  :Mask.               */
#define UARTxSR_RXFF_R_REACHED    ((uint32_t)0x00000020)    /*!< RXFF  :[read]  Receive done. */
#define UARTxSR_RXFF_W_CLEAR      ((uint32_t)0x00000020)    /*!< RXFF  :[write] Clear Flag. */
/* RLVL */
#define UARTxSR_RLVL_MASK         ((uint32_t)0x0000000F)    /*!< RLVL  :Mask.               */
/**
 *  @}
 */ /* End of group UARTxSR */
/** 
 *  @defgroup UARTxERR  UARTxERR Register
 *  @brief    UARTxERR Register Definition.
 *  @{
 */
/* TRGERR */
#define UARTxERR_TRGERR_MASK      ((uint32_t)0x00000010)    /*!< TRGERR :Mask.               */
#define UARTxERR_TRGERR_R_ERR     ((uint32_t)0x00000010)    /*!< TRGERR :[read]  Error.      */
#define UARTxERR_TRGERR_W_CLEAR   ((uint32_t)0x00000010)    /*!< TRGERR :[write] Clear Flag. */
/* OVRERR */
#define UARTxERR_OVRERR_MASK      ((uint32_t)0x00000008)    /*!< OVRERR :Mask.               */
#define UARTxERR_OVRERR_R_ERR     ((uint32_t)0x00000008)    /*!< OVRERR :[read]  Error.      */
#define UARTxERR_OVRERR_W_CLEAR   ((uint32_t)0x00000008)    /*!< OVRERR :[write] Clear Flag. */
/* PERR */
#define UARTxERR_PERR_MASK        ((uint32_t)0x00000004)    /*!< PERR   :Mask.               */
#define UARTxERR_PERR_R_ERR       ((uint32_t)0x00000004)    /*!< PERR   :[read]  Error.      */
#define UARTxERR_PERR_W_CLEAR     ((uint32_t)0x00000004)    /*!< PERR   :[write] Clear Flag. */
/* FERR */
#define UARTxERR_FERR_MASK        ((uint32_t)0x00000002)    /*!< FERR   :Mask.               */
#define UARTxERR_FERR_R_ERR       ((uint32_t)0x00000002)    /*!< FERR   :[read]  Error.      */
#define UARTxERR_FERR_W_CLEAR     ((uint32_t)0x00000002)    /*!< FERR   :[write] Clear Flag. */
/* BERR */
#define UARTxERR_BERR_MASK        ((uint32_t)0x00000001)    /*!< BERR   :Mask.               */
#define UARTxERR_BERR_R_ERR       ((uint32_t)0x00000001)    /*!< BERR   :[read]  Error.      */
#define UARTxERR_BERR_W_CLEAR     ((uint32_t)0x00000001)    /*!< BERR   :[write] Clear Flag. */
/**
 *  @}
 */ /* End of group UARTxERR */
/** 
 *  @defgroup UART_FifoMax  Max Num of FIFO
 *  @brief    Max Num of Tx/Rx Fifo.
 *  @{
 */
#define UART_TX_FIFO_MAX                ((uint32_t)0x00000008)    /*!< TX FIFO Max.   */
#define UART_RX_FIFO_MAX                ((uint32_t)0x00000008)    /*!< RX FIFO Max.   */
/**
 *  @}
 */ /* End of group UART_FifoMax */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @enum   SendState
 * @brief  Send state.
*/
/*----------------------------------*/
typedef enum {
    SEND_IDLE = 0U,     /*!< Idle                   */
    SEND_RUN,           /*!< Transmiting.           */
    SEND_SUCCESS,       /*!< Transmission complete. */
    SEND_FAILURE        /*!< Transmission error.    */
} SendState;

/*----------------------------------*/
/** 
 * @enum   ReceiveState
 * @brief  Receive state.
*/
/*----------------------------------*/
typedef enum {
    RECV_IDLE = 0U,     /*!< Idle                   */
    RECV_RUN,           /*!< Receiving              */
    RECV_FAILURE        /*!< Received error.        */
} ReceiveState;

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @struct send_t
 * @brief  Send information.
*/
/*----------------------------------*/
typedef struct
{
    volatile SendState  state;                          /*!< Status.                     */
} send_t;

/*----------------------------------*/
/** 
 * @struct receive_t
 * @brief  Receive information.
*/
/*----------------------------------*/
typedef struct
{
    volatile ReceiveState state;                        /*!< Status.                     */
              int32_t      wp;                           /*!< Write pointer.              */
              int32_t      rp;                           /*!< Read pointer.               */
    volatile uint32_t     length;                       /*!< Num of receive data.        */
    volatile uint8_t      buff[UART_IO_RECEIVE_MAX];   /*!< Receive data.               */
} receive_t;

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
static uint8_t      rxWork[UART_RX_FIFO_MAX];       /*!< Receive buffer for uart driver . */
static send_t       send;                                         /*!< Send information.     */
static receive_t    receive;                                    /*!< Receive information.  */

/*------------------------------------------------------------------------------*/
/*  Const                                                                       */
/*------------------------------------------------------------------------------*/
/* no define */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
static void debug_error(void);
static void transmit_handler(TXZ_Result result);
static void receive_handler(TXZ_Result result, uint32_t num);
static void UART_Config(void);
/*--------------------------------------------------*/
/** 
  * @brief  Error Hook
  * @param  -
  * @retval -
  * @note   for debug
  */
/*--------------------------------------------------*/
static void debug_error(void)
{
#ifdef UART_IO_IO_CFG_UART_IO
    while(1)
    {
        __nop();
    }
#endif
}
/*--------------------------------------------------*/
/** 
  * @brief      Transmit handler.
  * @param[in]  result  :Result.
  * @retval     -
  * @note       Called by uart driver
  */
/*--------------------------------------------------*/
static void transmit_handler(TXZ_Result result)
{
    if (send.state != SEND_IDLE)
    {
        /*------------------------------*/
        /*  Result Check                */
        /*------------------------------*/
        if (result == TXZ_SUCCESS)
        {
            send.state = SEND_SUCCESS;
        }
        else
        {
            send.state = SEND_FAILURE;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief      Receive handler.
  * @param[in]  result  :Result.
  * @param[in]  num  :  :number of data.
  * @retval     -
  * @note       Called by uart driver
  */
/*--------------------------------------------------*/
static void receive_handler(TXZ_Result result, uint32_t num)
{
    if (receive.state == RECV_RUN)
    {
        if (result == TXZ_SUCCESS)
        {
            if (receive.length < UART_IO_RECEIVE_MAX) 
            {
                uint32_t  i;
                
                for(i=0; i<num; ++i)
                {
                    receive.buff[receive.wp] = rxWork[i];
                    if (++receive.wp >= UART_IO_RECEIVE_MAX)
                    {
                        receive.wp = 0;
                    }
                    /*>>> Critical information. */
                    ++receive.length;
                    /*<<< Critical information. */
                    if (receive.length >= UART_IO_RECEIVE_MAX) 
                    {
                        break;
                    }
                }
            }
            else
            {
                receive.state = RECV_FAILURE;
            }
        }
        else
        {
            receive.state = RECV_FAILURE;
        }
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_UART_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void UART_Config(void)
{
    UART_IO_IO_CFG_UART_FSYSEN = 1;
    /* (1) Enable the peripheral clock of UART1  */
    TSB_UART1->SWRST = 0x2U;
    TSB_UART1->SWRST = 0x1U;
    
    while(TSB_UART1->SWRST==  0x80)
    {
    }
    SystemCoreClockUpdate();
    /* (2) oversampling by 115200 baud */
    if(SystemCoreClock == 160000000)
    {
        TSB_UART1->BRD = CFG_BAUDRATE_SET_VAL;
        TSB_UART1->CLK = (UART_PLESCALER_1 << 4);
    }
    else if(SystemCoreClock == 10000000)
    {
        TSB_UART1->BRD = CFG_BAUDRATE_SET_VAL_10M;
        TSB_UART1->CLK = (UART_PLESCALER_1 << 4);
    }
    /* (3) 8 data bit, 1 start bit, 1 stop bit, no parity */
    TSB_UART1->CR0 = (UART_DATA_DIRECTION_LSB | UART_STOP_BIT_1 | UART_PARITY_BIT_ODD | \
                    UART_PARITY_DISABLE | UART_DATA_LENGTH_8);
    /* (4) Transmit interrupt Enable    */
    TSB_UART1->CR1 = (UART_RX_INT_ENABLE | UART_TX_INT_ENABLE);
    /*--- UARTxTRANS ---*/
    TSB_UART1_TRANS_RXE = 1;
}

/**
 *  @}
 */ /* End of group UART_IO_IO_PRIVATE */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/*--------------------------------------------------*/
/** 
  * @brief  Initialize
  * @param  -
  * @retval -
  * @pre    -
  * @note   -
  */
/*--------------------------------------------------*/
void uart_io_initialize(void)
{
    /*------------------------------*/
    /*  Clear RAM                   */
    /*------------------------------*/
    send.state      = SEND_IDLE;
    receive.state   = RECV_RUN;     /* Auto run. */
    receive.wp      = 0;
    receive.rp      = 0;
    receive.length  = 0;
    /*------------------------------*/
    /*  Port CG Fsys Enable         */
    /*------------------------------*/
    UART_IO_IO_CFG_PORT_FSYSEN = 1;
    /*------------------------------*/
    /*  Port Setting                */
    /*------------------------------*/
    /* rx */
    UART_IO_IO_CFG_RX_OD      = 0;    /* OD Control :CMOS    */
    UART_IO_IO_CFG_RX_PUP     = 0;    /* Pull-up    :Disable */
    UART_IO_IO_CFG_RX_PDN     = 0;    /* Pull-down  :Disable */
    UART_IO_IO_CFG_RX_DATA    = 0;    /* Data       :0       */
    UART_IO_IO_CFG_RX_IE      = 1;    /* Input      :Enable  */
    UART_IO_IO_CFG_RX_FR      = 1;    /* Function   :RXD     */
    UART_IO_IO_CFG_RX_CR      = 0;    /* Output     :Disable */
    /* tx */
    UART_IO_IO_CFG_TX_OD      = 0;    /* OD Control :CMOS    */
    UART_IO_IO_CFG_TX_PUP     = 0;    /* Pull-up    :Disable */
    UART_IO_IO_CFG_TX_PDN     = 0;    /* Pull-down  :Disable */
    UART_IO_IO_CFG_TX_DATA    = 0;    /* Data       :0       */
    UART_IO_IO_CFG_TX_IE      = 0;    /* Input      :Disable */
    UART_IO_IO_CFG_TX_FR      = 1;    /* Function   :TXD     */
    UART_IO_IO_CFG_TX_CR      = 1;    /* Output     :Enable  */
    /*------------------------------*/
    /*  UART Driver                 */
    /*------------------------------*/
    UART_Config();
    /*------------------------------*/
    /*  Enable Interrupt            */
    /*------------------------------*/
    NVIC_EnableIRQ(UART_IO_IO_CFG_INTTX_IRQ);
    NVIC_EnableIRQ(UART_IO_IO_CFG_INTRX_IRQ);
    NVIC_EnableIRQ(UART_IO_IO_CFG_INTERR_IRQ);
}

/*--------------------------------------------------*/
/** 
  * @brief  Finalize
  * @param  -
  * @retval -
  * @pre    -
  * @note   -
  */
/*--------------------------------------------------*/
void uart_io_finalize(void)
{
    /*------------------------------*/
    /*  Disable Enable              */
    /*------------------------------*/
    NVIC_DisableIRQ(UART_IO_IO_CFG_INTTX_IRQ);
    NVIC_DisableIRQ(UART_IO_IO_CFG_INTRX_IRQ);
    NVIC_DisableIRQ(UART_IO_IO_CFG_INTERR_IRQ);
    /*------------------------------*/
    /*  Uart Driver Finalize        */
    /*------------------------------*/
    TSB_UART1->TRANS = 0;   /* UARTxTRANS_BK_STOP     | UARTxTRANS_TXTRG_DISABLE | 
                               UARTxTRANS_TXE_DISABLE | UARTxTRANS_RXE_DISABLE */
    /*--- UARTxCR1 ---*/
    TSB_UART1->CR1   = 0;
    /*--- UARTxCR0 ---*/
    TSB_UART1->CR0   = 0;
    /*------------------------------*/
    /*  Uart CG Fsys Disable        */
    /*------------------------------*/
    UART_IO_IO_CFG_UART_FSYSEN = 0;
    /*------------------------------*/
    /*  Port Setting                */
    /*------------------------------*/
    /* rx */
    UART_IO_IO_CFG_RX_IE      = 0;    /* Input      :Disable */
    UART_IO_IO_CFG_RX_CR      = 0;    /* Output     :Disable */
    UART_IO_IO_CFG_RX_OD      = 0;    /* OD Control :CMOS    */
    UART_IO_IO_CFG_RX_PUP     = 0;    /* Pull-up    :Disable */
    UART_IO_IO_CFG_RX_PDN     = 0;    /* Pull-down  :Disable */
    UART_IO_IO_CFG_RX_DATA    = 0;    /* Data       :0       */
    UART_IO_IO_CFG_RX_FR      = 0;    /* Function   :Port    */
    /* tx */
    UART_IO_IO_CFG_TX_IE      = 0;    /* Input      :Disable */
    UART_IO_IO_CFG_TX_CR      = 0;    /* Output     :Disable */
    UART_IO_IO_CFG_TX_OD      = 0;    /* OD Control :CMOS    */
    UART_IO_IO_CFG_TX_PUP     = 0;    /* Pull-up    :Disable */
    UART_IO_IO_CFG_TX_PDN     = 0;    /* Pull-down  :Disable */
    UART_IO_IO_CFG_TX_DATA    = 0;    /* Data       :0       */
    UART_IO_IO_CFG_TX_FR      = 0;    /* Function   :Port    */
    /*------------------------------*/
    /*  Port CG Fsys Disable        */
    /*------------------------------*/
    UART_IO_IO_CFG_PORT_FSYSEN = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief       The character is shown to an output stream.
  * @param[in]   c      :Character data.
  * @retval      Suceess:c
  * @retval      Failure:-1
  * @note        -
  */
/*--------------------------------------------------*/
int uart_io_put_char(int c)
{
    int ret = -1;

    if (send.state == SEND_IDLE)
    {
        /*------------------------------*/
        /*  Disable Transmit            */
        /*------------------------------*/
        /*--- UARTxTRANS ---*/
        /* Bitband Access.                 */
        TSB_UART1_TRANS_TXE = 0;
        /*------------------------------*/
        /*  Information Setting         */
        /*------------------------------*/
        send.state   = SEND_RUN;
        /*------------------------------*/
        /*  Data Setting                */
        /*------------------------------*/
         TSB_UART1->DR  = ((uint8_t)c & UARTxDR_DR_8BIT_MASK);
        /*------------------------------*/
        /*  Enable Transmit             */
        /*------------------------------*/
        /*--- UARTxTRANS ---*/
        /* Bitband Access.                 */
        TSB_UART1_TRANS_TXE = 1;
        /*------------------------------*/
        /*  Wait to transmit            */
        /*------------------------------*/
        while(send.state == SEND_RUN)
        {
        }
        /*------------------------------*/
        /*  Check Error                 */
        /*------------------------------*/
        if (send.state == SEND_FAILURE)
        {
            debug_error();
        }
        /* Clear status. */
        send.state = SEND_IDLE;
        /* Set result. */
        ret = c;
    }

    return (ret);
}

/*--------------------------------------------------*/
/** 
  * @brief  The character is acquired from an input stream.
  * @param  -
  * @retval Suceess     :Input stream.
  * @retval Failure     :-1
  * @note   -
  */
/*--------------------------------------------------*/
int uart_io_get_char(void)
{
    volatile int c = -1;
    
    while (1)
    {
        if (receive.state == RECV_RUN)
        {
            /*>>> Critical information. */
            volatile uint32_t length = receive.length;
            /*<<< Critical information. */
            
            /*------------------------------*/
            /*  Check Receive Length        */
            /*------------------------------*/
            if (length > 0)
            {
                c = (int)receive.buff[receive.rp];
                if (++receive.rp >= UART_IO_RECEIVE_MAX)
                {
                    receive.rp = 0;
                }
                /*>>> Critical information. */
                receive.length--;
                /*<<< Critical information. */
                break;
            }
        }
        else
        {
            break;
        }
    }

    return (c);
}

/*--------------------------------------------------*/
/** 
  * @brief  TX IRQ Handler.
  * @param  -
  * @retval -
  * @pre    -
  * @note   -
  */
/*--------------------------------------------------*/
void INTSC1TX_IRQHandler(void)
{
    uint32_t status;

    /*------------------------------*/
    /*  Status Registar Control     */
    /*------------------------------*/
    /* Read current UARTxSR. */
    status = TSB_UART1->SR; 
    /* Clear the transmit's end flag.    */
    /* Write to TXEND(=1), and TXFF(=1). */
    TSB_UART1->SR = (UARTxSR_TXEND_W_CLEAR | UARTxSR_TXFF_W_CLEAR);
    /*------------------------------*/
    /*  Transmit Status Check       */
    /*------------------------------*/
    if (TSB_UART1_TRANS_RXE == 1)
    {
        /*---- UARTxSR ---*/
        /* Check the transmit's end flag. */
        if ((status & UARTxSR_TXEND_MASK) == UARTxSR_TXEND_R_END)
        {
            /*=== Transmit Done!! ===*/
            /*------------------------------*/
            /*  Disable Transmit            */
            /*------------------------------*/
            /*--- UARTxTRANS ---*/
            /* Bitband Access.                 */
            TSB_UART1_TRANS_TXE = 0;
            /*------------------------------*/
            /* Call Handler                 */
            /*------------------------------*/
            transmit_handler(TXZ_SUCCESS);
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  RX IRQ Handler.
  * @param  -
  * @retval -
  * @pre    -
  * @note   -
  */
/*--------------------------------------------------*/
void INTSC1RX_IRQHandler(void)
{
    uint32_t status;
    /*------------------------------*/
    /*  Status Registar Control     */
    /*------------------------------*/
    /* Read current UARTxSR. */
    status = TSB_UART1->SR;
    /* Clear the transmit's end flag.    */
    /* Write to RXEND(=1), and RXFF(=1). */
    TSB_UART1->SR = (UARTxSR_RXEND_W_CLEAR | UARTxSR_RXFF_W_CLEAR);
    /*------------------------------*/
    /*  Receive Status Check        */
    /*------------------------------*/
    if (TSB_UART1_TRANS_RXE == 1)
    {
        /* Check the receive's end flag. */
        if ((status & UARTxSR_RXEND_MASK) == UARTxSR_RXEND_R_END)
        {
            /* Read FIFO fill level. */
            uint32_t i;
            uint32_t rlvl = (status & UARTxSR_RLVL_MASK);
            /* FIFO Max = UART_RX_FIFO_MAX */
            if (rlvl > UART_RX_FIFO_MAX)
            {
                rlvl = UART_RX_FIFO_MAX;
            }
            for(i=0; i<rlvl; ++i)
            {
                rxWork[i]  =  (uint8_t)(TSB_UART1->DR & UARTxDR_DR_8BIT_MASK);
            }
            /*------------------------------*/
            /* Call Handler                 */
            /*------------------------------*/
            /* Call the receive handler with TXZ_SUCCESS. */
            receive_handler(TXZ_SUCCESS,  rlvl);
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  ERR IRQ Handler.
  * @param  -
  * @retval -
  * @pre    -
  * @note   -
  */
/*--------------------------------------------------*/
void INTSC1ERR_IRQHandler(void)
{
    uint32_t error;
    /*------------------------------*/
    /*  Error Registar Control      */
    /*------------------------------*/
    /* Read current UARTxERR.           */
    error = TSB_UART1->ERR;
    /* Now, no clear the error flag.    */
    /*------------------------------*/
    /*  Error Check                 */
    /*------------------------------*/
    /*--- UARTxERR ---*/
    /* Check the transmit error. */
    /* TRGERR */
    if ((error & UARTxERR_TRGERR_MASK) == UARTxERR_TRGERR_R_ERR)
    {
        /*------------------------------*/
        /*  Transmit Check              */
        /*------------------------------*/
        if (TSB_UART1_TRANS_TXE == 1)
        {
            /*------------------------------*/
            /*  Disable Transmit            */
            /*------------------------------*/
            /*--- UARTxTRANS ---*/
            /* Bitband Access.                 */
            TSB_UART1_TRANS_TXE = 0;
            /*------------------------------*/
            /* Call Handler                 */
            /*------------------------------*/
                /* Call the transmit handler with TXZ_ERROR. */
            transmit_handler(TXZ_ERROR);
        }
    }
    /* Check the receive error. */
    {
        if ((error & (UARTxERR_OVRERR_MASK | UARTxERR_PERR_MASK | UARTxERR_FERR_MASK | UARTxERR_BERR_MASK)) != 0)
        {
            /*------------------------------*/
            /*  Receive Check               */
            /*------------------------------*/
            if (TSB_UART1_TRANS_RXE == 1)
            {
                /*------------------------------*/
                /*  Disable Receive             */
                /*------------------------------*/
                /*--- UARTxTRANS ---*/
                /* Bitband Access.                 */
                TSB_UART1_TRANS_RXE = 0;
                /*------------------------------*/
                /* Call Handler                 */
                /*------------------------------*/
                /* Call the receive handler with TXZ_ERROR. */
                receive_handler(TXZ_ERROR, 0);
            }
        }
    }
}

/*------------------------------------------------------------------------------*/
/*  Semihosting Retarget                                                        */
/*------------------------------------------------------------------------------*/
#if defined ( __CC_ARM   )      /* RealView Compiler */
struct __FILE { int handle; /* Add whatever you need here */ };
FILE __stdout;
FILE __stdin;
int fputc(int ch, FILE *f) {
#elif defined ( __ICCARM__ )    /*IAR Compiler */
int putchar(int ch) {
#endif
    return (uart_io_put_char(ch));
}

#if defined ( __CC_ARM   )      /* RealView Compiler */
int getc(FILE * f)
#elif defined ( __ICCARM__ )    /*IAR Compiler */
int getchar(void)
#endif
{
    return (uart_io_get_char());
}

/**
 *  @} 
 */ /* End of group UART_IO_IO */

/**
 *  @} 
 */ /* End of group Sample_Appli */

#endif /* defined(__UART_IO_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */
