/** 
 *******************************************************************************
 * @file    uart_io_dma.c
 * @brief   UART input/output
 * @version V1.0.0.0
 * $Date:: 2018-10-26 13:24:12 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdarg.h>
#include "uart_io_dma.h"

#if defined(__UART_IO_DMA_H)
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup UART_IO_DMA
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_IO_DMA_Private_macro
 *  @{
 */

/** 
 *  @addtogroup DMA_Exported_macro
 *  @{
 */
#define DMA_EX_TRANS_NUM(x)   ((uint32_t)((uint32_t)((x) - 1) << 4))  /*!< The number of transfer times(for Channel configration control). */
/** 
 *  @}
 */ /* End of group DMA_Exported_macro */

/** 
 *  @}
 */ /* End of group UART_IO_DMA_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_IO_DMA_Private_define
 *  @{
 */
/** 
 *  @defgroup DMA_Exported_define DMA Exported Define
 *  @{
 */
/** 
 *  @defgroup DataLengtheConfiguration Data Length Configuration
 *  @brief    Data Length Configuration.
 *  @{
 */
#define CFG_DATA_LENGTH            ((uint32_t)16)
/** 
 *  @}
 */ /* End of group DataLengtheConfiguration */
/* master_enable */
#define CFG_MASTER_DISABLE       ((uint32_t)0x00000000)    /*!< master_enable :Disable.   */
#define CFG_MASTER_ENABLE        ((uint32_t)0x00000001)    /*!< master_enable :Enable.    */
/* ctrl_base_ptr  */
#define CTRLBASEPTR_MASK         ((uint32_t)0xFFFFFC00)    /*!< ctrl_base_ptr :Mask.      */
/* chnl_req_mask_set  */
#define CHNLREQMASKSET_ALL_MASK  ((uint32_t)0xFFFFFFFF)    /*!< chnl_req_mask_set :All Mask. */
/* chnl_enable_clr  */
#define CHNLENABLECLR_ALL_DISABLE   ((uint32_t)0xFFFFFFFF)    /*!< chnl_enable_clr :All Mask. */
/* err_clr  */
#define ERRCLR_MASK_BUS_W_CLR   ((uint32_t)0x00000001)    /*!< err_clr:[write] Clear.        */
/** 
 *  @defgroup DMA_ChannelNum  Channel Num
 *  @brief    DMA Channel.
 *  @{
 */
#define DMA_CH_NUM                      ((uint32_t)32)            /*!< Num of total channel (1<= value <= 32). */
/**
 *  @}
 */ /* End of group DMA_ChannelNum */

/** 
 *  @defgroup DMA_DestinationInc  Destination Address Increment Type
 *  @brief    Destination address increment type.
 *  @{
 */
#define DMA_DST_INC_1BYTE               ((uint32_t)0x00000000)    /*!< 1 byte.               */
#define DMA_DST_INC_2BYTE               ((uint32_t)0x40000000)    /*!< 2 byte.               */
#define DMA_DST_INC_4BYTE               ((uint32_t)0x80000000)    /*!< 4 byte.               */
#define DMA_DST_INC_NON                 ((uint32_t)0xC0000000)    /*!< Without increment.    */
/**
 *  @}
 */ /* End of group DMA_DestinationInc */

/** 
 *  @defgroup DMA_DestinationSize  Destination Data Size
 *  @brief    Destination data size.
 *  @{
 */
#define DMA_DST_SIZE_1BYTE              ((uint32_t)0x00000000)    /*!< 1 byte.               */
#define DMA_DST_SIZE_2BYTE              ((uint32_t)0x10000000)    /*!< 2 byte.               */
#define DMA_DST_SIZE_4BYTE              ((uint32_t)0x20000000)    /*!< 4 byte.               */
/**
 *  @}
 */ /* End of group DMA_DestinationSize */

/** 
 *  @defgroup DMA_SourceInc  Source Address Increment Type
 *  @brief    Source address increment type.
 *  @{
 */
#define DMA_SRC_INC_1BYTE               ((uint32_t)0x00000000)    /*!< 1 byte.               */
#define DMA_SRC_INC_2BYTE               ((uint32_t)0x04000000)    /*!< 2 byte.               */
#define DMA_SRC_INC_4BYTE               ((uint32_t)0x08000000)    /*!< 4 byte.               */
#define DMA_SRC_INC_NON                 ((uint32_t)0x0C000000)    /*!< Without increment.    */
/**
 *  @}
 */ /* End of group DMA_SourceInc */

/** 
 *  @defgroup DMA_SourceSize  Source Data Size
 *  @brief    Source data size.
 *  @{
 */
#define DMA_SRC_SIZE_1BYTE              ((uint32_t)0x00000000)    /*!< 1 byte.               */
#define DMA_SRC_SIZE_2BYTE              ((uint32_t)0x01000000)    /*!< 2 byte.               */
#define DMA_SRC_SIZE_4BYTE              ((uint32_t)0x02000000)    /*!< 4 byte.               */
/**
 *  @}
 */ /* End of group DMA_SourceSize */

/** 
 *  @defgroup DMA_Arbitration  Arbitration Timing
 *  @brief    Arbitration timing.
 *  @{
 */
#define DMA_ARBITRATION_1               ((uint32_t)0x00000000)    /*!< After   1 Transfer.   */
#define DMA_ARBITRATION_NON             ((uint32_t)0x0003C000)    /*!< Without arbitration.  */
/**
 *  @}
 */ /* End of group DMA_Arbitration */

/** 
 *  @defgroup DMA_TransferNumRange  Transfer Num Range
 *  @brief    Range of Transfer Num Macro Definisiton.
 *  @brief    Range of Transfer Num be set "(DMA_TRANS_NUM_RANGE_MIN <= Value <= DMA_TRANS_NUM_RANGE_MAX)".
 *  @{
 */
#define DMA_TRANS_NUM_RANGE_MIN         ((uint32_t)0x00000001)    /*!< Minimum  Value :1    */
#define DMA_TRANS_NUM_RANGE_MAX         ((uint32_t)0x00000400)    /*!< Maximum  Value :1024 */
/**
 *  @}
 */ /* End of group DMA_TransferNumRange */

/** 
 *  @defgroup DMA_Burst  Use Burst
 *  @brief    Use burst transfer.
 *  @{
 */
#define DMA_BURST_UNUSE                 ((uint32_t)0x00000000)    /*!< Unuse.  */
#define DMA_BURST_USE                   ((uint32_t)0x00000008)    /*!< Use.    */
/**
 *  @}
 */ /* End of group DMA_Burst */

/** 
 *  @defgroup DMA_CycleControl  Cycle Control Mode
 *  @brief    DMA cycle control mode.
 *  @{
 */
#define DMA_CYCLE_MODE_DISABLE          ((uint32_t)0x00000000)    /*!< Stop cycle.                          */
#define DMA_CYCLE_MODE_BASE             ((uint32_t)0x00000001)    /*!< Basic mode.                          */
#define DMA_CYCLE_MODE_CNT_BASE         ((uint32_t)0x00000002)    /*!< Continuation Basic mode.             */
#define DMA_CYCLE_MODE_REPEAT           ((uint32_t)0x00000003)    /*!< Repeat mode.                         */
#define DMA_CYCLE_MODE_CNT_CHAIN_1      ((uint32_t)0x00000004)    /*!< Continuation Chain mode. (1st data). */
#define DMA_CYCLE_MODE_CNT_CHAIN_2      ((uint32_t)0x00000005)    /*!< Continuation Chain mode. (2nd data). */
#define DMA_CYCLE_MODE_CHAIN_1          ((uint32_t)0x00000006)    /*!< Unit Chain mode. (1st data).         */
#define DMA_CYCLE_MODE_CHAIN_2          ((uint32_t)0x00000007)    /*!< Unit Chain mode. (2nd data).         */
/**
 *  @}
 */ /* End of group DMA_CycleControl */

/** 
 *  @defgroup DMA_Priority  Priority
 *  @brief    Priority.
 *  @{
 */
#define DMA_PRIORITY_NORMAL             ((uint32_t)0x00000000)    /*!< Normal.    */
#define DMA_PRIORITY_HIGH               ((uint32_t)0x00000001)    /*!< High.      */
/**
 *  @}
 */ /* End of group DMA_Priority */
/** 
 *  @defgroup ChannelCfg  ChannelCfg
 *  @brief    ChannelCfg Definition.
 *  @{
 */
#define CH_CFG_MASK_DST_INC     ((uint32_t)0xC0000000)    /*!< Mask: dst_inc.      */
#define CH_CFG_MASK_DST_SIZE    ((uint32_t)0x30000000)    /*!< Mask: dst_size.     */
#define CH_CFG_MASK_SRC_INC     ((uint32_t)0x0C000000)    /*!< Mask: src_inc.      */
#define CH_CFG_MASK_SRC_SIZE    ((uint32_t)0x03000000)    /*!< Mask: src_size.     */
#define CH_CFG_MASK_TRANS_NUM   ((uint32_t)0x00003FF0)    /*!< Mask: trans num.    */
#define CH_CFG_MASK_CYCLE_CTRL  ((uint32_t)0x00000007)    /*!< Mask: cycle ctrl.   */
/**
 *  @}
 */ /* End of group ChannelCfg */

/**
 *  @}
 */ /* End of group DMA_Exported_define */
/*----------------------*/
/*  for Debug           */
/*----------------------*/
#ifdef DEBUG
    #if 1
        #define UART_IO_CFG_DEBUG
    #endif
#endif
/** 
 *  @}
 */ /* End of group UART_IO_DMA_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_IO_DMA_Private_define
 *  @{
 */

/*----------------------*/
/*  DMA Setting         */
/*----------------------*/
/* Work Area = ((The total number of the used channel) * (4 * DWORD) * 2)  */
#define CFG_DMA_WORK_SIZE     ((uint32_t)(DMA_CH_NUM * 4 * sizeof(uint32_t) * 2))     /*!< DMA work area. */

#define UART1RX_DMAREQ_CH   6
#define UART1TX_DMAREQ_CH   7
/** 
 *  @}
 */ /* End of group UART_IO_DMA_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_IO_DMA_Private_define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Send state.
*/
/*----------------------------------*/
typedef enum {
    UART_IO_DMA_SEND_IDLE = 0U,     /*!< Idle                   */
    UART_IO_DMA_SEND_RUN,           /*!< Transmiting.           */
    UART_IO_DMA_SEND_SUCCESS,       /*!< Transmission complete. */
    UART_IO_DMA_SEND_FAILURE        /*!< Transmission error.    */
} UartIODmaTxState;

/*----------------------------------*/
/** 
 * @brief  Receive state.
*/
/*----------------------------------*/
typedef enum {
    UART_IO_DMA_RECV_IDLE = 0U,     /*!< Idle                   */
    UART_IO_DMA_RECV_RUN,           /*!< Receiving              */
    UART_IO_DMA_RECV_SUCCESS,       /*!< Received complete.     */
    UART_IO_DMA_RECV_FAILURE        /*!< Received error.        */
} UartIODmaRxState;

/** 
 *  @}
 */ /* End of group UART_IO_DMA_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_IO_DMA_Private_typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Control data structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t srcEnd;        /*!< Source end pointer.      */
    uint32_t dstEnd;        /*!< Destination end pointer. */
    uint32_t control;       /*!< Control data.            */
    uint32_t reserved;      /*!< Reserved.                */
} control_data_t;

/*----------------------------------*/
/** 
 * @brief  USB UART Send information.
 * @brief  Private Member.
*/
/*----------------------------------*/
typedef struct
{
    volatile UartIODmaTxState   state;  /*!< Status.                     */
} uart_io_dma_private_tx_t;

/*----------------------------------*/
/** 
 * @brief  USB UART Receive information.
 * @brief  Private Member.
*/
/*----------------------------------*/
typedef struct
{
    volatile UartIODmaRxState   state;  /*!< Status.                     */
} uart_io_dma_private_rx_t;

/*----------------------------------*/
/** 
 * @brief  USB UART Private Member.
*/
/*----------------------------------*/
typedef struct
{
    uart_io_dma_private_tx_t    tx;     /*!< Send information.           */
    uart_io_dma_private_rx_t    rx;     /*!< Receive information.        */
} uart_io_dma_private_info_t;

/*----------------------------------*/
/** 
 * @brief  USB UART class.
*/
/*----------------------------------*/
typedef struct
{
    uart_io_dma_private_info_t    info; /*!< Private information. */
} uart_io_dma_t;
typedef struct
{
    uint32_t      source;       /*!< Source data start pointer.      */
    uint32_t      destination;  /*!< Destination data start pointer. */
    uint32_t      control;      /*!< Control.                        */
} dma_normal_cfg_t;

/**
 *  @}
 */ /* End of group UART_IO_DMA_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_IO_DMA_Private_variables
 *  @{
 */
extern uint8_t rx_data[CFG_DATA_LENGTH];
static uint8_t stringtosend[] = "TMPM4KQ\n";
static uart_io_dma_t    instance;   /*!< USB UART Instance. */
#if defined ( __CC_ARM   )      /* RealView Compiler */
    static __align(0x400) uint8_t primary[CFG_DMA_WORK_SIZE]; /*!< DMA work area. */
#elif defined ( __ICCARM__ )    /*IAR Compiler */
    #pragma location="dma_work"
    #pragma data_alignment=1024
    static uint8_t primary[CFG_DMA_WORK_SIZE]; /*!< DMA work area. */
#endif
/**
 *  @}
 */ /* End of group UART_IO_DMA_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_IO_DMA_Private_fuctions
 *  @{
 */
__STATIC_INLINE uint32_t convert_ch_to_reg(uint32_t ch);
__STATIC_INLINE uint32_t convert_transfer_num_to_if(uint32_t num);
__STATIC_INLINE uint32_t get_end_address(uint32_t start, uint32_t inc, uint32_t size, uint32_t num);
static void uart_io_error(void);
void transmit_handler(TXZ_Result result);
void receive_handler(TXZ_Result result);
TXZ_Result dma_normal_startIt(uint32_t ch, uint32_t burst, dma_normal_cfg_t *p_cfg);
/*--------------------------------------------------*/
/** 
  * @brief  Convert channel to register.
  * @param  ch :Channel(Interface format).
  * @retval Channel(Register format).
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE uint32_t convert_ch_to_reg(uint32_t ch)
{
    uint32_t result = (uint32_t)((uint32_t)1 << (uint8_t)ch);

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Convert transfer num to interface.
  * @param  num :Transfer num(Register format).
  * @retval Transfer num(Interface format).
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE uint32_t convert_transfer_num_to_if(uint32_t num)
{
    uint32_t result = (uint32_t)((uint32_t)(num >> 4) + 1);

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Get Tansfer End Address.
  * @param  start :Start Address.
  * @param  inc   :Address increment type.
  * @param  size  :One data size.
  * @param  num   :Num of transfer data. 
  * @retval Tansfer End Address.
  * @note   Must be set (num > 0).
  */
/*--------------------------------------------------*/
__STATIC_INLINE uint32_t get_end_address(uint32_t start, uint32_t inc, uint32_t size, uint32_t num)
{
    uint32_t result = (uint32_t)(start + (uint32_t)((uint32_t)(num - 1) * inc));

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Error Hook
  * @param  -
  * @return -
  * @retval -
  * @note   for debug
  */
/*--------------------------------------------------*/
static void uart_io_error(void)
{
#ifdef UART_IO_CFG_DEBUG
    while(1)
    {
        __nop();
    }
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA Trans start.
  * @param  num
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void UART_DMA_Trans(uint32_t num)
{
    if (instance.info.tx.state == UART_IO_DMA_SEND_IDLE)
    {
		instance.info.tx.state = UART_IO_DMA_SEND_RUN;
        /*------------------------------*/
        /*  Disable Transmit            */
        /*------------------------------*/
        /*--- UARTxTRANS ---*/
        /* Bitband Access.                 */
        TSB_UART1_TRANS_TXE = 0;
        {
            dma_normal_cfg_t cfg;
            
            cfg.destination = (uint32_t)&TSB_UART1->DR;
            cfg.source  = (uint32_t)&stringtosend[0];
            cfg.control = (uint32_t)(DMA_DST_INC_NON     | DMA_DST_SIZE_1BYTE |
                                     DMA_SRC_INC_1BYTE   | DMA_SRC_SIZE_1BYTE |
                                     DMA_ARBITRATION_1   | DMA_EX_TRANS_NUM(num) |
                                     DMA_BURST_UNUSE     | DMA_CYCLE_MODE_BASE);
            if (dma_normal_startIt(UART1TX_DMAREQ_CH, DMA_BURST_UNUSE, &cfg) != TXZ_SUCCESS)
            {
                uart_io_error();
            }
        }
        /*------------------------------*/
        /*  Enable Transmit             */
        /*------------------------------*/
        /*--- UARTxTRANS ---*/
        /* Bitband Access.              */
        TSB_UART1_TRANS_TXE = 1;
        /*------------------------------*/
        /*  Wait to transmit            */
        /*------------------------------*/
        while(instance.info.tx.state == UART_IO_DMA_SEND_RUN)
        {
        }
        /*------------------------------*/
        /*  Check Error                 */
        /*------------------------------*/
        if (instance.info.tx.state == UART_IO_DMA_SEND_FAILURE)
        {
            uart_io_error();
        }
        /* Clear status. */
        instance.info.tx.state = UART_IO_DMA_SEND_IDLE;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA Receive start
  * @param  num
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void UART_DMA_Receive(uint32_t num)
{
    if (instance.info.rx.state == UART_IO_DMA_RECV_IDLE)
    {
        /*------------------------------*/
        /*  Receive                     */
        /*------------------------------*/
        instance.info.rx.state   = UART_IO_DMA_RECV_RUN;
        
        /*--- UARTxTRANS ---*/
        TSB_UART1_TRANS_RXE = 0;
        {
            dma_normal_cfg_t cfg;
            
            cfg.source = (uint32_t)&TSB_UART1->DR;
            cfg.destination  = (uint32_t)&rx_data[0];
            cfg.control = (uint32_t)(DMA_DST_INC_1BYTE  | DMA_DST_SIZE_1BYTE |
                                     DMA_SRC_INC_NON    | DMA_SRC_SIZE_1BYTE |
                                     DMA_ARBITRATION_1  | DMA_EX_TRANS_NUM(num) |
                                     DMA_BURST_UNUSE    | DMA_CYCLE_MODE_BASE);
           if (dma_normal_startIt(UART1RX_DMAREQ_CH, DMA_BURST_UNUSE, &cfg) != TXZ_SUCCESS)
           {
		 	   uart_io_error();
           }
        }
        /*--- UARTxTRANS ---*/
        TSB_UART1_TRANS_RXE = 1;
        /*------------------------------*/
        /*  Wait to receive             */
        /*------------------------------*/
        while(instance.info.rx.state == UART_IO_DMA_RECV_RUN)
        {
            /* no processing */
        }
        /*------------------------------*/
        /*  Check Error                 */
        /*------------------------------*/
        if (instance.info.rx.state == UART_IO_DMA_RECV_FAILURE)
        {
            uart_io_error();
        }
        /* Clear status. */
        instance.info.rx.state = UART_IO_DMA_RECV_IDLE;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Transmit handler.
  * @param  result  :Result.
  * @return -
  * @retval -
  * @note   Called by uart driver.
  */
/*--------------------------------------------------*/
static void transmit_handler(TXZ_Result result)
{
    if (instance.info.tx.state != UART_IO_DMA_SEND_IDLE)
    {
        /*------------------------------*/
        /*  Result Check                */
        /*------------------------------*/
        if (result == TXZ_SUCCESS)
        {
            instance.info.tx.state = UART_IO_DMA_SEND_SUCCESS;
        }
        else
        {
            instance.info.tx.state = UART_IO_DMA_SEND_FAILURE;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Receive handler.
  * @param  result  :Result.
  * @return -
  * @retval -
  * @note   Called by uart driver.
  */
/*--------------------------------------------------*/
static void receive_handler(TXZ_Result result)
{
    if (instance.info.rx.state == UART_IO_DMA_RECV_RUN)
    {
        /*------------------------------*/
        /*  Result Check                */
        /*------------------------------*/
        if (result == TXZ_SUCCESS)
        {
            instance.info.rx.state = UART_IO_DMA_RECV_SUCCESS;
        }
        else
        {
            instance.info.rx.state = UART_IO_DMA_RECV_FAILURE;
        }
    }
}
/*--------------------------------------------------*/
/**
  * @brief     Start DMA normal mode.
  * @param     ch      :Channel.
  * @param     burst   :Use burst. @ref DMA_Burst.
  * @param     p_cfg   :config.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result dma_normal_startIt(uint32_t ch, uint32_t burst, dma_normal_cfg_t *p_cfg)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Channel Control Data Making */
    /*------------------------------*/
    /* Use primary. */
    {
        uint32_t *p_primary = (uint32_t *)((uint32_t)primary + (uint32_t)(ch * sizeof(control_data_t)));
        
        /*--- source ---*/
        {
            uint32_t inc  = 0;
            uint32_t size = 0;
            uint32_t num  = 0;
            
            /* increment */
            switch(p_cfg->control & CH_CFG_MASK_SRC_INC)
            {
            case DMA_SRC_INC_1BYTE:
                inc  = 1;
                break;
            case DMA_SRC_INC_2BYTE:
                inc  = 2;
                break;
            case DMA_SRC_INC_4BYTE:
                inc  = 4;
                break;
            default:
                inc  = 0;
                break;
            }
            /* size */
            switch(p_cfg->control & CH_CFG_MASK_SRC_SIZE)
            {
            case DMA_SRC_SIZE_1BYTE:
                size  = 1;
                break;
            case DMA_SRC_SIZE_2BYTE:
                size  = 2;
                break;
            case DMA_SRC_SIZE_4BYTE:
                size  = 4;
                break;
            default:
                size  = 0;
                break;
            }
            /* num */
            num = convert_transfer_num_to_if((p_cfg->control & CH_CFG_MASK_TRANS_NUM));
            /* Set value. */
            *p_primary = get_end_address(p_cfg->source, inc, size, num);
        }
        /*--- destination ---*/
        {
            uint32_t inc  = 0;
            uint32_t size = 0;
            uint32_t num  = 0;

            /* increment */
            switch(p_cfg->control & CH_CFG_MASK_DST_INC)
            {
            case DMA_DST_INC_1BYTE:
                inc  = 1;
                break;
            case DMA_DST_INC_2BYTE:
                inc  = 2;
                break;
            case DMA_DST_INC_4BYTE:
                inc  = 4;
                break;
            default:
                inc  = 0;
                break;
            }
            /* size */
            switch(p_cfg->control & CH_CFG_MASK_DST_SIZE)
            {
            case DMA_DST_SIZE_1BYTE:
                size  = 1;
                break;
            case DMA_DST_SIZE_2BYTE:
                size  = 2;
                break;
            case DMA_DST_SIZE_4BYTE:
                size  = 4;
                break;
            default:
                size  = 0;
                break;
            }
            /* num */
            num = convert_transfer_num_to_if((p_cfg->control & CH_CFG_MASK_TRANS_NUM));
            /* Set value. */
            *(p_primary + 1) = get_end_address(p_cfg->destination, inc, size, num);
        }
        /*---- control ----*/
        *(p_primary + 2) = p_cfg->control;
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        uint32_t regCh = convert_ch_to_reg(ch);

        /*--- DMAxChnlPriAltClr ---*/
        TSB_DMAA->CHNLPRIALTCLR  = regCh;
        if (burst == DMA_BURST_USE)
        {
            /*--- DMAxChnlUseburstSet ---*/
            /* Burst transfer. */
            TSB_DMAA->CHNLUSEBURSTSET = regCh;
        }
        else
        {
            /*--- DMAxChnlUseburstClr ---*/
            /* Single transfer. */
            TSB_DMAA->CHNLUSEBURSTCLR = regCh;
        }
        /*--- DMAxChnlReqMaskClr  ---*/
        TSB_DMAA->CHNLREQMASKCLR = regCh;
    }

    return (result);
}
/**
 *  @}
 */ /* End of group UART_IO_DMA_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_IO_DMA_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  DMA_Configure
  * @param  -
  * @return -
  * @retval -
  * @attention   Singleton.
  */
/*--------------------------------------------------*/
void DMA_Config(void)
{
    /*------------------------------*/
    /*  Initialize Variable         */
    /*------------------------------*/
    instance.info.tx.state     = UART_IO_DMA_SEND_IDLE;
    instance.info.rx.state     = UART_IO_DMA_RECV_IDLE;
    /*------------------------------*/
    /*  Port CG Fsys Enable         */
    /*------------------------------*/
    TSB_CG_FSYSMENA_IPMENA16 = 1;
    
    TSB_CG_FSYSMENB_IPMENB17 = 1;     /* DMAC  */
    /*--- DMAxCtrlBasePtr    ---*/
    TSB_DMAA->CTRLBASEPTR     = (CTRLBASEPTR_MASK & (uint32_t)&primary[0]);
    /*--- DMAxChnlReqMaskSet ---*/
    TSB_DMAA->CHNLREQMASKSET  = CHNLREQMASKSET_ALL_MASK;
    /*--- DMAxCfg            ---*/
    TSB_DMAA->CFG             = CFG_MASTER_ENABLE;
    
    uint32_t regCh = convert_ch_to_reg(UART1RX_DMAREQ_CH);
    TSB_DMAA->CHNLPRIORITYCLR = regCh;
    /*>>> To stop occurrence of ABnormal END interrupt. */
    /*--- DMAxChnlReqMaskSet  ---*/
    TSB_DMAA->CHNLREQMASKSET = regCh;
    /*--- DMAxChnlEnableSet   ---*/
    TSB_DMAA->CHNLENABLESET  = regCh;
    
    regCh = convert_ch_to_reg(UART1TX_DMAREQ_CH);
    TSB_DMAA->CHNLPRIORITYCLR = regCh;
    /*>>> To stop occurrence of ABnormal END interrupt. */
    /*--- DMAxChnlReqMaskSet  ---*/
    TSB_DMAA->CHNLREQMASKSET = regCh;
    /*--- DMAxChnlEnableSet   ---*/
    TSB_DMAA->CHNLENABLESET  = regCh;
    
    NVIC_EnableIRQ(INTDMAATC_IRQn);
    NVIC_EnableIRQ(INTDMAAERR_IRQn);
    
    return;
}

/*--------------------------------------------------*/
/**
  * @brief     IRQ handler.
  * @param     ch      :Channel.
  * @retval    -
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
void dma_irq_handler(uint32_t ch)
{
    /*------------------------------*/
    /* Interrupt Check              */
    /*------------------------------*/
    /* Check the interrupt flag. */
    if (ch == UART1TX_DMAREQ_CH)
    {
        uint32_t regCh = convert_ch_to_reg(UART1TX_DMAREQ_CH);
        /*>>> To stop occurrence of ABnormal END interrupt. */
        /*--- DMAxChnlReqMaskSet  ---*/
        TSB_DMAA->CHNLREQMASKSET = regCh;
        /*--- DMAxChnlEnableSet   ---*/
        TSB_DMAA->CHNLENABLESET  = regCh;
        /*<<< To stop occurrence of ABnormal END interrupt. */
        /* Call handler. */
        transmit_handler(TXZ_SUCCESS);
    }
    else if(ch == UART1RX_DMAREQ_CH)
    {
        uint32_t regCh = convert_ch_to_reg(UART1RX_DMAREQ_CH);
        /*>>> To stop occurrence of ABnormal END interrupt. */
        /*--- DMAxChnlReqMaskSet  ---*/
        TSB_DMAA->CHNLREQMASKSET = regCh;
        /*--- DMAxChnlEnableSet   ---*/
        TSB_DMAA->CHNLENABLESET  = regCh;
        /*<<< To stop occurrence of ABnormal END interrupt. */
        /* Call handler. */
        receive_handler(TXZ_SUCCESS);
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  INTDMAATCINTDMACTC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   110: DMA end of transfer
  */
/*--------------------------------------------------*/
void INTDMAATC_IRQHandler(void)
{
    uint32_t i;
    uint32_t flag;
    uint32_t mask;

    /* Read the interrupt flag(IMNFLG3).  */
    flag = TSB_IMN->FLG3;
    for (i=0; i<DMA_CH_NUM; i++)
    {
        /* Check the DMA channel.  */
        mask = (uint32_t)(1 << i);
        if ((flag & mask) == mask)
        {
            /* Clear the interrupt flag. */
            *(__IO uint8_t *)(&TSB_IB->IMC000 + (i)) = 0xC0;
            /* Call Handler. */
            dma_irq_handler(i);
        }
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  INTDMAAERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   111: DMA transfer error
  */
/*--------------------------------------------------*/
void INTDMAAERR_IRQHandler(void)
{
     transmit_handler(TXZ_ERROR);
     receive_handler(TXZ_ERROR);
}
/**
 *  @}
 */ /* End of group UART_IO_DMA_Exported_functions */

/**
 *  @}
 */ /* End of group UART_IO_DMA */

/**
 *  @} 
 */ /* End of group Example */

#endif /* defined(__UART_IO_DMA_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */
