/** 
 *******************************************************************************
 * @file    uart_io_dma.h
 * @brief   External reference declaration for UART input/output
 * @version V1.0.0.0
 * $Date:: 2018-10-26 12:04:12 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __UART_IO_DMA_H
#define __UART_IO_DMA_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup UART_IO_DMA
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_IO_DMA_Exported_define
 *  @{
 */

#define UART_IO_DMA_SEND_MAX       ((uint32_t)32)   /*!< Size of send max. (byte)    */
#define UART_IO_DMA_RECEIVE_MAX    ((uint32_t)32)   /*!< Size of receive max. (byte) */
/** 
 *  @}
 */ /* End of group UART_IO_DMA_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_IO_DMA_Exported_define
 *  @{
 */

/** @defgroup UART_IO_DMA_Exported_typedef UART_IO_DMA Exported typedef
  * @{
  */
typedef enum {
    TXZ_SUCCESS = 0U,
    TXZ_ERROR = 1U
} TXZ_Result;

typedef enum {
    TXZ_BUSY = 0U,
    TXZ_DONE = 1U
} TXZ_WorkState;

typedef enum {
    TXZ_DISABLE = 0U,
    TXZ_ENABLE = 1U
} TXZ_FunctionalState;
/** 
 *  @addtogroup UART_IO_DMA_Exported_typedef
 *  @{
 */
/**
 *  @}
 */ /* End of group UART_IO_DMA_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_IO_DMA_Exported_functions
 *  @{
 */
void DMA_Config(void);
void UART_DMA_Trans(uint32_t num);
void UART_DMA_Receive(uint32_t num);
/** 
 *  @}
 */ /* End of group UART_IO_DMA_Exported_functions */

/**
 *  @}
 */ /* End of group UART_IO_DMA */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __UART_IO_DMA_H */

